@@ -28,11 +28,6 @@
 #define CSRI_TRACE
 #define CSRI_PROFILESIZES
 #define CSRI_DEBUG
-#elif (MALLOC_PACKAGE == 3)
-#define USE_SMALLOC
-#elif (MALLOC_PACKAGE == 4)
-#define USE_SMALLOC
-#define SLOW_STATISTICS
 #endif
 
 #endif				/* _MYMALLOC_H */
