@@ -20,7 +20,7 @@ typedef struct {
   BYTE reverse_wanted;		/* true to reverse (little_endian) false to not */
 } SHS_INFO;
 
-void shsInit _((SHS_INFO * shsInfo));
-void shsUpdate _((SHS_INFO * shsInfo, BYTE * buffer, int count));
-void shsFinal _((SHS_INFO * shsInfo));
+void shsInit(SHS_INFO * shsInfo);
+void shsUpdate(SHS_INFO * shsInfo, const BYTE * buffer, int count);
+void shsFinal(SHS_INFO * shsInfo);
 #endif
