@@ -1,11 +1,7 @@
 #include "copyrite.h"
 
 #include "config.h"
-#ifdef I_STRING
 #include <string.h>
-#else
-#include <strings.h>
-#endif
 #include <ctype.h>
 #include "conf.h"
 #include "attrib.h"
@@ -15,80 +11,26 @@
 #ifdef MEM_CHECK
 #include "memcheck.h"
 #endif
-#include "oldattrb.h"
 #include "parse.h"
 #include "htab.h"
 #include "privtab.h"
 #include "mymalloc.h"
+#include "strtree.h"
+#include "flags.h"
+#include "mushdb.h"
+#include "lock.h"
+#include "log.h"
 #include "confmagic.h"
 
-
-struct boolatr *alloc_atr _((const char *name, char *s));
-
-ATTR *atr_match _((char const *string));
-int string_to_atrflag _((dbref player, char const *p));
-void atr_new_add _((dbref thing, char const *atr, char const *s,
-		    dbref player, int flags));
-int atr_add _((dbref thing, char const *atr, char const *s,
-	       dbref player, int flags));
-int atr_clr _((dbref thing, char const *atr, dbref player));
-ATTR *atr_get _((dbref thing, char const *atr));
-ATTR *atr_get_noparent _((dbref thing, char const *atr));
-ATTR *atr_complete_match _((dbref player, char const *atr, dbref privs));
-void atr_free _((dbref thing));
-void atr_cpy _((dbref dest, dbref source));
-
-char const *const convert_atr _((int oldatr));
-
-int atr_comm_match
-_(
-  (dbref thing, dbref player, int type, int end, char const *str,
-   int just_match));
-
-int do_set_atr _((dbref thing, char const *atr, char const *s, dbref player,
-		  int flags));
-void do_atrlock _((dbref player, char const *arg1, char const *arg2));
-void do_atrchown _((dbref player, char const *arg1, char const *arg2));
-
-static int good_atr_name _((char const *s));
-static char const *standard_atr_name _((char const *atr));
-
-static char *atr_hash_lookup _((char const *s));
-static void atr_hash_insert _((char const *name));
-static void atr_hash_init _((void));
-static void atr_hash_free _((void));
-static int atr_hash_initialized = 0;
-
 /* A hash table that temporarily stores matching attributes from current
  * and parent objects when we're searching for commands on attributes
  */
-HASHTAB htab_temp_attrib;
+StrTree temp_attrib;
+StrTree atr_names;
 extern PRIV attr_privs[];
 
 /*======================================================================*/
 
-/* This should be with the lock stuff, not the attribute stuff.
- * Who was the idiot who put it here? */
-
-struct boolatr *
-alloc_atr(name, s)
-    const char *name;
-    char *s;
-{
-  struct boolatr *a;
-  const unsigned char *p;
-
-  a = (struct boolatr *) mush_malloc(sizeof(struct boolatr), "boolatr");
-  a->name = (char *) mush_malloc(strlen(name) + 1, "boolatr_name");
-  strcpy(a->name, name);
-  p = compress(s);
-  a->text = (unsigned char *) mush_malloc(u_strlen(p) + 1, "boolatr_text");
-  u_strcpy(a->text, p);
-  return a;
-}
-
-/*======================================================================*/
-
 #define ATTRS_PER_PAGE (200)
 
 typedef struct atrpage {
@@ -97,35 +39,33 @@ typedef struct atrpage {
 
 static ATTR *atr_free_list;
 
-/* for attrib hash stuff, in atr_tab.c */
-extern ATTR *aname_hash_lookup _((const char *name));
-
 /*======================================================================*/
 
-static int
+void
+init_atr_name_tree()
+{
+  st_init(&atr_names);
+}
+
+/* Lookup table for good_atr_name */
+extern char atr_name_table[UCHAR_MAX + 1];
+
+int
 good_atr_name(s)
     char const *s;
 {
-  const char *a;
+  const unsigned char *a;
+  int len = 0;
   if (!s || !*s)
     return 0;
-  for (a = s; *a; a++)
-    if (!(isascii(*a) && isprint(*a) && !isspace(*a) && (*a != '^'))) {
+  for (a = (const unsigned char *) s; *a; a++, len++)
+    if (!atr_name_table[*a])
       return 0;
-    }
-  return 1;
-}
-
-static char const *
-standard_atr_name(atr)
-    char const *atr;
-{
-  return NULL;
+  return len <= ATTRIBUTE_NAME_LIMIT;
 }
 
 ATTR *
-atr_match(string)
-    char const *string;
+atr_match(const char *string)
 {
   return aname_hash_lookup(string);
 }
@@ -139,15 +79,18 @@ string_to_atrflag(player, p)
   f = string_to_privs(attr_privs, p, 0);
   if (!f)
     return -1;
-  if (!Hasprivs(player) && (f & (AF_MDARK | AF_WIZARD)))
+  if (!Hasprivs(player) && (f & AF_MDARK))
+    return -1;
+  if (!See_All(player) && (f & AF_WIZARD))
     return -1;
   return f;
 }
 
 /* This is a stripped down version of atr_add, without duplicate checking,
- * permissions checking, or auto-ODARKing.  If anyone uses this outside
- * of database load or atr_cpy (below), I will personally string them up
- * by their toes.  - Alex */
+ * permissions checking, attribute count checking, or auto-ODARKing.  
+ * If anyone uses this outside of database load or atr_cpy (below), 
+ * I will personally string them up  by their toes.  - Alex 
+ */
 void
 atr_new_add(thing, atr, s, player, flags)
     dbref thing;
@@ -156,8 +99,9 @@ atr_new_add(thing, atr, s, player, flags)
     dbref player;
     int flags;
 {
-  ATTR *ptr, *std;
-  char *name;
+  ATTR *ptr;
+  char const *name;
+  ATTR **ins;
 
 #ifndef EMPTY_ATTRS
   if (!*s)
@@ -179,43 +123,43 @@ atr_new_add(thing, atr, s, player, flags)
   ptr = atr_free_list;
   atr_free_list = AL_NEXT(ptr);
 
-  /* initialize atr */
-  std = atr_match(atr);
-  if (std && !strcmp(AL_NAME(std), atr)) {
-    AL_NAME(ptr) = AL_NAME(std);
-    AL_FLAGS(ptr) = flags | AF_STATIC;
-  } else {
-    name = (char *) mush_malloc(strlen(atr) + 1, "ATTR name");
-    if (!name) {
-      AL_NEXT(ptr) = atr_free_list;
-      atr_free_list = ptr;
-      return;
-    }
-    strcpy(name, atr);
-    AL_NAME(ptr) = name;
-    AL_FLAGS(ptr) = 0;
-    if (flags != NOTHING)
-      AL_FLAGS(ptr) |= flags;
+  /* Don't fail on a bad name, but do log it */
+  if (!good_atr_name(atr))
+    do_rawlog(LT_ERR, T("Bad attribute name %s on object %s"), atr,
+	      unparse_dbref(thing));
+
+  /* put the name in the string table */
+  name = st_insert(atr, &atr_names);
+  if (!name) {
+    AL_NEXT(ptr) = atr_free_list;
+    atr_free_list = ptr;
+    return;
   }
+  /* initialize atr */
+  AL_NAME(ptr) = name;
+  AL_FLAGS(ptr) = (flags != NOTHING) ? flags : 0;
+  AL_FLAGS(ptr) &= ~AF_COMMAND & ~AF_LISTEN;
   AL_CREATOR(ptr) = player;
+  AL_STR(ptr) = NULL;
 
   /* link it in */
-  AL_NEXT(ptr) = List(thing);
-  List(thing) = ptr;
-
-  AL_FLAGS(ptr) &= ~AF_COMMAND & ~AF_LISTEN;
+  ins = &List(thing);
+  while (*ins && strcoll(AL_NAME(*ins), AL_NAME(ptr)) < 0)
+    ins = &AL_NEXT(*ins);
+  AL_NEXT(ptr) = *ins;
+  *ins = ptr;
 
   /* replace string with new string */
   if (!s || !*s) {
     AL_STR(ptr) = (unsigned char *) "";
   } else {
-    AL_STR(ptr) = (unsigned char *) u_strdup(compress((char *) s));
+    AL_STR(ptr) = compress(s);
     if (!AL_STR(ptr)) {
       AL_STR(ptr) = (unsigned char *) "";
       return;
     }
 #ifdef MEM_CHECK
-    add_check("ATTR value");
+    add_check(T("ATTR value"));
 #endif
   }
   if (*s == '$')
@@ -224,6 +168,10 @@ atr_new_add(thing, atr, s, player, flags)
     AL_FLAGS(ptr) |= AF_LISTEN;
 }
 
+#define AE_BADNAME -3
+#define AE_SAFE -2
+#define AE_ERROR -1
+
 int
 atr_add(thing, atr, s, player, flags)
     dbref thing;
@@ -233,7 +181,9 @@ atr_add(thing, atr, s, player, flags)
     int flags;
 {
   ATTR *ptr, *std;
-  char *name;
+  char const *name;
+  ATTR **ins;
+  int comp = 0;
 
   if (!s)
     return atr_clr(thing, atr, player);
@@ -244,16 +194,20 @@ atr_add(thing, atr, s, player, flags)
 #endif
 
   if (!good_atr_name(atr))
-    return -1;
+    return AE_BADNAME;
 
   /* walk the list, looking for a preexisting value */
   ptr = List(thing);
-  while (ptr && strcmp(atr, AL_NAME(ptr)))
+  while (ptr && (comp = strcoll(atr, AL_NAME(ptr))) > 0)
     ptr = AL_NEXT(ptr);
+  if (comp)
+    ptr = NULL;
 
   /* check for permission to modify existing atr */
-  if (ptr && !Can_Write_Attr(Owner(player), thing, ptr))
-    return -1;
+  if (ptr && (AL_FLAGS(ptr) & AF_SAFE))
+    return AE_SAFE;
+  if (ptr && !Can_Write_Attr(player, thing, ptr))
+    return AE_ERROR;
 
   /* make a new atr, if needed */
   if (!ptr) {
@@ -272,20 +226,19 @@ atr_add(thing, atr, s, player, flags)
     ptr = atr_free_list;
     atr_free_list = AL_NEXT(ptr);
 
+    /* put the name in the string table */
+    name = st_insert(atr, &atr_names);
+    if (!name) {
+      AL_NEXT(ptr) = atr_free_list;
+      atr_free_list = ptr;
+      return AE_ERROR;
+    }
     /* initialize atr */
+    AL_NAME(ptr) = name;
     std = atr_match(atr);
     if (std && !strcmp(AL_NAME(std), atr)) {
-      AL_NAME(ptr) = AL_NAME(std);
-      AL_FLAGS(ptr) = AL_FLAGS(std) | AF_STATIC;
+      AL_FLAGS(ptr) = AL_FLAGS(std);
     } else {
-      name = (char *) mush_malloc(strlen(atr) + 1, "ATTR name");
-      if (!name) {
-	AL_NEXT(ptr) = atr_free_list;
-	atr_free_list = ptr;
-	return -1;
-      }
-      strcpy(name, atr);
-      AL_NAME(ptr) = name;
       AL_FLAGS(ptr) = 0;
       if (flags != NOTHING)
 	AL_FLAGS(ptr) |= flags;
@@ -294,20 +247,30 @@ atr_add(thing, atr, s, player, flags)
     AL_STR(ptr) = NULL;
 
     /* check for permission to create atr */
-    if (!Can_Write_Attr(Owner(player), thing, ptr)) {
+    if (!Can_Write_Attr(player, thing, ptr) ||
+	(AttrCount(thing) >= MAX_ATTRCOUNT)) {
+      st_delete(AL_NAME(ptr), &atr_names);
       AL_NEXT(ptr) = atr_free_list;
       atr_free_list = ptr;
-      return -1;
+      if (AttrCount(thing) >= MAX_ATTRCOUNT)
+	do_log(LT_ERR, player, thing,
+	       T("Attempt by %s(%d) to create too many attributes on %s(%d)"),
+	       Name(player), player, Name(thing), thing);
+      return AE_ERROR;
     }
     /* link it in */
-    AL_NEXT(ptr) = List(thing);
-    List(thing) = ptr;
+    ins = &List(thing);
+    while (*ins && strcoll(AL_NAME(*ins), AL_NAME(ptr)) < 0)
+      ins = &AL_NEXT(*ins);
+    AL_NEXT(ptr) = *ins;
+    *ins = ptr;
+    AttrCount(thing)++;
   }
   /* update modification time here, because from now on,
    * we modify even if we fail */
 #ifdef CREATION_TIMES
-  if (!IsPlayer(thing))
-    ModTime(thing) = time((time_t *) 0);
+  if (!IsPlayer(thing) && !(AL_FLAGS(ptr) & AF_NODUMP))
+    ModTime(thing) = mudtime;
 #endif
 
   /* change owner */
@@ -317,17 +280,17 @@ atr_add(thing, atr, s, player, flags)
 
   /* replace string with new string */
   if (AL_STR(ptr) && *AL_STR(ptr))
-    mush_free((Malloc_t) AL_STR(ptr), "ATTR value");
+    mush_free((Malloc_t) AL_STR(ptr), T("ATTR value"));
   if (!s || !*s) {
     AL_STR(ptr) = (unsigned char *) "";
   } else {
-    AL_STR(ptr) = (unsigned char *) u_strdup(compress((char *) s));
+    AL_STR(ptr) = compress(s);
     if (!AL_STR(ptr)) {
       AL_STR(ptr) = (unsigned char *) "";
-      return -1;
+      return AE_ERROR;
     }
 #ifdef MEM_CHECK
-    add_check("ATTR value");
+    add_check(T("ATTR value"));
 #endif
   }
   if (*s == '$')
@@ -345,6 +308,7 @@ atr_clr(thing, atr, player)
     dbref player;
 {
   ATTR *ptr, **prev;
+  int comp = 0;
 
 #ifdef EMPTY_ATTRS
 #ifndef DELETE_ATTRS
@@ -352,37 +316,36 @@ atr_clr(thing, atr, player)
 #endif
 #endif
 
-  if (!good_atr_name(atr))
-    return -1;
-
   prev = &List(thing);
   ptr = *prev;
-  while (ptr && strcmp(atr, AL_NAME(ptr))) {
+  while (ptr && (comp = strcoll(atr, AL_NAME(ptr))) > 0) {
     prev = &AL_NEXT(ptr);
     ptr = *prev;
   }
+  if (comp)
+    ptr = NULL;
 
   if (!ptr)
     return 0;
 
-  if (!Can_Write_Attr(Owner(player), thing, ptr))
+  if (!Can_Write_Attr(player, thing, ptr))
     return -1;
 
 #ifdef CREATION_TIMES
-  if (!IsPlayer(thing))
-    ModTime(thing) = time((time_t *) 0);
+  if (!IsPlayer(thing) && !(AL_FLAGS(ptr) & AF_NODUMP))
+    ModTime(thing) = mudtime;
 #endif
 
   *prev = AL_NEXT(ptr);
 
   if (AL_STR(ptr) && *AL_STR(ptr))
-    mush_free((Malloc_t) AL_STR(ptr), "ATTR value");
-  if (!(AL_FLAGS(ptr) & AF_STATIC))
-    mush_free((Malloc_t) AL_NAME(ptr), "ATTR name");
+    mush_free((Malloc_t) AL_STR(ptr), T("ATTR value"));
+  st_delete(AL_NAME(ptr), &atr_names);
 
   AL_NEXT(ptr) = atr_free_list;
   AL_FLAGS(ptr) = 0;
   atr_free_list = ptr;
+  AttrCount(thing)--;
   return 1;
 }
 
@@ -394,6 +357,7 @@ atr_get(thing, atr)
   ATTR *ptr;
   int parent_depth;
   dbref temp;
+  int comp;
 
   if (thing == NOTHING || !good_atr_name(atr))
     return NULL;
@@ -402,8 +366,11 @@ atr_get(thing, atr)
   for (parent_depth = 0, temp = thing;
        parent_depth < MAX_PARENTS && temp != NOTHING;
        parent_depth++, temp = Parent(temp))
-    for (ptr = List(temp); ptr; ptr = AL_NEXT(ptr))
-      if (!strcmp(AL_NAME(ptr), atr)
+    for (ptr = List(temp); ptr; ptr = AL_NEXT(ptr)) {
+      comp = strcoll(atr, AL_NAME(ptr));
+      if (comp < 0)
+	break;
+      if (comp == 0
 #ifndef VISIBLE_EMPTY_ATTRS
 	  && *AL_STR(ptr)
 #endif
@@ -413,6 +380,7 @@ atr_get(thing, atr)
 	else
 	  break;
       }
+    }
   ptr = atr_match(atr);
   if (!ptr || !strcmp(atr, AL_NAME(ptr)))
     return NULL;
@@ -422,8 +390,11 @@ atr_get(thing, atr)
   for (parent_depth = 0, temp = thing;
        parent_depth < MAX_PARENTS && temp != NOTHING;
        parent_depth++, temp = Parent(temp))
-    for (ptr = List(temp); ptr; ptr = AL_NEXT(ptr))
-      if (!strcmp(AL_NAME(ptr), atr)
+    for (ptr = List(temp); ptr; ptr = AL_NEXT(ptr)) {
+      comp = strcoll(atr, AL_NAME(ptr));
+      if (comp < 0)
+	break;
+      if (comp == 0
 #ifndef VISIBLE_EMPTY_ATTRS
 	  && *AL_STR(ptr)
 #endif
@@ -433,6 +404,7 @@ atr_get(thing, atr)
 	else
 	  break;
       }
+    }
   return NULL;
 }
 
@@ -442,18 +414,23 @@ atr_get_noparent(thing, atr)
     char const *atr;
 {
   ATTR *ptr;
+  int comp;
 
   if (thing == NOTHING || !good_atr_name(atr))
     return NULL;
 
   /* try real name */
-  for (ptr = List(thing); ptr; ptr = AL_NEXT(ptr))
-    if (!strcmp(AL_NAME(ptr), atr)
+  for (ptr = List(thing); ptr; ptr = AL_NEXT(ptr)) {
+    comp = strcoll(atr, AL_NAME(ptr));
+    if (comp < 0)
+      break;
+    if (comp == 0
 #ifndef VISIBLE_EMPTY_ATTRS
 	&& *AL_STR(ptr)
 #endif
       )
       return ptr;
+  }
 
   ptr = atr_match(atr);
   if (!ptr || !strcmp(atr, AL_NAME(ptr)))
@@ -461,24 +438,25 @@ atr_get_noparent(thing, atr)
   atr = AL_NAME(ptr);
 
   /* try alias */
-  for (ptr = List(thing); ptr; ptr = AL_NEXT(ptr))
-    if (!strcmp(AL_NAME(ptr), atr)
+  for (ptr = List(thing); ptr; ptr = AL_NEXT(ptr)) {
+    comp = strcoll(atr, AL_NAME(ptr));
+    if (comp < 0)
+      break;
+    if (comp == 0
 #ifndef VISIBLE_EMPTY_ATTRS
 	&& *AL_STR(ptr)
 #endif
       )
       return ptr;
+  }
 
   return NULL;
 }
 
+
 int
-atr_iter_get(player, thing, name, func, args)
-    dbref player;
-    dbref thing;
-    char const *name;
-    int (*func) ();
-    void *args;
+atr_iter_get(dbref player, dbref thing, const char *name, aig_func func,
+	     void *args)
 {
   ATTR *ptr, *next;
   int result;
@@ -510,6 +488,8 @@ atr_iter_get(player, thing, name, func, args)
   return result;
 }
 
+#ifdef NEVER
+/* Old code that we don't use any more */
 ATTR *
 atr_complete_match(player, atr, privs)
     dbref player;
@@ -530,6 +510,7 @@ atr_complete_match(player, atr, privs)
 
   return NULL;
 }
+#endif
 
 void
 atr_free(thing)
@@ -542,16 +523,15 @@ atr_free(thing)
 
 #ifdef CREATION_TIMES
   if (!IsPlayer(thing))
-    ModTime(thing) = time((time_t *) 0);
+    ModTime(thing) = mudtime;
 #endif
 
   while ((ptr = List(thing))) {
     List(thing) = AL_NEXT(ptr);
 
     if (AL_STR(ptr) && *AL_STR(ptr))
-      mush_free((Malloc_t) AL_STR(ptr), "ATTR value");
-    if (!(AL_FLAGS(ptr) & AF_STATIC))
-      mush_free((Malloc_t) AL_NAME(ptr), "ATTR name");
+      mush_free((Malloc_t) AL_STR(ptr), T("ATTR value"));
+    st_delete(AL_NAME(ptr), &atr_names);
 
     AL_NEXT(ptr) = atr_free_list;
     atr_free_list = ptr;
@@ -571,175 +551,39 @@ atr_cpy(dest, source)
 #ifndef VISIBLE_EMPTY_ATTRS
 	&& *AL_STR(ptr)
 #endif
-      )
+	&& (AttrCount(dest) < MAX_ATTRCOUNT)) {
       atr_new_add(dest, AL_NAME(ptr), uncompress(AL_STR(ptr)),
 		  AL_CREATOR(ptr), AL_FLAGS(ptr));
-}
-
-/*======================================================================*/
-
-char const *const
-convert_atr(oldatr)
-    int oldatr;
-{
-  static char result[3];
-
-  switch (oldatr) {
-  case A_OSUCC:
-    return "OSUCCESS";
-  case A_OFAIL:
-    return "OFAILURE";
-  case A_FAIL:
-    return "FAILURE";
-  case A_SUCC:
-    return "SUCCESS";
-  case A_PASS:
-    return "XYXXY";
-  case A_DESC:
-    return "DESCRIBE";
-  case A_SEX:
-    return "SEX";
-  case A_ODROP:
-    return "ODROP";
-  case A_DROP:
-    return "DROP";
-  case A_OKILL:
-    return "OKILL";
-  case A_KILL:
-    return "KILL";
-  case A_ASUCC:
-    return "ASUCCESS";
-  case A_AFAIL:
-    return "AFAILURE";
-  case A_ADROP:
-    return "ADROP";
-  case A_AKILL:
-    return "AKILL";
-  case A_USE:
-    return "DOES";
-  case A_CHARGES:
-    return "CHARGES";
-  case A_RUNOUT:
-    return "RUNOUT";
-  case A_STARTUP:
-    return "STARTUP";
-  case A_ACLONE:
-    return "ACLONE";
-  case A_APAY:
-    return "APAYMENT";
-  case A_OPAY:
-    return "OPAYMENT";
-  case A_PAY:
-    return "PAYMENT";
-  case A_COST:
-    return "COST";
-  case A_RAND:
-    return "RAND";
-  case A_LISTEN:
-    return "LISTEN";
-  case A_AAHEAR:
-    return "AAHEAR";
-  case A_AMHEAR:
-    return "AMHEAR";
-  case A_AHEAR:
-    return "AHEAR";
-  case A_LAST:
-    return "LAST";
-  case A_QUEUE:
-    return "QUEUE";
-  case A_IDESC:
-    return "IDESCRIBE";
-  case A_ENTER:
-    return "ENTER";
-  case A_OXENTER:
-    return "OXENTER";
-  case A_AENTER:
-    return "AENTER";
-  case A_ADESC:
-    return "ADESCRIBE";
-  case A_ODESC:
-    return "ODESCRIBE";
-  case A_RQUOTA:
-    return "RQUOTA";
-  case A_ACONNECT:
-    return "ACONNECT";
-  case A_ADISCONNECT:
-    return "ADISCONNECT";
-  case A_LEAVE:
-    return "LEAVE";
-  case A_ALEAVE:
-    return "ALEAVE";
-  case A_OLEAVE:
-    return "OLEAVE";
-  case A_OENTER:
-    return "OENTER";
-  case A_OXLEAVE:
-    return "OXLEAVE";
-  default:
-    if (oldatr >= 100 && oldatr < 178) {
-      result[0] = 'V' + (oldatr - 100) / 26;
-      result[1] = 'A' + (oldatr - 100) % 26;
-      return result;
-    } else {
-      fprintf(stderr,
-	      "ERROR: Invalid attribute number in convert_atr. aborting.\n");
-      fflush(stderr);
-      abort();
+      AttrCount(dest)++;
     }
-  }
-  /*NOTREACHED */
-  return "";
-}
-
-/*======================================================================*/
-
-static char *
-atr_hash_lookup(s)
-    char const *s;
-{
-  return (char *) hashfind(s, &htab_temp_attrib);
-}
-
-static void
-atr_hash_insert(name)
-    char const *name;
-{
-  hashadd(name, (void *) name, &htab_temp_attrib);
-}
-
-static void
-atr_hash_free()
-{
-  hashflush(&htab_temp_attrib, 32);
-}
-
-static void
-atr_hash_init()
-{
-  hashinit(&htab_temp_attrib, 32);
-  atr_hash_initialized = 1;
 }
 
 int
-atr_comm_match(thing, player, type, end, str, just_match)
+atr_comm_match(thing, player, type, end, str, just_match, atrname, abp)
     dbref thing;
     dbref player;
     int type;
     int end;
     char const *str;
     int just_match;
+    char *atrname;
+    char **abp;
 {
   int flag_mask;
   ATTR *ptr;
   int parent_depth;
   char tbuf1[BUFFER_LEN];
+  char tbuf2[BUFFER_LEN];
   char *s;
   int match;
   dbref parent;
 
   /* check for lots of easy ways out */
-  if ((type != '$' && type != '^') || !GoodObject(thing) || Halted(thing) ||
-      (NoCommand(thing) && type == '$') || !eval_lock(player, thing, Use_Lock))
+  if ((type != '$' && type != '^') || !GoodObject(thing) || Halted(thing)
+      || (type == '$'
+	  && (NoCommand(thing) || !eval_lock(player, thing, Command_Lock)))
+      || (type == '^' && !eval_lock(player, thing, Listen_Lock))
+      || !eval_lock(player, thing, Use_Lock))
     return 0;
 
   if (type == '$') {
@@ -747,12 +591,15 @@ atr_comm_match(thing, player, type, end, str, just_match)
     parent_depth = GoodObject(Parent(thing));
   } else {
     flag_mask = AF_LISTEN;
-    parent_depth = 0;
+    if (ThingInhearit(thing) || RoomInhearit(thing)) {
+      parent_depth = GoodObject(Parent(thing));
+    } else {
+      parent_depth = 0;
+    }
   }
 
   match = 0;
-  if (!atr_hash_initialized)
-    atr_hash_init();
+  st_init(&temp_attrib);
 
   for (ptr = List(thing); ptr; ptr = AL_NEXT(ptr)) {
 #ifndef VISIBLE_EMPTY_ATTRS
@@ -760,7 +607,7 @@ atr_comm_match(thing, player, type, end, str, just_match)
       continue;
 #endif
     if (parent_depth)
-      atr_hash_insert(AL_NAME(ptr));
+      st_insert(AL_NAME(ptr), &temp_attrib);
     if (AL_FLAGS(ptr) & AF_NOPROG || !(AL_FLAGS(ptr) & flag_mask))
       continue;
     strcpy(tbuf1, uncompress(AL_STR(ptr)));
@@ -771,10 +618,29 @@ atr_comm_match(thing, player, type, end, str, just_match)
     if (!s)
       continue;
     *s++ = '\0';
+
+    if (AL_FLAGS(ptr) & AF_REGEXP) {
+      /* Turn \: into : */
+      char *from, *to;
+      for (from = tbuf1, to = tbuf2; *from; from++, to++) {
+	if (*from == '\\' && *(from + 1) == ':')
+	  from++;
+	*to = *from;
+      }
+      *to = '\0';
+    } else
+      strcpy(tbuf2, tbuf1);
+
     if ((AL_FLAGS(ptr) & AF_REGEXP) ?
-	regexp_match_case(tbuf1 + 1, str, (AL_FLAGS(ptr) & AF_CASE)) :
-	wild_match_case(tbuf1 + 1, str, (AL_FLAGS(ptr) & AF_CASE))) {
+	regexp_match_case(tbuf2 + 1, str, (AL_FLAGS(ptr) & AF_CASE)) :
+	wild_match_case(tbuf2 + 1, str, (AL_FLAGS(ptr) & AF_CASE))) {
       match++;
+      if (atrname && abp) {
+	safe_chr(' ', atrname, abp);
+	safe_dbref(thing, atrname, abp);
+	safe_chr('/', atrname, abp);
+	safe_str(AL_NAME(ptr), atrname, abp);
+      }
       if (!just_match)
 	parse_que(thing, s, player);
     }
@@ -785,18 +651,17 @@ atr_comm_match(thing, player, type, end, str, just_match)
 
   for (parent_depth = MAX_PARENTS, parent = Parent(thing);
        parent_depth-- && parent != NOTHING; parent = Parent(parent)) {
-    if (!eval_lock(player, parent, Use_Lock))
-      break;
     for (ptr = List(parent); ptr; ptr = AL_NEXT(ptr)) {
 #ifndef VISIBLE_EMPTY_ATTRS
       if (!*AL_STR(ptr))
 	continue;
 #endif
-      if (atr_hash_lookup(AL_NAME(ptr)))
+      if (st_find(AL_NAME(ptr), &temp_attrib))
 	continue;
       if (Parent(parent) != NOTHING)
-	atr_hash_insert(AL_NAME(ptr));
-      if (AL_FLAGS(ptr) & AF_NOPROG || !(AL_FLAGS(ptr) & flag_mask))
+	st_insert(AL_NAME(ptr), &temp_attrib);
+      if (AL_FLAGS(ptr) & (AF_NOPROG | AF_PRIVATE)
+	  || !(AL_FLAGS(ptr) & flag_mask))
 	continue;
       strcpy(tbuf1, uncompress(AL_STR(ptr)));
       s = tbuf1;
@@ -806,17 +671,38 @@ atr_comm_match(thing, player, type, end, str, just_match)
       if (!s)
 	continue;
       *s++ = '\0';
+
+      if (AL_FLAGS(ptr) & AF_REGEXP) {
+	/* Turn \: into : */
+	char *from, *to;
+	for (from = tbuf1, to = tbuf2; *from; from++, to++) {
+	  if (*from == '\\' && *(from + 1) == ':')
+	    from++;
+	  *to = *from;
+	}
+	*to = '\0';
+      } else
+	strcpy(tbuf2, tbuf1);
+
       if ((AL_FLAGS(ptr) & AF_REGEXP) ?
-	  regexp_match_case(tbuf1 + 1, str, (AL_FLAGS(ptr) & AF_CASE)) :
-	  wild_match_case(tbuf1 + 1, str, (AL_FLAGS(ptr) & AF_CASE))) {
+	  regexp_match_case(tbuf2 + 1, str, (AL_FLAGS(ptr) & AF_CASE)) :
+	  wild_match_case(tbuf2 + 1, str, (AL_FLAGS(ptr) & AF_CASE))) {
 	match++;
+	if (atrname && abp) {
+	  safe_chr(' ', atrname, abp);
+	  if (Can_Examine(player, parent))
+	    safe_dbref(parent, atrname, abp);
+	  else
+	    safe_dbref(thing, atrname, abp);
+	  safe_chr('/', atrname, abp);
+	  safe_str(AL_NAME(ptr), atrname, abp);
+	}
 	if (!just_match)
 	  parse_que(thing, s, player);
       }
     }
   }
-
-  atr_hash_free();
+  st_flush(&temp_attrib);
   return match;
 }
 
@@ -851,6 +737,10 @@ do_set_atr(thing, atr, s, player, flags)
     s = NULL;
 #endif
 
+  if (IsGarbage(thing)) {
+    notify(player, T("Garbage is garbage."));
+    return 0;
+  }
   if (!controls(player, thing))
     return 0;
 
@@ -861,8 +751,8 @@ do_set_atr(thing, atr, s, player, flags)
     if (old) {
       /* Old alias - we're allowed to change to a different case */
       strcpy(tbuf1, uncompress(AL_STR(old)));
-      if (s && *s && strcasecmp(s, tbuf1) && !ok_player_name(s)) {
-	notify(player, "That is not a valid alias.");
+      if (s && (!*s || (strcasecmp(s, tbuf1) && !ok_player_name(s)))) {
+	notify(player, T("That is not a valid alias."));
 	return -1;
       }
       if (Can_Write_Attr(player, thing, old))
@@ -870,55 +760,88 @@ do_set_atr(thing, atr, s, player, flags)
     } else {
       /* No old alias */
       if (s && *s && !ok_player_name(s)) {
-	notify(player, "That is not a valid alias.");
+	notify(player, T("That is not a valid alias."));
 	return -1;
       }
     }
+  } else if (s && *s && !strcmp(name, "FORWARDLIST")) {
+    /* You can only set this to dbrefs of things you're allowed to
+     * forward to. If you get one wrong, we puke.
+     */
+    char *fwdstr, *curr;
+    dbref fwd;
+    strcpy(tbuf1, s);
+    fwdstr = trim_space_sep(tbuf1, ' ');
+    while ((curr = split_token(&fwdstr, ' ')) != NULL) {
+      if (!is_dbref(curr)) {
+	notify(player, T("@forwardlist should contain only dbrefs."));
+	return -1;
+      }
+      fwd = parse_dbref(curr);
+      if (!GoodObject(fwd) || IsGarbage(fwd)) {
+	notify_format(player, T("Invalid dbref #%d in @forwardlist."), fwd);
+	return -1;
+      }
+      if (!Can_Forward(thing, fwd)) {
+	notify_format(player, T("I don't think #%d wants to hear from you."),
+		      fwd);
+	return -1;
+      }
+    }
+    /* If you made it here, all your dbrefs were ok */
   }
+
   was_hearer = Hearer(thing);
   was_listener = Listener(thing);
 
   res =
-    s ? atr_add(thing, name, s, player,
-		(flags & 0x02) ? AF_NOPROG : NOTHING) : atr_clr(thing, name,
-								player);
-  if (res == -1) {
-    notify(player, "That attribute cannot be changed by you.");
+    s ? atr_add(thing, name, s, player, (flags & 0x02) ? AF_NOPROG : NOTHING)
+    : atr_clr(thing, name, player);
+  if (res == AE_SAFE) {
+    notify(player, T("That attribute is SAFE.  Set it !SAFE to modify it."));
+    return 0;
+  } else if (res == AE_BADNAME) {
+    notify(player, T("That's not a very good name for an attribute."));
+    return 0;
+  } else if (res == AE_ERROR) {
+    notify(player, T("That attribute cannot be changed by you."));
     return 0;
   } else if (!res) {
-    notify(player, "No such attribute to reset.");
+    notify(player, T("No such attribute to reset."));
     return 0;
   }
   if (!strcmp(name, "ALIAS") && IsPlayer(thing)) {
     if (s && *s) {
-      add_player(thing, (char *) s);
-      notify(player, "Alias set.");
+      add_player(thing, s);
+      notify(player, T("Alias set."));
     } else {
-      notify(player, "Alias removed.");
+      notify(player, T("Alias removed."));
     }
     return 1;
   } else if (!strcmp(name, "LISTEN")) {
     if (IsRoom(thing))
       contents = Contents(thing);
-    else
-      contents = Contents(Location(thing));
-    if (!s && !was_listener && !Hearer(thing)) {
-      notify_except(contents, thing,
-		    tprintf("%s loses its ears and becomes deaf.",
-			    Name(thing)));
-    } else if (s && !was_hearer && !was_listener) {
-      notify_except(contents, thing,
-		    tprintf("%s grows ears and can now hear.", Name(thing)));
+    else {
+      contents = Location(thing);
+      if (GoodObject(contents))
+	contents = Contents(contents);
+    }
+    if (GoodObject(contents)) {
+      if (!s && !was_listener && !Hearer(thing)) {
+	notify_except(contents, thing,
+		      tprintf(T("%s loses its ears and becomes deaf."),
+			      Name(thing)));
+      } else if (s && !was_hearer && !was_listener) {
+	notify_except(contents, thing,
+		      tprintf(T("%s grows ears and can now hear."),
+			      Name(thing)));
+      }
     }
-  } else if (!strcmp(name, "STARTUP")) {
-    if (s && *s)
-      Flags(thing) |= STARTUP;
-    else
-      Flags(thing) &= ~STARTUP;
   }
-  if ((flags & 0x01) && !Quiet(player) && !Quiet(thing))
-    notify(player,
-	   tprintf("%s/%s - %s.", Name(thing), name, s ? "Set" : "Cleared"));
+  if ((flags & 0x01) && !AreQuiet(player, thing))
+    notify_format(player,
+		  "%s/%s - %s.", Name(thing), name,
+		  s ? T("Set") : T("Cleared"));
   return 1;
 }
 
@@ -930,7 +853,7 @@ do_atrlock(player, arg1, arg2)
 {
   dbref thing;
   char *p;
-  ALIST *ptr;
+  ATTR *ptr;
   int status;
 
   if (!arg2 || !*arg2)
@@ -945,65 +868,52 @@ do_atrlock(player, arg1, arg2)
   }
 
   if (!arg1 || !*arg1) {
-    notify(player, "You need to give an object/attribute pair.");
+    notify(player, T("You need to give an object/attribute pair."));
     return;
   }
-  if (!(p = (char *) index(arg1, '/')) || !(*(p + 1))) {
-    notify(player, "You need to give an object/attribute pair.");
+  if (!(p = strchr(arg1, '/')) || !(*(p + 1))) {
+    notify(player, T("You need to give an object/attribute pair."));
     return;
   }
   *p++ = '\0';
 
   if ((thing = noisy_match_result(player, arg1, NOTYPE, MAT_EVERYTHING)) ==
-      NOTHING) return;
+      NOTHING)
+    return;
 
-  for (ptr = List(thing); ptr; ptr = AL_NEXT(ptr))
-    if (!strcasecmp(AL_NAME(ptr), p)
-#ifndef VISIBLE_EMPTY_ATTRS
-	&& *AL_STR(ptr)
-#endif
-      )
-      break;
-
-  if (!ptr) {
-    ptr = atr_match(strupper(p));
-    if (!ptr || !strcmp(p, AL_NAME(ptr)))
-      ptr = NULL;
-    else {
-      p = (char *) AL_NAME(ptr);
-      for (ptr = List(thing); ptr; ptr = AL_NEXT(ptr))
-	if (!strcmp(AL_NAME(ptr), p)
-#ifndef VISIBLE_EMPTY_ATTRS
-	    && *AL_STR(ptr)
-#endif
-	  )
-	  break;
-    }
+  if (!controls(player, thing)) {
+    notify(player, T("Permission denied."));
+    return;
   }
-  if (ptr) {
+
+  ptr = atr_get_noparent(thing, strupper(p));
+
+  if (ptr && Can_Read_Attr(player, thing, ptr)) {
     if (!status) {
-      notify(player, tprintf("That attribute is %slocked.",
-			     (AL_FLAGS(ptr) & AF_LOCKED) ? "" : "un"));
+      notify_format(player, T("That attribute is %slocked."),
+		    (AL_FLAGS(ptr) & AF_LOCKED) ? "" : "un");
       return;
-    } else if (!Wizard(player) && (Owner(AL_CREATOR(ptr)) != Owner(player))) {
-      notify(player, "You need to own the attribute to change its lock.");
+    } else if (!Can_Write_Attr(player, thing, ptr)) {
+      notify(player,
+	     T("You need to be able to set the attribute to change its lock."));
       return;
     } else {
       if (status == 1) {
 	AL_FLAGS(ptr) |= AF_LOCKED;
-	notify(player, "Attribute locked.");
+	AL_CREATOR(ptr) = Owner(player);
+	notify(player, T("Attribute locked."));
 	return;
       } else if (status == 2) {
 	AL_FLAGS(ptr) &= ~AF_LOCKED;
-	notify(player, "Attribute unlocked.");
+	notify(player, T("Attribute unlocked."));
 	return;
       } else {
-	notify(player, "Invalid status on atrlock.. Notify god.");
+	notify(player, T("Invalid status on atrlock.. Notify god."));
 	return;
       }
     }
   } else
-    notify(player, "No such attribute.");
+    notify(player, T("No such attribute."));
   return;
 }
 
@@ -1015,20 +925,26 @@ do_atrchown(player, arg1, arg2)
 {
   dbref thing, new_owner;
   char *p;
-  ALIST *ptr;
+  ATTR *ptr;
 
   if (!arg1 || !*arg1) {
-    notify(player, "You need to give an object/attribute pair.");
+    notify(player, T("You need to give an object/attribute pair."));
     return;
   }
-  if (!(p = (char *) index(arg1, '/')) || !(*(p + 1))) {
-    notify(player, "You need to give an object/attribute pair.");
+  if (!(p = strchr(arg1, '/')) || !(*(p + 1))) {
+    notify(player, T("You need to give an object/attribute pair."));
     return;
   }
   *p++ = '\0';
 
   if ((thing = noisy_match_result(player, arg1, NOTYPE, MAT_EVERYTHING)) ==
-      NOTHING) return;
+      NOTHING)
+    return;
+
+  if (!controls(player, thing)) {
+    notify(player, T("Permission denied."));
+    return;
+  }
 
   if ((!arg2 && !*arg2) || !strcasecmp(arg2, "me"))
     new_owner = player;
@@ -1036,47 +952,25 @@ do_atrchown(player, arg1, arg2)
     new_owner = lookup_player(arg2);
 
   if (new_owner == NOTHING) {
-    notify(player, "I can't find that player");
+    notify(player, T("I can't find that player"));
     return;
   }
-  for (ptr = List(thing); ptr; ptr = AL_NEXT(ptr))
-    if (!strcasecmp(AL_NAME(ptr), p)
-#ifndef VISIBLE_EMPTY_ATTRS
-	&& *AL_STR(ptr)
-#endif
-      )
-      break;
 
-  if (!ptr) {
-    ptr = atr_match(strupper(p));
-    if (!ptr || !strcmp(p, AL_NAME(ptr)))
-      ptr = NULL;
-    else {
-      p = (char *) AL_NAME(ptr);
-      for (ptr = List(thing); ptr; ptr = AL_NEXT(ptr))
-	if (!strcmp(AL_NAME(ptr), p)
-#ifndef VISIBLE_EMPTY_ATTRS
-	    && *AL_STR(ptr)
-#endif
-	  )
-	  break;
-    }
-  }
-  if (ptr) {
-    if ((controls(player, thing) && !(AL_FLAGS(ptr) & AF_LOCKED)) ||
-	(Owner(player) == Owner(AL_CREATOR(ptr)))) {
-      if (new_owner != Owner(thing) && !Wizard(player)) {
-	notify(player,
-	       "You can only chown an attribute to the current owner of the object.");
+  ptr = atr_get_noparent(thing, strupper(p));
+
+  if (ptr && Can_Read_Attr(player, thing, ptr)) {
+    if (Can_Write_Attr(player, thing, ptr)) {
+      if (new_owner != Owner(player) && !Wizard(player)) {
+	notify(player, T("You can only chown an attribute to yourself."));
 	return;
       }
       AL_CREATOR(ptr) = Owner(new_owner);
-      notify(player, "Attribute owner changed.");
+      notify(player, T("Attribute owner changed."));
       return;
     } else {
-      notify(player, "You don't have the permission to chown that.");
+      notify(player, T("You don't have the permission to chown that."));
       return;
     }
   } else
-    notify(player, "No such attribute.");
+    notify(player, T("No such attribute."));
 }
