@@ -6,21 +6,17 @@
 #include "copyrite.h"
 #include "config.h"
 
-#ifdef I_STRING
 #include <string.h>
-#else
-#include <strings.h>
-#endif
 
 #include "conf.h"
 #include "dbdefs.h"
 #include "mushdb.h"
 #include "externs.h"
-#include "intrface.h"
 #include "match.h"
 #include "game.h"
-#include "globals.h"
+#include "attrib.h"
 #include "extmail.h"
+#include "malias.h"
 #ifdef CHAT_SYSTEM
 #include "extchat.h"
 #endif
@@ -28,24 +24,34 @@
 #include "parse.h"
 #include "access.h"
 #include "version.h"
-
+#include "lock.h"
 #include "function.h"
 #include "command.h"
-
+#include "flags.h"
+#include "log.h"
 #include "confmagic.h"
 
 /* External Stuff */
-void do_config_list _((dbref player, const char *type));
-void do_poor _((dbref player, char *arg1));
-void do_dolist _((dbref player, char *list, char *command,
-		  dbref cause, int flag, int notify_flag));
-void do_list _((dbref player, char *arg));
-void do_writelog _((dbref player, char *str, int ltype));
-void do_reboot _((dbref player, int flag));
-void do_readcache _((dbref player));
-void do_scan _((dbref player, char *command, int flag));
-void do_uptime _((dbref player));
+void do_poor(dbref player, char *arg1);
+void do_list_memstats(dbref player);
 
+#define DOL_MAP 1
+#define DOL_NOTIFY 2
+#define DOL_DELIM 4
+
+void do_dolist(dbref player, char *list, char *command,
+	       dbref cause, unsigned int flags);
+void do_list(dbref player, char *arg, int lc);
+void do_writelog(dbref player, char *str, int ltype);
+void do_readcache(dbref player);
+void do_scan(dbref player, char *command, int flag);
+void do_uptime(dbref player, int mortal);
+extern int config_set(const char *opt, char *val, int source);
+
+/* From command.c */
+extern int rhs_present;
+void do_hook(dbref player, char *command, char *obj, char *attrname,
+	     int hook_type);
 
 COMMAND (cmd_allhalt) {
   do_allhalt(player);
@@ -87,6 +93,12 @@ COMMAND (cmd_boot) {
     do_boot(player, arg_left, 0);
 }
 
+extern int break_called;
+COMMAND (cmd_break) {
+  if (parse_boolean(arg_left))
+    break_called = 1;
+}
+
 #ifdef CHAT_SYSTEM
 COMMAND (cmd_cemit) {
   do_cemit(player, arg_left, arg_right, SW_ISSET(sw, SWITCH_NOISY));
@@ -110,9 +122,11 @@ COMMAND (cmd_channel) {
     do_chan_admin(player, arg_left, arg_right, 3);
   else if (SW_ISSET(sw, SWITCH_QUIET))
     do_chan_admin(player, arg_left, arg_right, 4);
+  else if (SW_ISSET(sw, SWITCH_NOISY))
+    do_chan_admin(player, arg_left, "n", 4);
   else if (SW_ISSET(sw, SWITCH_DECOMPILE))
-    do_chan_decompile(player, arg_left);
-  else if (SW_ISSET(sw, SWITCH_DESC))
+    do_chan_decompile(player, arg_left, SW_ISSET(sw, SWITCH_BRIEF));
+  else if (SW_ISSET(sw, SWITCH_DESCRIBE))
     do_chan_desc(player, arg_left, arg_right);
   else if (SW_ISSET(sw, SWITCH_TITLE))
     do_chan_title(player, arg_left, arg_right);
@@ -122,10 +136,16 @@ COMMAND (cmd_channel) {
     do_chan_wipe(player, arg_left);
   else if (SW_ISSET(sw, SWITCH_MUTE))
     do_chan_user_flags(player, arg_left, arg_right, 0, 0);
+  else if (SW_ISSET(sw, SWITCH_UNMUTE))
+    do_chan_user_flags(player, arg_left, "n", 0, 0);
   else if (SW_ISSET(sw, SWITCH_HIDE))
     do_chan_user_flags(player, arg_left, arg_right, 1, 0);
+  else if (SW_ISSET(sw, SWITCH_UNHIDE))
+    do_chan_user_flags(player, arg_left, "n", 1, 0);
   else if (SW_ISSET(sw, SWITCH_GAG))
     do_chan_user_flags(player, arg_left, arg_right, 2, 0);
+  else if (SW_ISSET(sw, SWITCH_UNGAG))
+    do_chan_user_flags(player, arg_left, "n", 2, 0);
   else if (SW_ISSET(sw, SWITCH_WHAT))
     do_chan_what(player, arg_left);
   else
@@ -133,17 +153,17 @@ COMMAND (cmd_channel) {
 }
 
 COMMAND (cmd_chat) {
-  do_chat_by_name(player, arg_left, arg_right);
+  do_chat_by_name(player, arg_left, arg_right, 1);
 }
 
 #endif
 
 COMMAND (cmd_chownall) {
-  do_chownall(player, arg_left, arg_right);
+  do_chownall(player, arg_left, arg_right, SW_ISSET(sw, SWITCH_PRESERVE));
 }
 
 COMMAND (cmd_chown) {
-  do_chown(player, arg_left, arg_right);
+  do_chown(player, arg_left, arg_right, SW_ISSET(sw, SWITCH_PRESERVE));
 }
 
 COMMAND (cmd_chzoneall) {
@@ -151,30 +171,49 @@ COMMAND (cmd_chzoneall) {
 }
 
 COMMAND (cmd_chzone) {
-  do_chzone(player, arg_left, arg_right);
+  (void) do_chzone(player, arg_left, arg_right, 1);
 }
 
 COMMAND (cmd_config) {
+  int lc;
+  lc = SW_ISSET(sw, SWITCH_LOWERCASE);
   if (SW_ISSET(sw, SWITCH_GLOBALS))
-    do_config_list(player, NULL);
+    do_config_list(player, NULL, lc);
   else if (SW_ISSET(sw, SWITCH_DEFAULTS))
-    do_config_list(player, NULL);
+    do_config_list(player, NULL, lc);
   else if (SW_ISSET(sw, SWITCH_COSTS))
-    do_config_list(player, NULL);
+    do_config_list(player, NULL, lc);
   else if (SW_ISSET(sw, SWITCH_FUNCTIONS))
-    do_list_functions(player);
+    do_list_functions(player, lc);
   else if (SW_ISSET(sw, SWITCH_COMMANDS))
-    do_list_commands(player);
+    do_list_commands(player, lc);
   else if (SW_ISSET(sw, SWITCH_ATTRIBS))
-    do_list_attribs(player);
+    do_list_attribs(player, lc);
   else if (SW_ISSET(sw, SWITCH_LIST))
-    do_config_list(player, arg_left);
-  else
-    do_config_list(player, arg_left);
+    do_config_list(player, arg_left, lc);
+  else if (SW_ISSET(sw, SWITCH_SET)) {
+    if (!Wizard(player)) {
+      notify(player, T("You can't remake the world in your image."));
+      return;
+    }
+    if (!arg_left || !*arg_left) {
+      notify(player, T("What did you want to set?"));
+      return;
+    }
+    if (!arg_right || !*arg_right) {
+      notify(player, T("What did you want to set that to?"));
+      return;
+    }
+    if (!config_set(arg_left, arg_right, 1))
+      notify(player, T("Couldn't set that option"));
+    else
+      notify(player, T("Option set."));
+  } else
+    do_config_list(player, arg_left, lc);
 }
 
 COMMAND (cmd_cpattr) {
-  do_cpattr(player, arg_left, args_right, 0);
+  do_cpattr(player, arg_left, args_right, 0, SW_ISSET(sw, SWITCH_NOFLAGCOPY));
 }
 
 COMMAND (cmd_create) {
@@ -182,7 +221,10 @@ COMMAND (cmd_create) {
 }
 
 COMMAND (cmd_clone) {
-  do_clone(player, arg_left);
+  if (SW_ISSET(sw, SWITCH_PRESERVE))
+    do_clone(player, arg_left, arg_right, SWITCH_PRESERVE);
+  else
+    do_clone(player, arg_left, arg_right, SWITCH_NONE);
 }
 
 #ifdef CHAT_SYSTEM
@@ -198,7 +240,7 @@ COMMAND (cmd_clock) {
   else if (SW_ISSET(sw, SWITCH_HIDE))
     do_chan_lock(player, arg_left, arg_right, CL_HIDE);
   else
-    notify(player, "You must specify a type of lock");
+    notify(player, T("You must specify a type of lock"));
 }
 #endif
 
@@ -219,6 +261,10 @@ COMMAND (cmd_decompile) {
     do_decompile(player, arg_left, 0, SW_ISSET(sw, SWITCH_SKIPDEFAULTS));
 }
 
+COMMAND (cmd_teach) {
+  do_teach(player, cause, arg_left);
+}
+
 COMMAND (cmd_destroy) {
   do_destroy(player, arg_left, (SW_ISSET(sw, SWITCH_OVERRIDE)));
 }
@@ -239,12 +285,12 @@ COMMAND (cmd_doing) {
 }
 
 COMMAND (cmd_dolist) {
-  do_dolist(player, arg_left, arg_right, cause, 0,
-	    (SW_ISSET(sw, SWITCH_NOTIFY)));
-}
-
-COMMAND (cmd_drain) {
-  do_notify(player, cause, 2, arg_left, arg_right);
+  unsigned int flags = 0;
+  if (SW_ISSET(sw, SWITCH_NOTIFY))
+    flags |= DOL_NOTIFY;
+  if (SW_ISSET(sw, SWITCH_DELIMIT))
+    flags |= DOL_DELIM;
+  do_dolist(player, arg_left, arg_right, cause, flags);
 }
 
 COMMAND (cmd_dump) {
@@ -261,12 +307,12 @@ COMMAND (cmd_edit) {
 }
 
 COMMAND (cmd_elock) {
-  do_lock(player, arg_left, arg_right, (char *) Enter_Lock);
+  do_lock(player, arg_left, arg_right, Enter_Lock);
 }
 
 COMMAND (cmd_emit) {
   if (SW_ISSET(sw, SWITCH_ROOM))
-    do_lemit(player, arg_left);
+    do_lemit(player, arg_left, SW_ISSET(sw, SWITCH_SILENT));
   else
     do_emit(player, arg_left);
 }
@@ -289,7 +335,7 @@ COMMAND (cmd_entrances) {
 }
 
 COMMAND (cmd_eunlock) {
-  do_unlock(player, arg_left, (char *) Enter_Lock);
+  do_unlock(player, arg_left, Enter_Lock);
 }
 
 COMMAND (cmd_find) {
@@ -300,19 +346,6 @@ COMMAND (cmd_firstexit) {
   do_firstexit(player, arg_left);
 }
 
-COMMAND (cmd_fixdb) {
-  if (SW_ISSET(sw, SWITCH_LOCATION))
-    do_fixdb(player, arg_left, arg_right, 0);
-  else if (SW_ISSET(sw, SWITCH_CONTENTS))
-    do_fixdb(player, arg_left, arg_right, 1);
-  else if (SW_ISSET(sw, SWITCH_EXITS))
-    do_fixdb(player, arg_left, arg_right, 2);
-  else if (SW_ISSET(sw, SWITCH_NEXT))
-    do_fixdb(player, arg_left, arg_right, 3);
-  else
-    notify(player, "Command MUST have a switch.");
-}
-
 COMMAND (cmd_force) {
   do_force(player, arg_left, arg_right);
 }
@@ -320,8 +353,40 @@ COMMAND (cmd_force) {
 COMMAND (cmd_function) {
   if (SW_ISSET(sw, SWITCH_DELETE))
     do_function_delete(player, arg_left);
-  else
-    do_function(player, arg_left, args_right);
+  else if (SW_ISSET(sw, SWITCH_ENABLE))
+    do_function_toggle(player, arg_left, 1);
+  else if (SW_ISSET(sw, SWITCH_DISABLE))
+    do_function_toggle(player, arg_left, 0);
+  else if (SW_ISSET(sw, SWITCH_RESTRICT))
+    do_function_restrict(player, arg_left, args_right[1]);
+  else if (SW_ISSET(sw, SWITCH_RESTORE))
+    do_function_restore(player, arg_left);
+  else {
+    int split;
+    char *saved;
+    split = 0;
+    saved = NULL;
+    if (args_right[1] && *args_right[1] && !(args_right[2] && *args_right[2])) {
+      split = 1;
+      saved = args_right[2];
+      if ((args_right[2] = strchr(args_right[1], '/')) == NULL) {
+	notify(player, T("#-1 INVALID SECOND ARGUMENT"));
+	return;
+      }
+      *args_right[2]++ = '\0';
+    }
+    if (args_right[1] && *args_right[1])
+      do_function(player, arg_left, args_right);
+    else if (arg_left && *arg_left)
+      do_function_report(player, arg_left);
+    else
+      do_function(player, NULL, NULL);
+    if (split) {
+      if (args_right[2])
+	*--args_right[2] = '/';
+      args_right[2] = saved;
+    }
+  }
 }
 
 COMMAND (cmd_gedit) {
@@ -346,16 +411,35 @@ COMMAND (cmd_hide) {
   hide_player(player, !(SW_ISSET(sw, SWITCH_NO) || SW_ISSET(sw, SWITCH_OFF)));
 }
 
+COMMAND (cmd_hook) {
+  int hook_type = 0;
+
+  if (!Wizard(player)) {
+    notify(player, T("You need a fishing license to use that hook."));
+    return;
+  }
+
+  if (SW_ISSET(sw, SWITCH_AFTER))
+    hook_type = 2;
+  else if (SW_ISSET(sw, SWITCH_BEFORE))
+    hook_type = 1;
+  else {
+    notify(player, T("You must give a switch for @hook"));
+    return;
+  }
+  do_hook(player, arg_left, args_right[1], args_right[2], hook_type);
+}
+
 COMMAND (cmd_kick) {
   do_kick(player, arg_left);
 }
 
 COMMAND (cmd_lemit) {
-  do_lemit(player, arg_left);
+  do_lemit(player, arg_left, SW_ISSET(sw, SWITCH_SILENT));
 }
 
 COMMAND (cmd_link) {
-  do_link(player, arg_left, arg_right);
+  do_link(player, arg_left, arg_right, SW_ISSET(sw, SWITCH_PRESERVE));
 }
 
 COMMAND (cmd_listmotd) {
@@ -363,23 +447,25 @@ COMMAND (cmd_listmotd) {
 }
 
 COMMAND (cmd_list) {
+  int lc;
+  lc = SW_ISSET(sw, SWITCH_LOWERCASE);
   if (SW_ISSET(sw, SWITCH_MOTD))
     do_motd(player, 3, "");
   else if (SW_ISSET(sw, SWITCH_FUNCTIONS))
-    do_list_functions(player);
+    do_list_functions(player, lc);
   else if (SW_ISSET(sw, SWITCH_COMMANDS))
-    do_list_commands(player);
+    do_list_commands(player, lc);
   else if (SW_ISSET(sw, SWITCH_ATTRIBS))
-    do_list_attribs(player);
+    do_list_attribs(player, lc);
   else
-    do_list(player, arg_left);
+    do_list(player, arg_left, lc);
 }
 
 COMMAND (cmd_lock) {
   if ((switches) && (switches[0]))
     do_lock(player, arg_left, arg_right, switches);
   else
-    do_lock(player, arg_left, arg_right, (char *) Basic_Lock);
+    do_lock(player, arg_left, arg_right, Basic_Lock);
 }
 
 COMMAND (cmd_log) {
@@ -414,10 +500,16 @@ COMMAND (cmd_logwipe) {
     do_logwipe(player, LT_ERR, arg_left);
 }
 
+COMMAND (cmd_lset) {
+  do_lset(player, arg_left, arg_right);
+}
+
 #ifdef USE_MAILER
 COMMAND (cmd_mail) {
   int urgent = SW_ISSET(sw, SWITCH_URGENT);
   int silent = SW_ISSET(sw, SWITCH_SILENT);
+  int nosig = SW_ISSET(sw, SWITCH_NOSIG);
+  /* First, mail commands that can be used even if you're gagged */
   if (SW_ISSET(sw, SWITCH_STATS))
     do_mail_stats(player, arg_left, 0);
   else if (SW_ISSET(sw, SWITCH_DSTATS))
@@ -428,7 +520,7 @@ COMMAND (cmd_mail) {
     do_mail_debug(player, arg_left, arg_right);
   else if (SW_ISSET(sw, SWITCH_NUKE))
     do_mail_nuke(player);
-  else if (SW_ISSET(sw, SWITCH_FOLDER))
+  else if (SW_ISSET(sw, SWITCH_FOLDERS))
     do_mail_change_folder(player, arg_left, arg_right);
   else if (SW_ISSET(sw, SWITCH_UNFOLDER))
     do_mail_unfolder(player, arg_left);
@@ -448,20 +540,68 @@ COMMAND (cmd_mail) {
     do_mail_tag(player, arg_left);
   else if (SW_ISSET(sw, SWITCH_UNTAG))
     do_mail_untag(player, arg_left);
-  else if (SW_ISSET(sw, SWITCH_FWD))
-    do_mail_fwd(player, arg_left, arg_right);
-  else if (SW_ISSET(sw, SWITCH_FORWARD))
-    do_mail_fwd(player, arg_left, arg_right);
-  else if (SW_ISSET(sw, SWITCH_SEND) || silent || urgent)
-    do_mail_send(player, arg_left, arg_right,
-		 urgent ? M_URGENT : 0, silent ? 1 : 0);
-  else
-    do_mail(player, arg_left, arg_right);
+  else if (SW_ISSET(sw, SWITCH_FWD) || SW_ISSET(sw, SWITCH_FORWARD)
+	   || SW_ISSET(sw, SWITCH_SEND) || silent || urgent || nosig) {
+    /* These commands are not allowed to gagged players */
+    if (Gagged(player)) {
+      notify(player, T("You cannot do that while gagged."));
+      return;
+    }
+    if (SW_ISSET(sw, SWITCH_FWD))
+      do_mail_fwd(player, arg_left, arg_right);
+    else if (SW_ISSET(sw, SWITCH_FORWARD))
+      do_mail_fwd(player, arg_left, arg_right);
+    else if (SW_ISSET(sw, SWITCH_SEND) || silent || urgent || nosig)
+      do_mail_send(player, arg_left, arg_right,
+		   urgent ? M_URGENT : 0, silent, nosig);
+  } else
+    do_mail(player, arg_left, arg_right);	/* Does its own gagged check */
 }
+
+#ifdef MAIL_ALIASES
+
+COMMAND (cmd_malias) {
+  if (SW_ISSET(sw, SWITCH_LIST))
+    do_malias_list(player);
+  else if (SW_ISSET(sw, SWITCH_ALL))
+    do_malias_all(player);
+  else if (SW_ISSET(sw, SWITCH_MEMBERS) || SW_ISSET(sw, SWITCH_WHO))
+    do_malias_members(player, arg_left);
+  else if (SW_ISSET(sw, SWITCH_CREATE))
+    do_malias_create(player, arg_left, arg_right);
+  else if (SW_ISSET(sw, SWITCH_SET))
+    do_malias_set(player, arg_left, arg_right);
+  else if (SW_ISSET(sw, SWITCH_DESTROY))
+    do_malias_destroy(player, arg_left);
+  else if (SW_ISSET(sw, SWITCH_ADD))
+    do_malias_add(player, arg_left, arg_right);
+  else if (SW_ISSET(sw, SWITCH_REMOVE))
+    do_malias_remove(player, arg_left, arg_right);
+  else if (SW_ISSET(sw, SWITCH_DESCRIBE))
+    do_malias_desc(player, arg_left, arg_right);
+  else if (SW_ISSET(sw, SWITCH_RENAME))
+    do_malias_rename(player, arg_left, arg_right);
+  else if (SW_ISSET(sw, SWITCH_STATS))
+    do_malias_stats(player);
+  else if (SW_ISSET(sw, SWITCH_CHOWN))
+    do_malias_chown(player, arg_left, arg_right);
+  else if (SW_ISSET(sw, SWITCH_USEFLAG))
+    do_malias_privs(player, arg_left, arg_right, 0);
+  else if (SW_ISSET(sw, SWITCH_SEEFLAG))
+    do_malias_privs(player, arg_left, arg_right, 1);
+  else if (SW_ISSET(sw, SWITCH_NUKE))
+    do_malias_nuke(player);
+  else
+    do_malias(player, arg_left, arg_right);
+}
+#endif
 #endif
 
 COMMAND (cmd_map) {
-  do_dolist(player, arg_left, arg_right, cause, 1, 0);
+  unsigned int flags = DOL_MAP;
+  if (SW_ISSET(sw, SWITCH_DELIMIT))
+    flags |= DOL_DELIM;
+  do_dolist(player, arg_left, arg_right, cause, flags);
 }
 
 COMMAND (cmd_motd) {
@@ -480,7 +620,7 @@ COMMAND (cmd_motd) {
 }
 
 COMMAND (cmd_mvattr) {
-  do_cpattr(player, arg_left, args_right, 1);
+  do_cpattr(player, arg_left, args_right, 1, SW_ISSET(sw, SWITCH_NOFLAGCOPY));
 }
 
 COMMAND (cmd_name) {
@@ -491,8 +631,20 @@ COMMAND (cmd_newpassword) {
   do_newpassword(player, arg_left, arg_right);
 }
 
-COMMAND (cmd_notify) {
-  do_notify(player, cause, (SW_ISSET(sw, SWITCH_ALL)), arg_left, arg_right);
+COMMAND (cmd_nspemit) {
+  int silent;
+
+  if (SW_ISSET(sw, SWITCH_SILENT))
+    silent = 1;
+  else if (SW_ISSET(sw, SWITCH_NOISY))
+    silent = 0;
+  else
+    silent = SILENT_PEMIT ? 1 : 0;
+
+  if (SW_ISSET(sw, SWITCH_LIST))
+    do_pemit_list(player, arg_left, arg_right, 0);
+  else
+    do_pemit(player, arg_left, arg_right, silent, 0);
 }
 
 COMMAND (cmd_nuke) {
@@ -500,7 +652,7 @@ COMMAND (cmd_nuke) {
 }
 
 COMMAND (cmd_oemit) {
-  do_oemit(player, arg_left, arg_right);
+  do_oemit_list(player, arg_left, arg_right);
 }
 
 COMMAND (cmd_open) {
@@ -530,11 +682,11 @@ COMMAND (cmd_pemit) {
     silent = SILENT_PEMIT ? 1 : 0;
 
   if (SW_ISSET(sw, SWITCH_LIST))
-    do_pemit_list(player, arg_left, arg_right);
+    do_pemit_list(player, arg_left, arg_right, 1);
   else if (SW_ISSET(sw, SWITCH_CONTENTS))
-    do_remit(player, arg_left, arg_right);
+    do_remit(player, arg_left, arg_right, !silent);
   else
-    do_pemit(player, arg_left, arg_right, silent);
+    do_pemit(player, arg_left, arg_right, silent, 1);
 }
 
 COMMAND (cmd_poll) {
@@ -552,9 +704,7 @@ COMMAND (cmd_power) {
 COMMAND (cmd_ps) {
   if (SW_ISSET(sw, SWITCH_ALL))
     do_queue(player, arg_left, 1);
-  else if (SW_ISSET(sw, SWITCH_SUMMARY))
-    do_queue(player, arg_left, 2);
-  else if (SW_ISSET(sw, SWITCH_COUNT))
+  else if (SW_ISSET(sw, SWITCH_SUMMARY) || SW_ISSET(sw, SWITCH_COUNT))
     do_queue(player, arg_left, 2);
   else if (SW_ISSET(sw, SWITCH_QUICK))
     do_queue(player, arg_left, 3);
@@ -582,7 +732,16 @@ COMMAND (cmd_readcache) {
 }
 
 COMMAND (cmd_remit) {
-  do_remit(player, arg_left, arg_right);
+  int flags;
+  if (SW_ISSET(sw, SWITCH_SILENT))
+    flags = 0;
+  else if (SW_ISSET(sw, SWITCH_NOISY))
+    flags = 1;
+  else
+    flags = SILENT_PEMIT ? 0 : 1;
+  if (SW_ISSET(sw, SWITCH_LIST))
+    flags |= 0x2;
+  do_remit(player, arg_left, arg_right, flags);
 }
 
 COMMAND (cmd_rejectmotd) {
@@ -627,7 +786,8 @@ COMMAND (cmd_search) {
 }
 
 COMMAND (cmd_select) {
-  do_switch(player, arg_left, args_right, cause, 1);
+  do_switch(player, arg_left, args_right, cause, 1,
+	    (SW_ISSET(sw, SWITCH_NOTIFY)));
 }
 
 COMMAND (cmd_set) {
@@ -636,6 +796,10 @@ COMMAND (cmd_set) {
 
 COMMAND (cmd_shutdown) {
   int paranoid;
+  if (!Wizard(player)) {
+    notify(player, T("You don't have the authority to do that!"));
+    return;
+  }
   paranoid = (SW_ISSET(sw, SWITCH_PARANOID)) ? 1 : 0;
   if (SW_ISSET(sw, SWITCH_REBOOT))
     do_reboot(player, paranoid);
@@ -647,17 +811,22 @@ COMMAND (cmd_shutdown) {
 
 COMMAND (cmd_sitelock) {
   if (SW_ISSET(sw, SWITCH_BAN))
-    do_sitelock(player, arg_left, NULL, 1);
+    do_sitelock(player, arg_left, NULL, NULL, 1);
   else if (SW_ISSET(sw, SWITCH_REGISTER))
-    do_sitelock(player, arg_left, NULL, 0);
+    do_sitelock(player, arg_left, NULL, NULL, 0);
   else if (SW_ISSET(sw, SWITCH_NAME))
-    do_sitelock(player, arg_left, NULL, 2);
+    do_sitelock(player, arg_left, NULL, NULL, 2);
+  else if (SW_ISSET(sw, SWITCH_REMOVE))
+    do_sitelock(player, arg_left, NULL, NULL, 3);
   else
-    do_sitelock(player, arg_left, arg_right, 0);
+    do_sitelock(player, arg_left, args_right[1], args_right[2], 0);
 }
 
 COMMAND (cmd_stats) {
-  do_stats(player, arg_left);
+  if (SW_ISSET(sw, SWITCH_TABLES))
+    do_list_memstats(player);
+  else
+    do_stats(player, arg_left);
 }
 
 COMMAND (cmd_sweep) {
@@ -674,7 +843,8 @@ COMMAND (cmd_sweep) {
 }
 
 COMMAND (cmd_switch) {
-  do_switch(player, arg_left, args_right, cause, (SW_ISSET(sw, SWITCH_FIRST)));
+  do_switch(player, arg_left, args_right, cause, (SW_ISSET(sw, SWITCH_FIRST)),
+	    (SW_ISSET(sw, SWITCH_NOTIFY)));
 }
 
 #ifdef QUOTA
@@ -685,7 +855,10 @@ COMMAND (cmd_squota) {
 #endif
 
 COMMAND (cmd_teleport) {
-  do_teleport(player, arg_left, arg_right);
+  if (rhs_present && !*arg_right)
+    notify(player, T("You can't teleport to nothing!"));
+  else
+    do_teleport(player, arg_left, arg_right, (SW_ISSET(sw, SWITCH_SILENT)));
 }
 
 COMMAND (cmd_trigger) {
@@ -693,7 +866,7 @@ COMMAND (cmd_trigger) {
 }
 
 COMMAND (cmd_ulock) {
-  do_lock(player, arg_left, arg_right, (char *) Use_Lock);
+  do_lock(player, arg_left, arg_right, Use_Lock);
 }
 
 COMMAND (cmd_undestroy) {
@@ -708,15 +881,15 @@ COMMAND (cmd_unlock) {
   if ((switches) && (switches[0]))
     do_unlock(player, arg_left, switches);
   else
-    do_unlock(player, arg_left, (char *) Basic_Lock);
+    do_unlock(player, arg_left, Basic_Lock);
 }
 
 COMMAND (cmd_uptime) {
-  do_uptime(player);
+  do_uptime(player, SW_ISSET(sw, SWITCH_MORTAL));
 }
 
 COMMAND (cmd_uunlock) {
-  do_unlock(player, arg_left, (char *) Use_Lock);
+  do_unlock(player, arg_left, Use_Lock);
 }
 
 COMMAND (cmd_verb) {
@@ -728,7 +901,7 @@ COMMAND (cmd_version) {
 }
 
 COMMAND (cmd_wait) {
-  do_wait(player, cause, arg_left, arg_right);
+  do_wait(player, cause, arg_left, arg_right, SW_ISSET(sw, SWITCH_UNTIL));
 }
 
 COMMAND (cmd_wall) {
@@ -761,6 +934,8 @@ COMMAND (cmd_warnings) {
 COMMAND (cmd_wcheck) {
   if (SW_ISSET(sw, SWITCH_ALL))
     do_wcheck_all(player);
+  else if (SW_ISSET(sw, SWITCH_ME))
+    do_wcheck_me(player);
   else
     do_wcheck(player, arg_left);
 }
@@ -794,14 +969,6 @@ COMMAND (cmd_zemit) {
   do_zemit(player, arg_left, arg_right);
 }
 
-COMMAND (cmd_ahelp) {
-  do_new_spitfile(player, arg_left, HELPINDX, HELPTEXT, 1);
-}
-
-COMMAND (cmd_anews) {
-  do_new_spitfile(player, arg_left, NEWSINDX, NEWS_FILE, 1);
-}
-
 COMMAND (cmd_brief) {
   do_examine(player, arg_left, 1);
 }
@@ -822,15 +989,23 @@ COMMAND (cmd_examine) {
 }
 
 COMMAND (cmd_enter) {
-  do_enter(player, arg_left, 0);
+  do_enter(player, arg_left);
 }
 
-COMMAND (cmd_events) {
-  do_new_spitfile(player, arg_left, EVENTINDX, EVENT_FILE, 0);
+COMMAND (cmd_dismiss) {
+  do_dismiss(player, arg_left);
+}
+
+COMMAND (cmd_desert) {
+  do_desert(player, arg_left);
 }
 
 COMMAND (cmd_follow) {
-  notify(player, "COMMAND NOT IMPLEMENTED YET.");
+  do_follow(player, arg_left);
+}
+
+COMMAND (cmd_unfollow) {
+  do_unfollow(player, arg_left);
 }
 
 COMMAND (cmd_get) {
@@ -838,25 +1013,17 @@ COMMAND (cmd_get) {
 }
 
 COMMAND (cmd_give) {
-  do_give(player, arg_left, arg_right);
+  do_give(player, arg_left, arg_right, (SW_ISSET(sw, SWITCH_SILENT)));
 }
 
 COMMAND (cmd_goto) {
   move_wrapper(player, arg_left);
 }
 
-COMMAND (cmd_help) {
-  do_new_spitfile(player, arg_left, HELPINDX, HELPTEXT, 0);
-}
-
 COMMAND (cmd_inventory) {
   do_inventory(player);
 }
 
-COMMAND (cmd_index) {
-  do_new_spitfile(player, arg_left, INDEXINDX, INDEX_FILE, 0);
-}
-
 COMMAND (cmd_kill) {
   do_kill(player, arg_left, atol(arg_right), 0);
 }
@@ -869,10 +1036,6 @@ COMMAND (cmd_leave) {
   do_leave(player);
 }
 
-COMMAND (cmd_news) {
-  do_new_spitfile(player, arg_left, NEWSINDX, NEWS_FILE, 0);
-}
-
 COMMAND (cmd_page) {
   char *t;
   if ((arg_right) && (*arg_right)) {
@@ -880,39 +1043,19 @@ COMMAND (cmd_page) {
   } else {
     t = NULL;
   }
-  if (SW_ISSET(sw, SWITCH_LIST))
-    do_multipage(player, arg_left, t);
-  else if ((BLIND_PAGE) || (SW_ISSET(sw, SWITCH_BLIND)))
-    do_page(player, arg_left, t);
+  if (SW_ISSET(sw, SWITCH_PORT))
+    do_page_port(player, arg_left, t);
   else
-    do_multipage(player, arg_left, t);
+    do_page(player, arg_left, t, cause, SW_ISSET(sw, SWITCH_NOEVAL),
+	    !(SW_ISSET(sw, SWITCH_BLIND) ||
+	      (!(SW_ISSET(sw, SWITCH_LIST)) && (BLIND_PAGE))),
+	    SW_ISSET(sw, SWITCH_OVERRIDE));
 }
 
 COMMAND (cmd_pose) {
   do_pose(player, arg_left, (SW_ISSET(sw, SWITCH_NOSPACE)));
 }
 
-COMMAND (cmd_rob) {
-  do_rob(player, arg_left);
-}
-
-#ifdef ALLOW_RPAGE
-COMMAND (cmd_rpage) {
-  if (SW_ISSET(sw, SWITCH_LIST))
-    do_rpage_list(player);
-  else if (SW_ISSET(sw, SWITCH_ADD))
-    do_rpage_add(player, arg_left, arg_right);
-  else if (SW_ISSET(sw, SWITCH_DELETE))
-    do_rpage_delete(player, arg_left, arg_right);
-  else
-    do_rpage(player, arg_left, arg_right);
-}
-#endif
-
-COMMAND (cmd_rules) {
-  do_new_spitfile(player, arg_left, (char *) RULESINDX, RULES_FILE, 0);
-}
-
 COMMAND (cmd_say) {
   do_say(player, arg_left);
 }
@@ -938,17 +1081,9 @@ COMMAND (cmd_think) {
 }
 
 COMMAND (cmd_whisper) {
-  int noisy;
-  if (SW_ISSET(sw, SWITCH_NOISY))
-    noisy = 1;
-  else if (SW_ISSET(sw, SWITCH_SILENT))
-    noisy = 0;
-  else
-    noisy = (NOISY_WHISPER ? 1 : 0);
-  if (SW_ISSET(sw, SWITCH_LIST))
-    do_whisper_list(player, arg_left, arg_right, noisy);
-  else
-    do_whisper(player, arg_left, arg_right, noisy);
+  do_whisper(player, arg_left, arg_right,
+	     (SW_ISSET(sw, SWITCH_NOISY) ||
+	      (!SW_ISSET(sw, SWITCH_SILENT) && NOISY_WHISPER)));
 }
 
 COMMAND (cmd_use) {
@@ -962,7 +1097,7 @@ COMMAND (command_atrset) {
     return;
 
   /* If it's &attr obj, we must pass a NULL. If &attr obj=, pass "" */
-  if (strchr(raw, '=')) {
+  if (rhs_present) {
     do_set_atr(thing, switches, arg_right, player,
 	       0x1 | (SW_ISSET(sw, SWITCH_NOEVAL) ? 0 : 0x02));
   } else {
