@@ -3,6 +3,7 @@
 
 #include <ctype.h>
 #include <assert.h>
+#include <limits.h>
 #include <stdlib.h>
 #ifdef I_STRING
 #include <string.h>
@@ -15,7 +16,14 @@
 #include "mushdb.h"
 #include "match.h"
 #include "externs.h"
-#include "globals.h"
+#include "log.h"
+#include "game.h"
+#include "extmail.h"
+#include "malias.h"
+#include "attrib.h"
+#include "dbdefs.h"
+#include "flags.h"
+#include "lock.h"
 #include "confmagic.h"
 
 /* These lengthy comments are by Ralph Melton, December 1995. */
@@ -67,40 +75,31 @@
  */
 
 dbref first_free = NOTHING;
+static dbref what_to_destroy(dbref player, char *name, int confirm);
 
-void do_destroy _((dbref player, char *name, int confirm));
-void do_undestroy _((dbref player, char *name));
-dbref free_get _((void));
-void fix_free_list _((void));
-void purge _((void));
-void do_purge _((dbref player));
+static void pre_destroy(dbref player, dbref thing);
+static void free_object(dbref thing);
+static void empty_contents(dbref thing);
+static void clear_thing(dbref thing);
+static void clear_player(dbref thing);
+static void clear_room(dbref thing);
+static void clear_exit(dbref thing);
 
-void dbck _((void));
-void do_dbck _((dbref player));
-
-static dbref what_to_destroy _((dbref player, char *name, int confirm));
-
-static void pre_destroy _((dbref player, dbref thing));
-void undestroy _((dbref player, dbref thing));
-static void free_object _((dbref thing));
-static void empty_contents _((dbref thing));
-static void clear_thing _((dbref thing));
-static void clear_player _((dbref thing));
-static void clear_room _((dbref thing));
-static void clear_exit _((dbref thing));
-
-static void check_fields _((void));
-static void check_connected_rooms _((void));
-static void mark_connected _((dbref loc));
-static void check_connected_marks _((void));
-static void mark_contents _((dbref loc));
-static void check_contents _((void));
-static void check_locations _((void));
+static void check_fields(void);
+static void check_connected_rooms(void);
+static void mark_connected(dbref loc);
+static void check_connected_marks(void);
+static void mark_contents(dbref loc);
+static void check_contents(void);
+static void check_locations(void);
+static void check_zones(void);
 static int attribute_owner_helper
-_((dbref player, dbref thing, char const *pattern, ATTR *atr, void *args));
+  (dbref player, dbref thing, char const *pattern, ATTR *atr, void *args);
 
-extern void remove_all_obj_chan _((dbref thing));
-extern void chan_chownall _((dbref old, dbref new));
+extern void remove_all_obj_chan(dbref thing);
+extern void chan_chownall(dbref old, dbref new);
+
+extern struct db_stat_info current_state;
 
 /* Section I: do_destroy() and related functions. This section is where
  * the human interface of do_destroy() should largely be determined.
@@ -209,54 +208,55 @@ what_to_destroy(player, name, confirm)
     return NOTHING;
 
   if (IsGarbage(thing)) {
-    notify(player, "Destroying that again is hardly necessary.");
+    notify(player, T("Destroying that again is hardly necessary."));
     return NOTHING;
   }
   if (God(thing)) {
-    notify(player, "Destroying God would be blasphemous.");
+    notify(player, T("Destroying God would be blasphemous."));
     return NOTHING;
   }
   /* To destroy, you must either:
-   * 1. Own it (and control it)
-   * 2. Be at least a Wizard
-   * 3. Control its source or destination if it's an exit
-   * 4. Be dealing with a dest-ok thing
+   * 1. Control it
+   * 2. Control its source or destination if it's an exit
+   * 3. Be dealing with a dest-ok thing and pass its lock/destroy
    */
-  if (!(Owns(player, thing) && controls(player, thing)) &&
-      !Wizard(player) &&
+  if (!controls(player, thing) &&
       !(IsExit(thing) &&
 	(controls(player, Destination(thing)) ||
-	 controls(player, Source(thing)))) && !DestOk(thing)) {
-    notify(player, "Permission denied.");
+	 controls(player, Source(thing)))) &&
+      !(DestOk(thing) && eval_lock(player, thing, Destroy_Lock))) {
+    notify(player, T("Permission denied."));
     return NOTHING;
   }
-  if (thing == PLAYER_START || thing == MASTER_ROOM || God(thing)) {
-    notify(player, "That is too special to be destroyed.");
+  if (thing == PLAYER_START || thing == MASTER_ROOM || thing == BASE_ROOM
+      || God(thing)) {
+    notify(player, T("That is too special to be destroyed."));
     return NOTHING;
   }
   if (REALLY_SAFE) {
     if (Safe(thing) && !DestOk(thing)) {
       notify(player,
-	     "That object is set SAFE. You must set it !SAFE before destroying it.");
+	     T
+	     ("That object is set SAFE. You must set it !SAFE before destroying it."));
       return NOTHING;
     }
   } else {			/* REALLY_SAFE */
     if (Safe(thing) && !DestOk(thing) && !confirm) {
-      notify(player, "That object is marked SAFE. Use @nuke to destroy it.");
+      notify(player, T("That object is marked SAFE. Use @nuke to destroy it."));
       return NOTHING;
     }
   }
   /* check to make sure there's no accidental destruction */
   if (!confirm && !Owns(player, thing) && !DestOk(thing)) {
     notify(player,
-	   "That object does not belong to you. Use @nuke to destroy it.");
+	   T("That object does not belong to you. Use @nuke to destroy it."));
     return NOTHING;
   }
   /* what kind of thing we are destroying? */
   switch (Typeof(thing)) {
   case TYPE_PLAYER:
     if (!IsPlayer(player)) {
-      notify(player, "Programs don't kill people; people kill people!");
+      notify(player, T("Programs don't kill people; people kill people!"));
       return NOTHING;
     }
     /* The only player a player can own() is themselves...
@@ -265,38 +265,38 @@ what_to_destroy(player, name, confirm)
      * the error message is a bit confusing. -DTC
      */
     if (!Wizard(player)) {
-      notify(player, "Sorry, no suicide allowed.");
+      notify(player, T("Sorry, no suicide allowed."));
       return NOTHING;
     }
     /* Already checked for God(thing), so use Wizard() */
     if (Wizard(thing) && !God(player)) {
-      notify(player, "Even you can't do that!");
+      notify(player, T("Even you can't do that!"));
       return NOTHING;
     }
     if (Connected(thing)) {
       notify(player,
-	     "How gruesome. You may not destroy players who are connected.");
+	     T("How gruesome. You may not destroy players who are connected."));
       return NOTHING;
     }
     if (!confirm) {
-      notify(player, "You must use @nuke to destroy a player.");
+      notify(player, T("You must use @nuke to destroy a player."));
       return NOTHING;
     }
     break;
   case TYPE_THING:
     if (!confirm && Wizard(thing)) {
       notify(player,
-	     "That object is set WIZARD. You must use @nuke to destroy it.");
+	     T("That object is set WIZARD. You must use @nuke to destroy it."));
       return NOTHING;
     }
     if (thing == Home(player) && !IsExit(player)) {
-      notify(player, "No home-wrecking allowed! Relink first.");
+      notify(player, T("No home-wrecking allowed! Relink first."));
       return NOTHING;
     }
     break;
   case TYPE_ROOM:
     if (thing == Home(player) && !IsExit(player)) {
-      notify(player, "No home-wrecking allowed! Relink first.");
+      notify(player, T("No home-wrecking allowed! Relink first."));
       return NOTHING;
     }
     break;
@@ -324,61 +324,74 @@ do_destroy(player, name, confirm)
    */
   if (Going(thing)) {
     free_object(thing);
-    notify(player, "Destroyed.");
+    notify(player, T("Destroyed."));
     return;
   }
   /* Present informative messages. */
+  if (!REALLY_SAFE && Safe(thing))
+    notify(player,
+	   T
+	   ("Warning: Target is set SAFE, but scheduling for destruction anyway."));
   switch (Typeof(thing)) {
   case TYPE_ROOM:
     /* wait until dbck */
     notify_except(Contents(thing), NOTHING,
-		  "The room shakes and begins to crumble.");
+		  T("The room shakes and begins to crumble."));
     if (Owns(player, thing))
-      notify(player,
-	     tprintf("You will be rewarded shortly for %s.",
-		     object_header(player, thing)));
+      notify_format(player,
+		    T("You will be rewarded shortly for %s."),
+		    object_header(player, thing));
     else {
-      notify(player,
-	     tprintf
-	     ("The wrecking ball is on its way for %s's %s and its exits.",
-	      Name(Owner(thing)), object_header(player, thing)));
-      notify(Owner(thing),
-	     tprintf("%s has scheduled your room %s to be destroyed.",
-		     Name(player), object_header(Owner(thing), thing)));
+      notify_format(player,
+		    T
+		    ("The wrecking ball is on its way for %s's %s and its exits."),
+		    Name(Owner(thing)), object_header(player, thing));
+      notify_format(Owner(thing),
+		    T("%s has scheduled your room %s to be destroyed."),
+		    Name(player), object_header(Owner(thing), thing));
     }
     break;
   case TYPE_PLAYER:
     /* wait until dbck */
-    notify(player,
-	   tprintf("%s and all their objects are scheduled to be destroyed.",
-		   object_header(player, thing)));
+    notify_format(player,
+		  (DESTROY_POSSESSIONS ?
+		   (REALLY_SAFE ?
+		    T
+		    ("%s and all their (non-SAFE) objects are scheduled to be destroyed.")
+		    :
+		    T
+		    ("%s and all their objects are scheduled to be destroyed."))
+		   : T("%s is scheduled to be destroyed.")),
+		  object_header(player, thing));
     break;
   case TYPE_THING:
     if (!Owns(player, thing)) {
-      notify(player, tprintf("%s's %s is scheduled to be destroyed.",
-			     Name(Owner(thing)), object_header(player, thing)));
+      notify_format(player, T("%s's %s is scheduled to be destroyed."),
+		    Name(Owner(thing)), object_header(player, thing));
       if (!DestOk(thing))
-	notify(Owner(thing),
-	       tprintf("%s has scheduled your %s for destruction.",
-		       Name(player), object_header(Owner(thing), thing)));
+	notify_format(Owner(thing),
+		      T("%s has scheduled your %s for destruction."),
+		      Name(player), object_header(Owner(thing), thing));
     } else {
-      notify(player, tprintf("%s is scheduled to be destroyed.",
-			     object_header(player, thing)));
+      notify_format(player, T("%s is scheduled to be destroyed."),
+		    object_header(player, thing));
     }
     break;
   case TYPE_EXIT:
     if (!Owns(player, thing)) {
-      notify(Owner(thing), tprintf("%s has scheduled your %s for destruction.",
-				   Name(player),
-				   object_header(Owner(thing), thing)));
-      notify(player, tprintf("%s's %s is scheduled to be destroyed.",
-			     Name(Owner(thing)), object_header(player, thing)));
+      notify_format(Owner(thing),
+		    T("%s has scheduled your %s for destruction."),
+		    Name(player), object_header(Owner(thing), thing));
+      notify_format(player,
+		    T("%s's %s is scheduled to be destroyed."),
+		    Name(Owner(thing)), object_header(player, thing));
     } else
-      notify(player, tprintf("%s is scheduled to be destroyed.",
-			     object_header(player, thing)));
+      notify_format(player,
+		    T("%s is scheduled to be destroyed."),
+		    object_header(player, thing));
     break;
   default:
-    do_log(LT_ERR, NOTHING, NOTHING, "Surprising type in do_destroy.");
+    do_log(LT_ERR, NOTHING, NOTHING, T("Surprising type in do_destroy."));
     return;
   }
 
@@ -401,17 +414,17 @@ do_undestroy(player, name)
     return;
   }
   if (!controls(player, thing)) {
-    notify(player, "Alas, your efforts of mercy are in vain.");
+    notify(player, T("Alas, your efforts of mercy are in vain."));
     return;
   }
   undestroy(player, thing);
-  notify(Owner(thing),
-	 tprintf("Your %s has been spared from destruction.",
-		 object_header(Owner(thing), thing)));
+  notify_format(Owner(thing),
+		T("Your %s has been spared from destruction."),
+		object_header(Owner(thing), thing));
   if (player != Owner(thing)) {
-    notify(player,
-	   tprintf("%s has been spared from destruction.",
-		   object_header(player, thing)));
+    notify_format(player,
+		  T("%s's %s has been spared from destruction."),
+		  Name(Owner(thing)), object_header(player, thing));
   }
 }
 
@@ -459,14 +472,14 @@ pre_destroy(player, thing)
      * whose owner hasn't already been notified. */
     if ((Owner(thing) != Owner(Source(thing))) && Going(Source(thing))) {
       if (!Owns(player, thing)) {
-	notify(Owner(thing),
-	       tprintf("%s has scheduled your %s for destruction.",
-		       Name(player), object_header(Owner(thing), thing)));
+	notify_format(Owner(thing),
+		      T("%s has scheduled your %s for destruction."),
+		      Name(player), object_header(Owner(thing), thing));
       }
     }
     break;
   default:
-    do_log(LT_ERR, NOTHING, NOTHING, "Surprising type in pre_destroy.");
+    do_log(LT_ERR, NOTHING, NOTHING, T("Surprising type in pre_destroy."));
     return;
   }
 
@@ -492,28 +505,19 @@ undestroy(player, thing)
   }
   Flags(thing) &= ~GOING;
   Flags(thing) &= ~GOING_TWICE;
-  if (Startup(thing) && !Halted(thing)) {
-    ATTR *s;
-    char *r;
-    s = atr_get_noparent(thing, "STARTUP");
-    if (s) {
-      r = safe_uncompress(s->value);
-      parse_que(thing, r, thing);
-      free((Malloc_t) r);
-    }
-  }
+  if (!Halted(thing))
+    (void) queue_attribute_noparent(thing, "STARTUP", thing);
   /* undestroy owner, if need be. */
   if (Going(Owner(thing))) {
     if (Owner(thing) != player) {
-      notify(player,
-	     tprintf("%s has been spared from destruction.",
-		     object_header(player, Owner(thing)))
-	);
-      notify(Owner(thing),
-	     tprintf("You have been spared from destruction by %s.",
-		     Name(player)));
+      notify_format(player,
+		    T("%s has been spared from destruction."),
+		    object_header(player, Owner(thing)));
+      notify_format(Owner(thing),
+		    T("You have been spared from destruction by %s."),
+		    Name(player));
     } else {
-      notify(player, "You have been spared from destruction.");
+      notify(player, T("You have been spared from destruction."));
     }
     undestroy(player, Owner(thing));
   }
@@ -538,20 +542,20 @@ undestroy(player, thing)
     /* undestroy containing room. */
     if (Going(Source(thing))) {
       undestroy(player, Source(thing));
-      notify(player,
-	     tprintf("The room %s has been spared from destruction.",
-		     object_header(player, Source(thing))));
+      notify_format(player,
+		    T("The room %s has been spared from destruction."),
+		    object_header(player, Source(thing)));
       if (Owner(Source(thing)) != player) {
-	notify(Owner(Source(thing)),
-	       tprintf("The room %s has been spared from destruction by %s.",
-		       object_header(Owner(Source(thing)), Source(thing)),
-		       Name(player)));
+	notify_format(Owner(Source(thing)),
+		      T("The room %s has been spared from destruction by %s."),
+		      object_header(Owner(Source(thing)), Source(thing)),
+		      Name(player));
       }
     }
     break;
   case TYPE_ROOM:
     /* undestroy exits in this room, except exits that are going to be
-       * destroyed anyway due to a GOING player.
+     * destroyed anyway due to a GOING player.
      */
     DOLIST(tmp, Exits(thing)) {
       if (DESTROY_POSSESSIONS ? (!Going(Owner(tmp)) || Safe(tmp)) : 1) {
@@ -560,7 +564,7 @@ undestroy(player, thing)
     }
     break;
   default:
-    do_log(LT_ERR, NOTHING, NOTHING, "Surprising type in un_destroy.");
+    do_log(LT_ERR, NOTHING, NOTHING, T("Surprising type in un_destroy."));
     return;
   }
 }
@@ -574,7 +578,7 @@ free_object(thing)
  * if the database format changes, this will likely have to change too.
  */
 {
-  dbref i;
+  dbref i, loc;
   if (!GoodObject(thing))
     return;
 #ifdef LOCAL_DATA
@@ -594,7 +598,7 @@ free_object(thing)
     clear_room(thing);
     break;
   default:
-    do_log(LT_ERR, NOTHING, NOTHING, "Unknown type on #%d in free_object.",
+    do_log(LT_ERR, NOTHING, NOTHING, T("Unknown type on #%d in free_object."),
 	   thing);
     return;
   }
@@ -602,7 +606,8 @@ free_object(thing)
   change_quota(Owner(thing), QUOTA_COST);
 #endif				/* QUOTA */
   do_halt(thing, "", thing);
-  nfy_que(thing, 2, 0);		/* The equivalent of an @drain. */
+  /* The equivalent of an @drain/any/all: */
+  dequeue_semaphores(thing, NULL, INT_MAX, 1, 1);
 
   /* if something is zoned or parented or linked or chained or located
    * to/in destroyed object, undo */
@@ -623,15 +628,15 @@ free_object(thing)
 	/* Huh.  An exit that claims to be from here, but wasn't linked
 	 * in properly. */
 	do_rawlog(LT_ERR,
-		  "ERROR: Exit %s leading from invalid room #%d destroyed.",
-		  real_unparse(GOD, i, 0), thing);
+		  T("ERROR: Exit %s leading from invalid room #%d destroyed."),
+		  unparse_object(GOD, i), thing);
 	free_object(i);
 	break;
       case TYPE_ROOM:
 	/* Hrm.  It claims we're an exit from it, but we didn't agree.
 	 * Clean it up anyway. */
 	do_log(LT_ERR, NOTHING, NOTHING,
-	       "Found a destroyed exit #%d in room #%d", thing, i);
+	       T("Found a destroyed exit #%d in room #%d"), thing, i);
 	break;
       }
     }
@@ -641,7 +646,7 @@ free_object(thing)
       case TYPE_PLAYER:
       case TYPE_THING:
 	/* Huh.  It thought it was here, but we didn't agree. */
-	enter_room(i, Home(i));
+	enter_room(i, Home(i), 0);
 	break;
       case TYPE_EXIT:
 	/* If our destination is destroyed, then we relink to the
@@ -675,18 +680,51 @@ free_object(thing)
   Owner(thing) = GOD;
   Parent(thing) = NOTHING;
   Zone(thing) = NOTHING;
-  Flags(thing) = TYPE_GARBAGE;
 #ifdef CHAT_SYSTEM
   remove_all_obj_chan(thing);
 #endif
+
+  switch (Typeof(thing)) {
+    /* Make absolutely sure we are removed from Location's content or
+       exit list. If we are in a room we own and destroy_possessions
+       is yes, this can happen, causing much ickyness: All garbage
+       items would be in PLAYER_START. */
+  case TYPE_PLAYER:
+  case TYPE_THING:
+    loc = Location(thing);
+    if (GoodObject(loc))
+      Contents(loc) = remove_first(Contents(loc), thing);
+    if (Typeof(thing) == TYPE_THING)
+      current_state.things--;
+    else
+      current_state.players--;
+    break;
+  case TYPE_EXIT:		/* This probably won't be needed, but lets make sure */
+    loc = Source(thing);
+    if (GoodObject(loc))
+      Exits(loc) = remove_first(Exits(loc), thing);
+    current_state.exits--;
+    break;
+  case TYPE_ROOM:
+    current_state.rooms--;
+    break;
+  default:
+    /* Do nothing for rooms. */
+    break;
+  }
+
+  Flags(thing) = TYPE_GARBAGE;
+  Toggles(thing) = Powers(thing) = 0;
   Location(thing) = NOTHING;
-  SET(Name(thing), "Garbage");
+  set_name(thing, "Garbage");
   Exits(thing) = NOTHING;
   Home(thing) = NOTHING;
 
   Next(thing) = first_free;
   first_free = thing;
 
+  current_state.garbage++;
+
 #ifdef HAS_ASSERT
   assert(IsGarbage(thing));
 #endif
@@ -699,8 +737,10 @@ empty_contents(thing)
   /* Destroy any exits they may be carrying, send everything else home. */
   dbref first;
   dbref rest;
+  dbref target;
   notify_except(Contents(thing), NOTHING,
-		"The floor disappears under your feet, you fall through NOTHINGness and then:");
+		T
+		("The floor disappears under your feet, you fall through NOTHINGness and then:"));
   first = Contents(thing);
   Contents(thing) = NOTHING;
   /* send all objects to nowhere */
@@ -717,15 +757,18 @@ empty_contents(thing)
       break;
     case TYPE_THING:		/* move to home */
     case TYPE_PLAYER:
-      if (Home(first) == thing || IsExit(Home(first)))
+      /* Make sure the home is a reasonable object. */
+      if (!GoodObject(Home(first)) || IsExit(Home(first)) ||
+	  Home(first) == thing)
 	Home(first) = PLAYER_START;
-      if (Home(first) != NOTHING) {
-	PUSH(first, Contents(Home(first)));
-	Location(first) = Home(first);
-	/* notify players they have been moved */
-	if (Hearer(first)) {
-	  enter_room(first, HOME);
-	}
+      target = Home(first);
+      /* If home isn't a good place to send it, send it to PLAYER_START. */
+      if (!GoodObject(target) || recursive_member(target, first, 0))
+	target = PLAYER_START;
+      if (target != NOTHING) {
+	/* Use enter_room() on everything so that AENTER and such
+	 * are all triggered properly. */
+	enter_room(first, target, 0);
       }
       break;
     }
@@ -744,16 +787,19 @@ clear_thing(thing)
   if (loc != NOTHING) {
     Contents(loc) = remove_first(Contents(loc), thing);
   }
+  /* Remove object from any following chains */
+  clear_followers(thing, 0);
+  clear_following(thing, 0);
   /* give player money back */
-  giveto(Owner(thing), (a = OBJECT_DEPOSIT(Pennies(thing))));
+  giveto(Owner(thing), (a = Pennies(thing)));
   empty_contents(thing);
   if (Toggles(thing) & THING_PUPPET)
     Toggles(thing) &= ~THING_PUPPET;
   if (!Quiet(thing) && !Quiet(Owner(thing)))
-    notify(Owner(thing),
-	   tprintf("You get your %d %s deposit back for %s.",
-		   a, ((a == 1) ? MONEY : MONIES),
-		   object_header(Owner(thing), thing)));
+    notify_format(Owner(thing),
+		  T("You get your %d %s deposit back for %s."),
+		  a, ((a == 1) ? MONEY : MONIES),
+		  object_header(Owner(thing), thing));
 }
 
 static void
@@ -762,12 +808,15 @@ clear_player(thing)
 {
   dbref i;
   ATTR *atemp;
-  char alias[PLAYER_NAME_LIMIT];
+  char alias[BUFFER_LEN + 1];
 
   /* Clear out mail. */
 #ifdef USE_MAILER
   do_mail_clear(thing, NULL);
   do_mail_purge(thing);
+#ifdef MAIL_ALIASES
+  malias_cleanup(thing);
+#endif
 #endif
 
   /* Chown any chat channels they own to God */
@@ -788,7 +837,7 @@ clear_player(thing)
   for (i = 0; i < db_top; i++) {
     if (Owner(i) == thing && i != thing) {
       if (DESTROY_POSSESSIONS ? (REALLY_SAFE ? Safe(i) : 0) : 1) {
-	chown_object(GOD, i, GOD);
+	chown_object(GOD, i, GOD, 0);
       } else {
 	free_object(i);
       }
@@ -851,19 +900,19 @@ free_get()
     if (nrecur++ == 20) {
       first_free = NOTHING;
       report();
-      do_rawlog(LT_ERR, "ERROR: Removed free list and continued\n");
+      do_rawlog(LT_ERR, T("ERROR: Removed free list and continued\n"));
       return (NOTHING);
     }
     report();
-    do_rawlog(LT_TRACE, "ERROR: Object #%d should not be free\n", newobj);
-    do_rawlog(LT_TRACE, "ERROR: Corrupt free list, fixing\n");
+    do_rawlog(LT_TRACE, T("ERROR: Object #%d should not be free\n"), newobj);
+    do_rawlog(LT_TRACE, T("ERROR: Corrupt free list, fixing\n"));
     fix_free_list();
     temp = free_get();
     nrecur--;
     return (temp);
   }
   /* free object name */
-  SET(Name(newobj), NULL);
+  set_name(newobj, NULL);
   return (newobj);
 }
 
@@ -916,9 +965,9 @@ do_purge(player)
 {
   if (Wizard(player)) {
     purge();
-    notify(player, "Purge complete.");
+    notify(player, T("Purge complete."));
   } else
-    notify(player, "Sorry, you are a mortal.");
+    notify(player, T("Sorry, you are a mortal."));
 }
 
 
@@ -933,6 +982,8 @@ dbck()
   check_contents();
   check_locations();
   check_connected_rooms();
+  check_zones();
+  local_dbck();
 }
 
 /* Do sanity checks on non-destroyed objects. */
@@ -948,8 +999,8 @@ check_fields()
       dbref next;
       next = Next(thing);
       if ((!GoodObject(next) || !IsGarbage(next)) && (next != NOTHING)) {
-	do_rawlog(LT_ERR, "ERROR: Invalid next pointer #%d from object %s",
-		  next, real_unparse(GOD, thing, 0));
+	do_rawlog(LT_ERR, T("ERROR: Invalid next pointer #%d from object %s"),
+		  next, unparse_object(GOD, thing));
 	Next(thing) = NOTHING;
 	fix_free_list();
       }
@@ -965,15 +1016,15 @@ check_fields()
 	Parent(thing) = NOTHING;
       owner = Owner(thing);
       if (!GoodObject(owner) || IsGarbage(owner) || !IsPlayer(owner)) {
-	do_rawlog(LT_ERR, "ERROR: Invalid object owner on %s(%d)", Name(thing),
-		  thing);
+	do_rawlog(LT_ERR, T("ERROR: Invalid object owner on %s(%d)"),
+		  Name(thing), thing);
 	report();
 	Owner(thing) = GOD;
       }
       next = Next(thing);
       if ((!GoodObject(next) || IsGarbage(next)) && (next != NOTHING)) {
-	do_rawlog(LT_ERR, "ERROR: Invalid next pointer #%d from object %s",
-		  next, real_unparse(GOD, thing, 0));
+	do_rawlog(LT_ERR, T("ERROR: Invalid next pointer #%d from object %s"),
+		  next, unparse_object(GOD, thing));
 	Next(thing) = NOTHING;
       }
       /* This next bit has to be type-specific because of different uses
@@ -987,17 +1038,27 @@ check_fields()
 	if (!GoodObject(home) || IsGarbage(home) || IsExit(home))
 	  Home(thing) = PLAYER_START;
 	if (!GoodObject(loc) || IsGarbage(loc) || IsExit(loc))
-	  enter_room(thing, Home(thing));
+	  enter_room(thing, Home(thing), 0);
 	break;
       case TYPE_EXIT:
 	if (Contents(thing) != NOTHING) {
 	  /* Eww.. Exits can't have contents. Bad news */
 	  Contents(thing) = NOTHING;
 	  do_rawlog(LT_ERR,
-		    "ERROR: Exit %s has a contents list. Wiping it out.",
-		    real_unparse(GOD, thing, 0));
+		    T("ERROR: Exit %s has a contents list. Wiping it out."),
+		    unparse_object(GOD, thing));
 	}
-	if (GoodObject(loc) && IsGarbage(loc)) {
+	if (!GoodObject(loc)
+	    && !((loc == NOTHING) || (loc == AMBIGUOUS) || (loc == HOME))) {
+	  /* Bad news. We're linked to a really impossible object.
+	   * Relink to our source
+	   */
+	  Destination(thing) = Source(thing);
+	  do_rawlog(LT_ERR,
+		    T
+		    ("ERROR: Exit %s leading to invalid room #%d relinked to its source room."),
+		    unparse_object(GOD, thing), home);
+	} else if (GoodObject(loc) && IsGarbage(loc)) {
 	  /* If our destination is destroyed, then we relink to the
 	   * source room (so that the exit can't be stolen). Yes, it's
 	   * inconsistent with the treatment of exits leading from
@@ -1006,15 +1067,17 @@ check_fields()
 	   */
 	  Destination(thing) = Source(thing);
 	  do_rawlog(LT_ERR,
-		    "ERROR: Exit %s leading to invalid room #%d relinked to its source room.",
-		    real_unparse(GOD, thing, 0), home);
+		    T
+		    ("ERROR: Exit %s leading to garbage room #%d relinked to its source room."),
+		    unparse_object(GOD, thing), home);
 	}
 	/* This must come last */
 	if (!GoodObject(home) || !IsRoom(home)) {
 	  /* If our source is destroyed, just destroy the exit. */
 	  do_rawlog(LT_ERR,
-		    "ERROR: Exit %s leading from invalid room #%d destroyed.",
-		    real_unparse(GOD, thing, 0), home);
+		    T
+		    ("ERROR: Exit %s leading from invalid room #%d destroyed."),
+		    unparse_object(GOD, thing), home);
 	  free_object(thing);
 	}
 	break;
@@ -1022,7 +1085,7 @@ check_fields()
 	if (GoodObject(home) && IsGarbage(home)) {
 	  /* Eww. Destroyed exit. This isn't supposed to happen. */
 	  do_log(LT_ERR, NOTHING, NOTHING,
-		 "Found a destroyed exit #%d in room #%d", home, thing);
+		 T("Found a destroyed exit #%d in room #%d"), home, thing);
 	}
 	if (GoodObject(loc) && (IsGarbage(loc) || IsExit(loc))) {
 	  /* Just remove a dropto. */
@@ -1041,11 +1104,11 @@ check_fields()
 
 static int
 attribute_owner_helper(player, thing, pattern, atr, args)
-    dbref player;
-    dbref thing;
-    char const *pattern;
+    dbref player __attribute__ ((__unused__));
+    dbref thing __attribute__ ((__unused__));
+    char const *pattern __attribute__ ((__unused__));
     ATTR *atr;
-    void *args;
+    void *args __attribute__ ((__unused__));
 {
   if (!GoodObject(AL_CREATOR(atr)))
     AL_CREATOR(atr) = GOD;
@@ -1080,17 +1143,61 @@ check_connected_marks()
     if (Marked(loc))
       Flags(loc) &= ~MARKED;
     else if (IsRoom(loc)) {
-      if (!Floating(loc) && (!EXITS_CONNECT_ROOMS || (Exits(loc) == NOTHING))) {
-	if (!Name(loc)) {
-	  do_log(LT_ERR, NOTHING, NOTHING, "ERROR: no name for room #%d.", loc);
-	  SET(Name(loc), "XXXX");
-	}
-	notify(Owner(loc), tprintf("You own a disconnected room, %s",
-				   object_header(Owner(loc), loc)));
+      if (!Name(loc)) {
+	do_log(LT_ERR, NOTHING, NOTHING, T("ERROR: no name for room #%d."),
+	       loc);
+	set_name(loc, "XXXX");
+      }
+      if (!Going(loc) && !Floating(loc) &&
+	  (!EXITS_CONNECT_ROOMS || (Exits(loc) == NOTHING))) {
+	notify_format(Owner(loc), T("You own a disconnected room, %s"),
+		      object_header(Owner(loc), loc));
       }
     }
 }
 
+/* Warn about objects without @lock/zone used as zones */
+static void
+check_zones(void)
+{
+  dbref n, zone, tmp;
+
+  for (n = 0; n < db_top; n++) {
+    if (IsGarbage(n))
+      continue;
+    zone = Zone(n);
+    if (!GoodObject(zone))
+      continue;
+    if (ZONE_CONTROL_ZMP && !IsPlayer(zone))
+      continue;
+    if (zone != n)		/* Objects can be zoned to themselves */
+      for (tmp = Zone(zone); GoodObject(tmp); tmp = Zone(tmp)) {
+	if (tmp == n) {
+	  notify_format(Owner(n),
+			T("You own an object in a circular zone chain: %s"),
+			object_header(Owner(n), n));
+	  break;
+	}
+	if (tmp == Zone(tmp))	/* Object zoned to itself */
+	  break;
+      }
+
+    if (Marked(zone))
+      continue;
+    if (getlock(zone, Zone_Lock) == TRUE_BOOLEXP)
+      Flags(zone) |= MARKED;
+  }
+
+  for (n = 0; n < db_top; n++) {
+    if (Marked(n)) {
+      Flags(n) &= ~MARKED;
+      notify_format(Owner(n),
+		    T
+		    ("You own an object without a @lock/zone being used as a zone: %s"),
+		    object_header(Owner(n), n));
+    }
+  }
+}
 
 /* An auxiliary function for check_contents. */
 static void
@@ -1104,15 +1211,15 @@ mark_contents(thing)
   if ((field) != NOTHING) { \
      if (!GoodObject(field) || IsGarbage(field)) { \
        do_rawlog(LT_ERR, "Bad reference #%d from %s severed.", \
-                 (field), real_unparse(GOD, thing, 0)); \
+                 (field), unparse_object(GOD, thing)); \
        (field) = NOTHING; \
      } else if (IsRoom(field)) { \
        do_rawlog(LT_ERR, "Reference to room #%d from %s severed.", \
-                 (field), real_unparse(GOD, thing, 0)); \
+                 (field), unparse_object(GOD, thing)); \
        (field) = NOTHING; \
      } else if (Marked(field)) {  \
        do_rawlog(LT_ERR, "Multiple references to %s. Reference from #%d severed.", \
-                 real_unparse(GOD, (field), 0), thing); \
+                 unparse_object(GOD, (field)), thing); \
        (field) = NOTHING; \
      } else { \
        Flags(field) |= MARKED; \
@@ -1138,8 +1245,8 @@ mark_contents(thing)
     CHECK(Next(thing));
     break;
   default:
-    do_rawlog(LT_ERR, "Bad object type found for %s in mark_contents",
-	      real_unparse(GOD, thing, 0));
+    do_rawlog(LT_ERR, T("Bad object type found for %s in mark_contents"),
+	      unparse_object(GOD, thing));
     break;
   }
 #undef CHECK
@@ -1160,11 +1267,11 @@ check_contents()
   }
   for (thing = 0; thing < db_top; thing++) {
     if (!Marked(thing) && !IsRoom(thing) && !IsGarbage(thing)) {
-      do_rawlog(LT_ERR, "Object %s not pointed to by anything.",
-		real_unparse(GOD, thing, 0));
-      notify(Owner(thing),
-	     tprintf("You own an object %s that was \'orphaned\'.",
-		     object_header(Owner(thing), thing)));
+      do_rawlog(LT_ERR, T("Object %s not pointed to by anything."),
+		unparse_object(GOD, thing));
+      notify_format(Owner(thing),
+		    T("You own an object %s that was \'orphaned\'."),
+		    object_header(Owner(thing), thing));
       /* We try to fix this by trying to send players and things to
        * their current location, to their home, or to PLAYER_START, in
        * that order, and relinking exits to their source.
@@ -1188,39 +1295,37 @@ check_contents()
 	  PUSH(thing, Contents(PLAYER_START));
 	  Location(thing) = PLAYER_START;
 	}
-	enter_room(thing, Location(thing));
+	enter_room(thing, Location(thing), 0);
 	/* If we've managed to reconnect it, then we've reconnected
 	 * its contents. */
 	mark_contents(Contents(thing));
-	notify(Owner(thing), tprintf("It was moved to %s.",
-				     object_header(Owner(thing),
-						   Location(thing))));
-	do_rawlog(LT_ERR, "Moved to %s.",
-		  real_unparse(GOD, Location(thing), 0));
+	notify_format(Owner(thing), T("It was moved to %s."),
+		      object_header(Owner(thing), Location(thing)));
+	do_rawlog(LT_ERR, T("Moved to %s."),
+		  unparse_object(GOD, Location(thing)));
 	break;
       case TYPE_EXIT:
 	if (GoodObject(Source(thing)) && IsRoom(Source(thing))) {
 	  PUSH(thing, Exits(Source(thing)));
-	  notify(Owner(thing), tprintf("It was moved to %s.",
-				       object_header(Owner(thing),
-						     Source(thing))));
-	  do_rawlog(LT_ERR, "Moved to %s.",
-		    real_unparse(GOD, Source(thing), 0));
+	  notify_format(Owner(thing), T("It was moved to %s."),
+			object_header(Owner(thing), Source(thing)));
+	  do_rawlog(LT_ERR, T("Moved to %s."),
+		    unparse_object(GOD, Source(thing)));
 	} else {
 	  /* Just destroy the exit. */
 	  Source(thing) = NOTHING;
-	  notify(Owner(thing), "It was destroyed.");
-	  do_rawlog(LT_ERR, "Orphaned exit destroyed.");
+	  notify(Owner(thing), T("It was destroyed."));
+	  do_rawlog(LT_ERR, T("Orphaned exit destroyed."));
 	  free_object(thing);
 	}
 	break;
       case TYPE_ROOM:
 	/* We should never get here. */
-	do_log(LT_ERR, NOTHING, NOTHING, "Disconnected room. So what?");
+	do_log(LT_ERR, NOTHING, NOTHING, T("Disconnected room. So what?"));
 	break;
       default:
 	do_log(LT_ERR, NOTHING, NOTHING,
-	       "Surprising type on #%d found in check_cycles.", thing);
+	       T("Surprising type on #%d found in check_cycles."), thing);
 	break;
       }
     }
@@ -1244,7 +1349,8 @@ check_locations()
       for (thing = Contents(loc); thing != NOTHING; thing = Next(thing)) {
 	if (!Mobile(thing)) {
 	  do_rawlog(LT_ERR,
-		    "ERROR: Contents of object %d corrupt at object %d cleared",
+		    T
+		    ("ERROR: Contents of object %d corrupt at object %d cleared"),
 		    loc, thing);
 	  /* Remove this from the list and start over. */
 	  Contents(loc) = remove_first(Contents(loc), thing);
@@ -1255,8 +1361,9 @@ check_locations()
 	   * we've done a check_contents already, so let's just put it
 	   * here.
 	   */
-	  do_rawlog(LT_ERR, "Incorrect location on object %s. Reset to #%d.",
-		    real_unparse(GOD, thing, 0), loc);
+	  do_rawlog(LT_ERR,
+		    T("Incorrect location on object %s. Reset to #%d."),
+		    unparse_object(GOD, thing), loc);
 	  Location(thing) = loc;
 	}
 	Flags(thing) |= MARKED;
@@ -1266,7 +1373,7 @@ check_locations()
       for (thing = Exits(loc); thing != NOTHING; thing = Next(thing)) {
 	if (!IsExit(thing)) {
 	  do_rawlog(LT_ERR,
-		    "ERROR: Exits of room %d corrupt at object %d cleared",
+		    T("ERROR: Exits of room %d corrupt at object %d cleared"),
 		    loc, thing);
 	  /* Remove this from the list and start over. */
 	  Exits(loc) = remove_first(Exits(loc), thing);
@@ -1274,8 +1381,8 @@ check_locations()
 	  continue;
 	} else if (Source(thing) != loc) {
 	  do_rawlog(LT_ERR,
-		    "Incorrect source on exit %s. Reset to #%d.",
-		    real_unparse(GOD, thing, 0), loc);
+		    T("Incorrect source on exit %s. Reset to #%d."),
+		    unparse_object(GOD, thing), loc);
 	}
       }
     }
@@ -1286,7 +1393,7 @@ check_locations()
     if (Marked(thing))
       Flags(thing) &= ~MARKED;
     else if (Mobile(thing)) {
-      do_rawlog(LT_ERR, "ERROR DBCK: Moved object %d", thing);
+      do_rawlog(LT_ERR, T("ERROR DBCK: Moved object %d"), thing);
       moveto(thing, PLAYER_START);
     }
 }
@@ -1302,11 +1409,11 @@ do_dbck(player)
     dbref player;
 {
   if (!Wizard(player)) {
-    notify(player, "Silly mortal chicks are for kids!");
+    notify(player, T("Silly mortal chicks are for kids!"));
     return;
   }
-  notify(player, "GAME: Performing database consistency check.");
-  do_log(LT_WIZ, player, NOTHING, "DBCK done.");
+  notify(player, T("GAME: Performing database consistency check."));
+  do_log(LT_WIZ, player, NOTHING, T("DBCK done."));
   dbck();
-  notify(player, "GAME: Database consistency check complete.");
+  notify(player, T("GAME: Database consistency check complete."));
 }
