@@ -14,10 +14,6 @@
 #include "csrimalloc.c"
 #elif (MALLOC_PACKAGE == 2)
 #include "csrimalloc.c"
-#elif (MALLOC_PACKAGE == 3)
-#include "smalloc.c"
-#elif (MALLOC_PACKAGE == 4)
-#include "smalloc.c"
 #elif (MALLOC_PACKAGE == 5)
 #include "gmalloc.c"
 #endif
