@@ -4,30 +4,25 @@
 
 #include "config.h"
 #include <math.h>
-#ifdef I_LIMITS
 #include <limits.h>
-#endif
-#ifdef I_VALUES
-#include <values.h>
-#endif
-#ifdef I_STRING
 #include <string.h>
-#else
-#include <strings.h>
-#endif
 #include <ctype.h>
+#include <signal.h>
+#include <errno.h>
 
 #include "conf.h"
 #include "ansi.h"
 #include "dbdefs.h"
 #include "externs.h"
 #include "function.h"
-#include "globals.h"
-#include "intrface.h"
+#include "case.h"
 #include "match.h"
 #include "mushdb.h"
 #include "parse.h"
-
+#include "attrib.h"
+#include "pcre.h"
+#include "flags.h"
+#include "log.h"
 #ifdef MEM_CHECK
 #include "memcheck.h"
 #endif
@@ -36,32 +31,65 @@
 
 extern char ccom[];		/* bsd.c */
 extern char *absp[], *obj[], *poss[], *subj[];	/* fundb.c */
-
+extern int inum, inum_limit;
 int global_fun_invocations;
 int global_fun_recursions;
+extern int re_subpatterns;
+extern int *re_offsets;
+extern char *re_from;
+extern sig_atomic_t cpu_time_limit_hit;
+extern int cpu_limit_warning_sent;
 
-typedef struct debug_info Debug_Info;
 struct debug_info {
   char *string;
   Debug_Info *prev;
   Debug_Info *next;
 };
 
-struct pe_info {
-  int fun_invocations;
-  int fun_depth;
-  int nest_depth;
-  Debug_Info *debug_strings;
-};
+FUNCTION_PROTO(fun_gfun);
 
 /* Common error messages */
 char e_int[] = "#-1 ARGUMENT MUST BE INTEGER";
 char e_ints[] = "#-1 ARGUMENTS MUST BE INTEGERS";
-#ifdef FLOATING_POINTS
+char e_uint[] = "#-1 ARGUMENT MUST BE POSITIVE INTEGER";
+char e_uints[] = "#-1 ARGUMENTS MUST BE POSITIVE INTEGERS";
 char e_num[] = "#-1 ARGUMENT MUST BE NUMBER";
 char e_nums[] = "#-1 ARGUMENTS MUST BE NUMBERS";
-#endif
 char e_invoke[] = "#-1 FUNCTION INVOCATION LIMIT EXCEEDED";
+char e_call[] = "#-1 CALL LIMIT EXCEEDED";
+char e_perm[] = "#-1 PERMISSION DENIED";
+char e_atrperm[] = "#-1 NO PERMISSION TO GET ATTRIBUTE";
+char e_match[] = "#-1 NO MATCH";
+char e_notvis[] = "#-1 NO SUCH OBJECT VISIBLE";
+char e_disabled[] = "#-1 FUNCTION DISABLED";
+char e_range[] = "#-1 OUT OF RANGE";
+
+#if 0
+static void dummy_errors(void);
+
+static void
+dummy_errors()
+{
+  /* Just to make sure the error messages are in the translation
+     tables. */
+  char *temp;
+  temp = T("#-1 ARGUMENT MUST BE INTEGER");
+  temp = T("#-1 ARGUMENTS MUST BE INTEGERS");
+  temp = T("#-1 ARGUMENT MUST BE POSITIVE INTEGER");
+  temp = T("#-1 ARGUMENTS MUST BE POSITIVE INTEGERS");
+  temp = T("#-1 ARGUMENT MUST BE NUMBER");
+  temp = T("#-1 ARGUMENTS MUST BE NUMBERS");
+  temp = T("#-1 FUNCTION INVOCATION LIMIT EXCEEDED");
+  temp = T("#-1 CALL LIMIT EXCEEDED");
+  temp = T("#-1 PERMISSION DENIED");
+  temp = T("#-1 NO PERMISSION TO GET ATTRIBUTE");
+  temp = T("#-1 NO MATCH");
+  temp = T("#-1 NO SUCH OBJECT VISIBLE");
+  temp = T("#-1 FUNCTION DISABLED");
+  temp = T("#-1 OUT OF RANGE");
+}
+
+#endif
 
 dbref
 parse_dbref(str)
@@ -73,9 +101,9 @@ parse_dbref(str)
   char const *p;
   dbref num;
 
-  if (!str || (*str != NUMBER_TOKEN))
+  if (!str || (*str != NUMBER_TOKEN) || !*(str + 1))
     return NOTHING;
-  for (p = str + 1; isdigit(*p); p++) {
+  for (p = str + 1; isdigit((unsigned char) *p); p++) {
   }
   if (*p)
     return NOTHING;
@@ -124,12 +152,12 @@ int
 is_dbref(str)
     char const *str;
 {
-  if (!str || (*str != NUMBER_TOKEN))
+  if (!str || (*str != NUMBER_TOKEN) || !*(str + 1))
     return 0;
   if (*(str + 1) == '-') {
     str++;
   }
-  for (str++; isdigit(*str); str++) {
+  for (str++; isdigit((unsigned char) *str); str++) {
   }
   return !*str;
 }
@@ -138,27 +166,45 @@ int
 is_integer(str)
     char const *str;
 {
-  char const *p = str;
-  int val;
+  char *end;
+
   /* If we're emulating Tiny, anything is an integer */
-  if (options.tiny_math)
+  if (TINY_MATH)
     return 1;
   if (!str)
     return 0;
-  while (isspace(*str))
+  while (isspace((unsigned char) *str))
     str++;
-  if (!*str)
-    return (NULL_EQ_ZERO ? 1 : 0);
-  if ((*str == '-') && isdigit(str[1]))
-    str += 2;
-  while (isdigit(*str))
-    str++;
-  while (isspace(*str))
-    str++;
-  if (*str)
+  if (*str == '\0')
+    return NULL_EQ_ZERO;
+  errno = 0;
+  strtol(str, &end, 10);
+  if (errno == ERANGE || *end != '\0')
     return 0;
-  val = parse_integer(p);
-  if ((val > HUGE_INT) || (-val > HUGE_INT))
+  return 1;
+}
+
+int
+is_uinteger(str)
+    char const *str;
+{
+  char *end;
+
+  /* If we're emulating Tiny, anything is an integer */
+  if (TINY_MATH)
+    return 1;
+  if (!str)
+    return 0;
+  /* strtoul() accepts negative numbers, so we still have to do this check */
+  while (isspace((unsigned char) *str))
+    str++;
+  if (*str == '\0')
+    return NULL_EQ_ZERO;
+  if (!(isdigit((unsigned char) *str) || *str == '+'))
+    return 0;
+  errno = 0;
+  strtoul(str, &end, 10);
+  if (errno == ERANGE || *end != '\0')
     return 0;
   return 1;
 }
@@ -167,31 +213,30 @@ int
 is_strict_number(str)
     char const *str;
 {
-  char const *p = str;
+  char *end;
   NVAL val;
-
   if (!str)
     return 0;
-  while (isspace(*str))
-    str++;
-  if (!*str)
-    return (NULL_EQ_ZERO ? 1 : 0);
-  if (((*str == '-') || (*str == '+')) && str[1])
-    str++;
-  while (isdigit(*str))
-    str++;
-  if (*str == '.')
-    str++;
-  while (isdigit(*str))
-    str++;
-  while (isspace(*str))
-    str++;
-  if (*str)
+  errno = 0;
+  val = strtod(str, &end);
+  if (errno == ERANGE || *end != '\0')
     return 0;
-  val = parse_number(p);
-  if ((val > HUGE_NVAL) || (-val > HUGE_NVAL))
+  return end > str;
+}
+
+int
+is_strict_integer(str)
+    const char *str;
+{
+  char *end;
+  int val;
+  if (!str)
     return 0;
-  return 1;
+  errno = 0;
+  val = strtol(str, &end, 10);
+  if (errno == ERANGE || *end != '\0')
+    return 0;
+  return end > str;
 }
 
 int
@@ -199,38 +244,25 @@ is_number(str)
     char const *str;
 {
   /* If we're emulating Tiny, anything is a number */
-  if (options.tiny_math)
+  if (TINY_MATH)
     return 1;
+  while (isspace((unsigned char) *str))
+    str++;
+  if (*str == '\0')
+    return NULL_EQ_ZERO;
   return is_strict_number(str);
 }
 
 /* Table of interesting characters for process_expression() */
-static char active_table[UCHAR_MAX + 1];
-
-/* Function to initialize table above */
-void
-init_process_expression()
-{
-  active_table['\0'] = 1;
-  active_table['%'] = 1;
-  active_table['{'] = 1;
-  active_table['['] = 1;
-  active_table['('] = 1;
-  active_table['\\'] = 1;
-  active_table[' '] = 1;
-  active_table['}'] = 1;
-  active_table[']'] = 1;
-  active_table[')'] = 1;
-  active_table[','] = 1;
-  active_table[';'] = 1;
-  active_table['='] = 1;
-}
+extern char active_table[UCHAR_MAX + 1];
+/* Indexes of valid q-regs into the renv array. -1 is error. */
+extern signed char qreg_indexes[UCHAR_MAX + 1];
 
 #ifdef WIN32
 #pragma warning( disable : 4761)	/* NJG: disable warning re conversion */
 #endif
 /* Function and other substitution evaluation. */
-void
+int
 process_expression(buff, bp, str, executor, caller, enactor,
 		   eflags, tflags, pe_info)
     char *buff;
@@ -251,16 +283,51 @@ process_expression(buff, bp, str, executor, caller, enactor,
   int had_space = 0;
   char temp[3];
   int temp_eflags;
+  int old_iter_limit;
+  int qindex;
+  int e_len;
+  int retval = 0;
+  const char *e_msg;
 
   if (!buff || !bp || !str || !*str)
-    return;
+    return 0;
+  if (cpu_time_limit_hit) {
+    if (!cpu_limit_warning_sent) {
+      cpu_limit_warning_sent = 1;
+      /* Can't just put #-1 CPU USAGE EXCEEDED in buff here, because
+       * it might never get displayed.
+       */
+      if (!Quiet(enactor))
+	notify(enactor, T("CPU usage exceeded."));
+      do_rawlog(LT_TRACE,
+		T
+		("CPU time limit exceeded. enactor=#%d executor=#%d caller=#%d code=%s"),
+		enactor, executor, caller, *str);
+    }
+    return 1;
+  }
   if (Halted(executor))
     eflags = PE_NOTHING;
   if (eflags & PE_COMPRESS_SPACES)
     while (**str == ' ')
       (*str)++;
   if (!*str)
-    return;
+    return 0;
+
+  if (!pe_info) {
+    old_iter_limit = inum_limit;
+    inum_limit = inum;
+    made_info = 1;
+    pe_info = (PE_Info *) mush_malloc(sizeof(PE_Info),
+				      "process_expression.pe_info");
+    pe_info->fun_invocations = 0;
+    pe_info->fun_depth = 0;
+    pe_info->nest_depth = 0;
+    pe_info->call_depth = 0;
+    pe_info->debug_strings = NULL;
+  } else {
+    old_iter_limit = -1;
+  }
 
   if (eflags != PE_NOTHING) {
     if (((*bp) - buff) > (BUFFER_LEN - SBUF_LEN)) {
@@ -271,6 +338,18 @@ process_expression(buff, bp, str, executor, caller, enactor,
       *bp = buff;
       startpos = buff;
     }
+  }
+
+  if (CALL_LIMIT && (pe_info->call_depth++ > CALL_LIMIT)) {
+    e_msg = T(e_call);
+    e_len = strlen(e_msg);
+    if ((buff + e_len > *bp) || strcmp(e_msg, *bp - e_len))
+      safe_str(e_msg, buff, bp);
+    goto exit_sequence;
+  }
+
+
+  if (eflags != PE_NOTHING) {
     debugging = Debug(executor) && Connected(Owner(executor));
     if (debugging) {
       int j;
@@ -278,19 +357,10 @@ process_expression(buff, bp, str, executor, caller, enactor,
       char const *mark;
       Debug_Info *node;
 
-      if (!pe_info) {
-	made_info = 1;
-	pe_info = (PE_Info *) mush_malloc(sizeof(PE_Info),
-					  "process_expression.pe_info");
-	pe_info->fun_invocations = 0;
-	pe_info->fun_depth = 0;
-	pe_info->nest_depth = 0;
-	pe_info->debug_strings = NULL;
-      }
       debugstr = (char *) mush_malloc(BUFFER_LEN,
 				      "process_expression.debug_source");
       debugp = debugstr;
-      safe_str(unparse_dbref(executor), debugstr, &debugp);
+      safe_dbref(executor, debugstr, &debugp);
       safe_chr('!', debugstr, &debugp);
       for (j = 0; j <= pe_info->nest_depth; j++)
 	safe_chr(' ', debugstr, &debugp);
@@ -315,6 +385,11 @@ process_expression(buff, bp, str, executor, caller, enactor,
       pe_info->nest_depth++;
     }
   }
+
+  /* Only strip command braces if the first character is a brace. */
+  if (**str != '{')
+    eflags &= ~PE_COMMAND_BRACES;
+
   for (;;) {
     /* Find the first "interesting" character */
     {
@@ -371,8 +446,46 @@ process_expression(buff, bp, str, executor, caller, enactor,
     }
 
     switch (**str) {
+    case 0x1B:			/* ANSI escapes. */
+      /* Skip over until the 'm' that matches the end. */
+      for (; *str && **str && **str != 'm'; (*str)++)
+	safe_chr(**str, buff, bp);
+      if (*str && **str) {
+	safe_chr(**str, buff, bp);
+	(*str)++;
+      }
+      break;
+    case '$':			/* Dollar subs for regedit() */
+      if ((eflags & (PE_DOLLAR | PE_EVALUATE)) != (PE_DOLLAR | PE_EVALUATE)) {
+	safe_chr('$', buff, bp);
+	(*str)++;
+      } else {
+	char obuf[BUFFER_LEN];
+	int p = 0;
+
+	(*str)++;
+	/* Check the first two characters after the $ for a number */
+	if (isdigit((unsigned char) **str)) {
+	  p = **str - '0';
+	  (*str)++;
+	  if (isdigit((unsigned char) **str)) {
+	    p *= 10;
+	    p += **str - '0';
+	    (*str)++;
+	  }
+	} else
+	  break;
+
+	if (p >= re_subpatterns || re_offsets == NULL || re_from == NULL)
+	  break;
+
+	pcre_copy_substring(re_from, re_offsets, re_subpatterns, p, obuf,
+			    BUFFER_LEN);
+	safe_str(obuf, buff, bp);
+      }
+      break;
     case '%':			/* Percent substitutions */
-      if (!(eflags & PE_EVALUATE)) {
+      if (!(eflags & PE_EVALUATE) || (*bp - buff >= BUFFER_LEN - 1)) {
 	/* peak -- % escapes (at least) one character */
 	char savec;
 
@@ -383,6 +496,22 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	  goto exit_sequence;
 	safe_chr(savec, buff, bp);
 	(*str)++;
+	switch (savec) {
+	case 'Q':
+	case 'q':
+	case 'V':
+	case 'v':
+	case 'W':
+	case 'w':
+	case 'X':
+	case 'x':
+	  /* These sequences escape two characters */
+	  savec = **str;
+	  if (!savec)
+	    goto exit_sequence;
+	  safe_chr(savec, buff, bp);
+	  (*str)++;
+	}
 	break;
       } else {
 	char savec, nextc;
@@ -401,19 +530,19 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	  safe_chr('%', buff, bp);
 	  break;
 	case '!':		/* executor dbref */
-	  safe_str(unparse_dbref(executor), buff, bp);
+	  safe_dbref(executor, buff, bp);
 	  break;
 	case '@':		/* caller dbref */
-	  safe_str(unparse_dbref(caller), buff, bp);
+	  safe_dbref(caller, buff, bp);
 	  break;
 	case '#':		/* enactor dbref */
-	  safe_str(unparse_dbref(enactor), buff, bp);
+	  safe_dbref(enactor, buff, bp);
 	  break;
 	case '?':		/* function limits */
 	  if (pe_info) {
-	    safe_str(unparse_integer(pe_info->fun_invocations), buff, bp);
+	    safe_integer(pe_info->fun_invocations, buff, bp);
 	    safe_chr(' ', buff, bp);
-	    safe_str(unparse_integer(pe_info->fun_depth), buff, bp);
+	    safe_integer(pe_info->fun_depth, buff, bp);
 	  } else {
 	    safe_str("0 0", buff, bp);
 	  }
@@ -449,7 +578,7 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	case 'l':		/* enactor location dbref */
 	  /* The security implications of this have
 	   * already been talked to death.  Deal. */
-	  safe_str(unparse_dbref(Location(enactor)), buff, bp);
+	  safe_dbref(Location(enactor), buff, bp);
 	  break;
 	case 'N':
 	case 'n':		/* enactor name */
@@ -473,14 +602,17 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	  if (!nextc)
 	    goto exit_sequence;
 	  (*str)++;
-	  if (!isdigit(nextc))
+	  if ((qindex = qreg_indexes[(unsigned char) nextc]) == -1)
 	    break;
-	  if (renv[nextc - '0'])
-	    safe_str(renv[nextc - '0'], buff, bp);
+	  if (renv[qindex])
+	    safe_str(renv[qindex], buff, bp);
 	  break;
 	case 'R':
 	case 'r':		/* newline */
-	  safe_str("\r\n", buff, bp);
+	  if (NEWLINE_ONE_CHAR)
+	    safe_chr('\n', buff, bp);
+	  else
+	    safe_str("\r\n", buff, bp);
 	  break;
 	case 'S':
 	case 's':		/* enactor subjective pronoun */
@@ -513,49 +645,46 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	  safe_chr(savec, buff, bp);
 	}
 
-	if (isupper(savec))
+	if (isupper((unsigned char) savec))
 	  *savepos = UPCASE(*savepos);
       }
       break;
-
     case '{':			/* "{}" parse group; recurse with no function check */
-      if (!pe_info && eflags != PE_NOTHING) {
-	made_info = 1;
-	pe_info = (PE_Info *) mush_malloc(sizeof(PE_Info),
-					  "process_expression.pe_info");
-	pe_info->fun_invocations = 0;
-	pe_info->fun_depth = 0;
-	pe_info->nest_depth = 0;
-	pe_info->debug_strings = NULL;
+      if (CALL_LIMIT && (pe_info->call_depth > CALL_LIMIT)) {
+	(*str)++;
+	break;
       }
       if (eflags & PE_LITERAL) {
 	safe_chr('{', buff, bp);
 	(*str)++;
 	break;
       }
-      if (!(eflags & PE_STRIP_BRACES))
+      if (!(eflags & (PE_STRIP_BRACES | PE_COMMAND_BRACES)))
 	safe_chr('{', buff, bp);
       (*str)++;
-      process_expression(buff, bp, str,
-			 executor, caller, enactor,
-			 eflags & ~(PE_STRIP_BRACES | PE_FUNCTION_CHECK),
-			 PT_BRACE, pe_info);
+      if (process_expression(buff, bp, str,
+			     executor, caller, enactor,
+			     eflags & PE_COMMAND_BRACES
+			     ? (eflags & ~PE_COMMAND_BRACES)
+			     : (eflags &
+				~(PE_STRIP_BRACES | PE_FUNCTION_CHECK)),
+			     PT_BRACE, pe_info)) {
+	retval = 1;
+	break;
+      }
+
       if (**str == '}') {
-	if (!(eflags & PE_STRIP_BRACES))
+	if (!(eflags & (PE_STRIP_BRACES | PE_COMMAND_BRACES)))
 	  safe_chr('}', buff, bp);
 	(*str)++;
       }
+      /* Only strip one set of braces for commands */
+      eflags &= ~PE_COMMAND_BRACES;
       break;
-
     case '[':			/* "[]" parse group; recurse with mandatory function check */
-      if (!pe_info && eflags != PE_NOTHING) {
-	made_info = 1;
-	pe_info = (PE_Info *) mush_malloc(sizeof(PE_Info),
-					  "process_expression.pe_info");
-	pe_info->fun_invocations = 0;
-	pe_info->fun_depth = 0;
-	pe_info->nest_depth = 0;
-	pe_info->debug_strings = NULL;
+      if (CALL_LIMIT && (pe_info->call_depth > CALL_LIMIT)) {
+	(*str)++;
+	break;
       }
       if (eflags & PE_LITERAL) {
 	safe_chr('[', buff, bp);
@@ -568,32 +697,30 @@ process_expression(buff, bp, str, executor, caller, enactor,
       } else
 	temp_eflags = eflags | PE_FUNCTION_CHECK | PE_FUNCTION_MANDATORY;
       (*str)++;
-      process_expression(buff, bp, str,
-			 executor, caller, enactor,
-			 temp_eflags, PT_BRACKET, pe_info);
+      if (process_expression(buff, bp, str,
+			     executor, caller, enactor,
+			     temp_eflags, PT_BRACKET, pe_info)) {
+	retval = 1;
+	break;
+      }
       if (**str == ']') {
 	if (!(eflags & PE_EVALUATE))
 	  safe_chr(']', buff, bp);
 	(*str)++;
       }
       break;
-
     case '(':			/* Function call */
-      if (!pe_info && eflags != PE_NOTHING) {
-	made_info = 1;
-	pe_info = (PE_Info *) mush_malloc(sizeof(PE_Info),
-					  "process_expression.pe_info");
-	pe_info->fun_invocations = 0;
-	pe_info->fun_depth = 0;
-	pe_info->nest_depth = 0;
-	pe_info->debug_strings = NULL;
+      if (CALL_LIMIT && (pe_info->call_depth > CALL_LIMIT)) {
+	(*str)++;
+	break;
       }
       (*str)++;
       if (!(eflags & PE_EVALUATE) || !(eflags & PE_FUNCTION_CHECK)) {
 	safe_chr('(', buff, bp);
-	process_expression(buff, bp, str,
-			   executor, caller, enactor,
-			   eflags & ~PE_STRIP_BRACES, PT_PAREN, pe_info);
+	if (process_expression(buff, bp, str,
+			       executor, caller, enactor,
+			       eflags & ~PE_STRIP_BRACES, PT_PAREN, pe_info))
+	  retval = 1;
 	if (**str == ')') {
 	  safe_chr(')', buff, bp);
 	  (*str)++;
@@ -602,50 +729,54 @@ process_expression(buff, bp, str, executor, caller, enactor,
       } else {
 	char *sargs[10];
 	char **fargs;
+	int sarglens[10];
+	int *arglens;
 	int args_alloced;
 	int nfargs;
 	int j;
-	char *name;
+	static char name[BUFFER_LEN];
 	char *sp, *tp;
 	FUN *fp;
 	int temp_tflags;
+	int denied;
 
 	fargs = sargs;
-	for (j = 0; j < 10; j++)
+	arglens = sarglens;
+	for (j = 0; j < 10; j++) {
 	  fargs[j] = NULL;
+	  arglens[j] = 0;
+	}
 	args_alloced = 10;
-
 	eflags &= ~PE_FUNCTION_CHECK;
-
 	/* Get the function name */
-	name = (char *) mush_malloc(BUFFER_LEN,
-				    "process_expression.function_name");
 	for (sp = startpos, tp = name; sp < *bp; sp++)
 	  safe_chr(UPCASE(*sp), name, &tp);
 	*tp = '\0';
-
 	fp = func_hash_lookup(name);
 	if (!fp) {
 	  if (eflags & PE_FUNCTION_MANDATORY) {
 	    *bp = startpos;
-	    safe_str("#-1 FUNCTION (", buff, bp);
+	    safe_str(T("#-1 FUNCTION ("), buff, bp);
 	    safe_str(name, buff, bp);
 	    safe_str(") NOT FOUND", buff, bp);
-	    process_expression(name, &tp, str,
-			       executor, caller, enactor,
-			       PT_NOTHING, PT_PAREN, pe_info);
+	    if (process_expression(name, &tp, str,
+				   executor, caller, enactor,
+				   PE_NOTHING, PT_PAREN, pe_info))
+	      retval = 1;
 	    if (**str == ')')
 	      (*str)++;
-	    mush_free((Malloc_t) name, "process_expression.function_name");
-	    goto exit_sequence;
+	    break;
 	  }
-	  mush_free((Malloc_t) name, "process_expression.function_name");
 	  safe_chr('(', buff, bp);
-	  if (**str == ' ')
-	    safe_chr(*(*str)++, buff, bp);
-	  process_expression(buff, bp, str,
-			     executor, caller, enactor,
-			     eflags, PT_PAREN, pe_info);
+	  if (**str == ' ') {
+	    safe_chr(**str, buff, bp);
+	    (*str)++;
+	  }
+	  if (process_expression(buff, bp, str, executor, caller, enactor,
+				 eflags, PT_PAREN, pe_info)) {
+	    retval = 1;
+	    break;
+	  }
 	  if (**str == ')') {
 	    if (eflags & PE_COMPRESS_SPACES && (*str)[-1] == ' ')
 	      safe_chr(' ', buff, bp);
@@ -654,14 +785,40 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	  }
 	  break;
 	}
-	mush_free((Malloc_t) name, "process_expression.function_name");
 	*bp = startpos;
 
+	/* Check for the invocation limit */
+	if ((pe_info->fun_invocations >= FUNCTION_LIMIT) ||
+	    (global_fun_invocations >= FUNCTION_LIMIT * 5)) {
+	  e_msg = T(e_invoke);
+	  e_len = strlen(e_msg);
+	  if ((buff + e_len > *bp) || strcmp(e_msg, *bp - e_len))
+	    safe_str(e_msg, buff, bp);
+	  if (process_expression(name, &tp, str,
+				 executor, caller, enactor,
+				 PE_NOTHING, PT_PAREN, pe_info))
+	    retval = 1;
+	  if (**str == ')')
+	    (*str)++;
+	  break;
+	}
+	/* Check for the recursion limit */
+	if ((pe_info->fun_depth + 1 >= RECURSION_LIMIT) ||
+	    (global_fun_recursions + 1 >= RECURSION_LIMIT * 5)) {
+	  safe_str(T("#-1 FUNCTION RECURSION LIMIT EXCEEDED"), buff, bp);
+	  if (process_expression(name, &tp, str,
+				 executor, caller, enactor,
+				 PE_NOTHING, PT_PAREN, pe_info))
+	    retval = 1;
+	  if (**str == ')')
+	    (*str)++;
+	  break;
+	}
 	/* Get the arguments */
 	temp_eflags = (eflags & ~PE_FUNCTION_MANDATORY)
 	  | PE_COMPRESS_SPACES | PE_EVALUATE | PE_FUNCTION_CHECK;
-	if (fp->fun != fun_gfun) {
-	  switch (fp->ftype) {
+	if (fp->flags & FN_BUILTIN) {
+	  switch (fp->flags & FN_ARG_MASK) {
 	  case FN_LITERAL:
 	    temp_eflags |= PE_LITERAL;
 	    /* FALL THROUGH */
@@ -671,120 +828,142 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	    break;
 	  }
 	}
+	denied = !check_func(executor, fp);
+	if (denied)
+	  temp_eflags &=
+	    ~(PE_COMPRESS_SPACES | PE_EVALUATE | PE_FUNCTION_CHECK);
 	temp_tflags = PT_COMMA | PT_PAREN;
 	nfargs = 0;
 	do {
 	  char *argp;
-
 	  if ((fp->maxargs < 0) && ((nfargs + 1) >= -fp->maxargs))
 	    temp_tflags = PT_PAREN;
-
 	  if (nfargs >= args_alloced) {
 	    char **nargs;
+	    int *narglens;
 	    nargs = (char **) mush_malloc((nfargs + 10) * sizeof(char *),
 					  "process_expression.function_arglist");
-	    for (j = 0; j < nfargs; j++)
+	    narglens = (int *) mush_malloc((nfargs + 10) * sizeof(int),
+					   "process_expression.function_arglens");
+	    for (j = 0; j < nfargs; j++) {
 	      nargs[j] = fargs[j];
+	      narglens[j] = arglens[j];
+	    }
 	    if (fargs != sargs)
 	      mush_free((Malloc_t) fargs,
 			"process_expression.function_arglist");
+	    if (arglens != sarglens)
+	      mush_free((Malloc_t) arglens,
+			"process_expression.function_arglens");
 	    fargs = nargs;
+	    arglens = narglens;
 	    args_alloced += 10;
 	  }
 	  fargs[nfargs] = (char *) mush_malloc(BUFFER_LEN,
 					       "process_expression.function_argument");
 	  argp = fargs[nfargs];
-	  process_expression(fargs[nfargs], &argp, str,
-			     executor, caller, enactor,
-			     temp_eflags, temp_tflags, pe_info);
+	  if (process_expression(fargs[nfargs], &argp, str,
+				 executor, caller, enactor,
+				 temp_eflags, temp_tflags, pe_info)) {
+	    retval = 1;
+	    nfargs++;
+	    goto free_func_args;
+	  }
 	  *argp = '\0';
+	  arglens[nfargs] = argp - fargs[nfargs];
 	  (*str)++;
 	  nfargs++;
 	} while ((*str)[-1] == ',');
 	if ((*str)[-1] != ')')
 	  (*str)--;
-
-	/* If we have the right number of args, eval the function.
-	 * Otherwise, return an error message.
-	 * Special case: zero args is recognized as one null arg.
-	 */
-	if ((fp->minargs == 0) && (nfargs == 1) && !*fargs[0]) {
-	  mush_free((Malloc_t) fargs[0],
-		    "process_expression.function_argument");
-	  fargs[0] = NULL;
-	  nfargs = 0;
-	}
-	if ((nfargs < fp->minargs) || (nfargs > abs(fp->maxargs))) {
-	  *bp = buff;
-	  safe_str("#-1 FUNCTION (", buff, bp);
-	  safe_str(fp->name, buff, bp);
-	  safe_str(") EXPECTS ", buff, bp);
-	  if (fp->minargs == abs(fp->maxargs)) {
-	    safe_str(unparse_integer(fp->minargs), buff, bp);
-	  } else if ((fp->minargs + 1) == abs(fp->maxargs)) {
-	    safe_str(unparse_integer(fp->minargs), buff, bp);
-	    safe_str(" OR ", buff, bp);
-	    safe_str(unparse_integer(abs(fp->maxargs)), buff, bp);
-	  } else if (fp->maxargs == INT_MAX) {
-	    safe_str("AT LEAST ", buff, bp);
-	    safe_str(unparse_integer(fp->minargs), buff, bp);
-	  } else {
-	    safe_str("BETWEEN ", buff, bp);
-	    safe_str(unparse_integer(fp->minargs), buff, bp);
-	    safe_str(" AND ", buff, bp);
-	    safe_str(unparse_integer(abs(fp->maxargs)), buff, bp);
-	  }
-	  safe_str(" ARGUMENTS", buff, bp);
+	/* See if this function is enabled */
+	/* Can't do this check earlier, because of possible side effects
+	 * from the functions.  Bah. */
+	if (denied) {
+	  if (fp->flags & FN_DISABLED)
+	    safe_str(T(e_disabled), buff, bp);
+	  else
+	    safe_str(T(e_perm), buff, bp);
+	  goto free_func_args;
 	} else {
-	  global_fun_invocations++;
-	  global_fun_recursions++;
-	  pe_info->fun_invocations++;
-	  pe_info->fun_depth++;
-	  if ((pe_info->fun_invocations >= options.func_invk_lim) ||
-	      (global_fun_invocations >= options.func_invk_lim)) {
-	    int e_len = strlen(e_invoke);
-	    if ((buff + e_len > *bp) || strcmp(e_invoke, *bp - e_len))
-	      safe_str(e_invoke, buff, bp);
-	  } else if ((pe_info->fun_depth >= options.func_nest_lim) ||
-		     (global_fun_recursions >= options.func_nest_lim)) {
-	    safe_str("#-1 FUNCTION RECURSION LIMIT EXCEEDED", buff, bp);
-	  } else if (fp->fun != fun_gfun)
-	    fp->fun(buff, bp, nfargs, fargs, executor, caller, enactor,
-		    fp->name, pe_info);
-	  else {
-	    dbref thing;
-	    ATTR *attrib;
-	    thing = userfn_tab[GF_Index(fp->ftype)].thing;
-	    attrib = atr_get(thing, userfn_tab[GF_Index(fp->ftype)].name);
-	    if (!attrib) {
-	      do_rawlog(LT_ERR,
-			"ERROR: @function (%s) without attribute (#%d/%s)",
-			fp->name, thing, userfn_tab[GF_Index(fp->ftype)].name);
-	      safe_str("#-1 @FUNCTION (", buff, bp);
-	      safe_str(fp->name, buff, bp);
-	      safe_str(") MISSING ATTRIBUTE (", buff, bp);
-	      safe_str(unparse_dbref(thing), buff, bp);
-	      safe_chr('/', buff, bp);
-	      safe_str(userfn_tab[GF_Index(fp->ftype)].name, buff, bp);
-	      safe_chr(')', buff, bp);
-	    } else
-	      do_userfn(buff, bp, thing, attrib, nfargs, fargs,
-			executor, caller, enactor, pe_info);
+	  /* If we have the right number of args, eval the function.
+	   * Otherwise, return an error message.
+	   * Special case: zero args is recognized as one null arg.
+	   */
+	  if ((fp->minargs == 0) && (nfargs == 1) && !*fargs[0]) {
+	    mush_free((Malloc_t) fargs[0],
+		      "process_expression.function_argument");
+	    fargs[0] = NULL;
+	    arglens[0] = 0;
+	    nfargs = 0;
+	  }
+	  if ((nfargs < fp->minargs) || (nfargs > abs(fp->maxargs))) {
+	    safe_str(T("#-1 FUNCTION ("), buff, bp);
+	    safe_str(fp->name, buff, bp);
+	    safe_str(") EXPECTS ", buff, bp);
+	    if (fp->minargs == abs(fp->maxargs)) {
+	      safe_integer(fp->minargs, buff, bp);
+	    } else if ((fp->minargs + 1) == abs(fp->maxargs)) {
+	      safe_integer(fp->minargs, buff, bp);
+	      safe_str(" OR ", buff, bp);
+	      safe_integer(abs(fp->maxargs), buff, bp);
+	    } else if (fp->maxargs == INT_MAX) {
+	      safe_str("AT LEAST ", buff, bp);
+	      safe_integer(fp->minargs, buff, bp);
+	    } else {
+	      safe_str("BETWEEN ", buff, bp);
+	      safe_integer(fp->minargs, buff, bp);
+	      safe_str(" AND ", buff, bp);
+	      safe_integer(abs(fp->maxargs), buff, bp);
+	    }
+	    safe_str(" ARGUMENTS BUT GOT ", buff, bp);
+	    safe_integer(nfargs, buff, bp);
+	  } else {
+	    global_fun_recursions++;
+	    pe_info->fun_depth++;
+	    if (fp->flags & FN_BUILTIN) {
+	      global_fun_invocations++;
+	      pe_info->fun_invocations++;
+	      fp->where.fun(fp, buff, bp, nfargs, fargs, arglens, executor,
+			    caller, enactor, fp->name, pe_info);
+	    } else {
+	      dbref thing;
+	      ATTR *attrib;
+	      global_fun_invocations++;
+	      pe_info->fun_invocations++;
+	      thing = userfn_tab[fp->where.offset].thing;
+	      attrib = atr_get(thing, userfn_tab[fp->where.offset].name);
+	      if (!attrib) {
+		do_rawlog(LT_ERR,
+			  T("ERROR: @function (%s) without attribute (#%d/%s)"),
+			  fp->name, thing, userfn_tab[fp->where.offset].name);
+		safe_str("#-1 @FUNCTION (", buff, bp);
+		safe_str(fp->name, buff, bp);
+		safe_str(") MISSING ATTRIBUTE (", buff, bp);
+		safe_dbref(thing, buff, bp);
+		safe_chr('/', buff, bp);
+		safe_str(userfn_tab[fp->where.offset].name, buff, bp);
+		safe_chr(')', buff, bp);
+	      } else
+		do_userfn(buff, bp, thing, attrib, nfargs, fargs,
+			  executor, caller, enactor, pe_info);
+	    }
+	    pe_info->fun_depth--;
+	    global_fun_recursions--;
 	  }
-	  pe_info->fun_depth--;
-	  global_fun_recursions--;
 	}
-
 	/* Free up the space allocated for the args */
+      free_func_args:
 	for (j = 0; j < nfargs; j++)
 	  if (fargs[j])
 	    mush_free((Malloc_t) fargs[j],
 		      "process_expression.function_argument");
 	if (fargs != sargs)
 	  mush_free((Malloc_t) fargs, "process_expression.function_arglist");
+	if (arglens != sarglens)
+	  mush_free((Malloc_t) arglens, "process_expression.function_arglens");
       }
       break;
-
       /* Space compression */
     case ' ':
       had_space = 1;
@@ -799,7 +978,6 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	  (*str)++;
 	}
       break;
-
       /* Escape charater */
     case '\\':
       if (!(eflags & PE_EVALUATE))
@@ -808,7 +986,6 @@ process_expression(buff, bp, str, executor, caller, enactor,
       if (!**str)
 	goto exit_sequence;
       /* FALL THROUGH */
-
       /* Basic character */
     default:
       safe_chr(**str, buff, bp);
@@ -822,17 +999,21 @@ exit_sequence:
     if ((eflags & PE_COMPRESS_SPACES) && had_space &&
 	((*str)[-1] == ' ') && ((*bp)[-1] == ' '))
       (*bp)--;
-
     if (debugging) {
       pe_info->nest_depth--;
       **bp = '\0';
       if (strcmp(sourcestr, startpos)) {
+	static char dbuf[BUFFER_LEN];
+	char *dbp;
 	if (pe_info->debug_strings) {
 	  while (pe_info->debug_strings->prev)
 	    pe_info->debug_strings = pe_info->debug_strings->prev;
 	  while (pe_info->debug_strings->next) {
-	    raw_notify(Owner(executor),
-		       tprintf("%s :", pe_info->debug_strings->string));
+	    dbp = dbuf;
+	    dbuf[0] = '\0';
+	    safe_format(dbuf, &dbp, "%s :", pe_info->debug_strings->string);
+	    *dbp = '\0';
+	    raw_notify(Owner(executor), dbuf);
 	    pe_info->debug_strings = pe_info->debug_strings->next;
 	    mush_free((Malloc_t) pe_info->debug_strings->prev,
 		      "process_expression.debug_node");
@@ -841,7 +1022,11 @@ exit_sequence:
 		    "process_expression.debug_node");
 	  pe_info->debug_strings = NULL;
 	}
-	raw_notify(Owner(executor), tprintf("%s => %s", debugstr, startpos));
+	dbp = dbuf;
+	dbuf[0] = '\0';
+	safe_format(dbuf, &dbp, "%s => %s", debugstr, startpos);
+	*dbp = '\0';
+	raw_notify(Owner(executor), dbuf);
       } else {
 	Debug_Info *node;
 	node = pe_info->debug_strings;
@@ -854,8 +1039,6 @@ exit_sequence:
       }
       mush_free((Malloc_t) debugstr, "process_expression.debug_source");
     }
-    if (made_info)
-      mush_free((Malloc_t) pe_info, "process_expression.pe_info");
     if (realbuff) {
       **bp = '\0';
       *bp = realbp;
@@ -863,7 +1046,17 @@ exit_sequence:
       mush_free((Malloc_t) buff, "process_expression.buffer_extension");
     }
   }
+  /* Once we cross call limit, we stay in error */
+  if (pe_info && CALL_LIMIT && pe_info->call_depth <= CALL_LIMIT)
+    pe_info->call_depth--;
+  if (made_info)
+    mush_free((Malloc_t) pe_info, "process_expression.pe_info");
+  if (old_iter_limit != -1) {
+    inum_limit = old_iter_limit;
+  }
+  return retval;
 }
+
 #ifdef WIN32
 #pragma warning( default : 4761)	/* NJG: enable warning re conversion */
 #endif
