@@ -6,17 +6,9 @@
 /* Predicates for testing various conditions */
 
 #include <stdio.h>
-#ifdef I_STDARG
 #include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 #include <ctype.h>
-#ifdef I_STRING
 #include <string.h>
-#else
-#include <strings.h>
-#endif
 #ifdef I_SYS_TYPES
 #include <sys/types.h>
 #endif
@@ -25,98 +17,86 @@
 #else
 #include <time.h>
 #endif
-#ifdef I_STDLIB
 #include <stdlib.h>
-#endif
 
 #include "conf.h"
 #include "externs.h"
 #include "mushdb.h"
-#include "intrface.h"
-#include "globals.h"
+#include "attrib.h"
+#include "lock.h"
+#include "flags.h"
 #include "match.h"
 #include "ansi.h"
 #include "parse.h"
 #include "dbdefs.h"
-#ifdef MEM_CHECK
-#include "memcheck.h"
-#endif
+#include "privtab.h"
 #include "mymalloc.h"
 #include "confmagic.h"
 
-extern int first_free;		/* free object list, from destroy.c */
+int forbidden_name(const char *name);
+void sstrcat(char *string, char *app);
+void do_switch(dbref player, char *expression, char **argv,
+	       dbref cause, int first, int notifyme);
+void do_verb(dbref player, dbref cause, char *arg1, char **argv);
+static int grep_util_helper(dbref player, dbref thing,
+			    char const *pattern, ATTR *atr, void *args);
+static int grep_helper(dbref player, dbref thing,
+		       char const *pattern, ATTR *atr, void *args);
+void do_grep(dbref player, char *obj, char *lookfor, int flag, int insensitive);
+#ifdef QUOTA
+static int pay_quota(dbref, int);
+#endif				/* QUOTA */
+extern PRIV attr_privs[];
 
-int could_doit _((dbref player, dbref thing));
-void charge_action _((dbref player, dbref thing, const char *awhat));
-void did_it
-_(
-  (dbref player, dbref thing, const char *what, const char *def,
-   const char *owhat, const char *odef, const char *awhat, dbref loc));
-int can_see _((dbref player, dbref thing, int can_see_loc));
-int controls _((dbref who, dbref what));
-int can_pay_fees _((dbref who, int pennies));
-void giveto _((dbref who, dbref pennies));
-int payfor _((dbref who, int cost));
-int forbidden_name _((const char *name));
-int ok_name _((const char *name));
-int ok_player_name _((const char *name));
-int ok_password _((const char *password));
-void sstrcat _((char *string, char *app));
-void do_switch
-_((dbref player, char *expression, char **argv, dbref cause, int first));
-dbref parse_match_possessive _((dbref player, const char *str));
-void page_return
-_(
-  (dbref player, dbref target, const char *type, const char *message,
-   const char *def));
-dbref where_is _((dbref thing));
-int nearby _((dbref obj1, dbref obj2));
-void do_verb _((dbref player, dbref cause, char *arg1, char **argv));
-char *grep_util
-_(
-  (dbref player, dbref thing, char *pattern, char *lookfor, int len,
-   int insensitive));
-static int grep_util_helper
-_((dbref player, dbref thing, char const *pattern, ATTR *atr, void *args));
-static int grep_helper
-_((dbref player, dbref thing, char const *pattern, ATTR *atr, void *args));
-void do_grep
-_((dbref player, char *obj, char *lookfor, int flag, int insensitive));
-#ifdef __GNUC__
-char *tprintf(const char *fmt, ...)
-  __attribute__ ((__format__(__printf__, 1, 2)));
-#else
-char *tprintf _((const char *fmt, ...));
-#endif
-
-#ifdef I_STDARG
-char *
+char *WIN32_CDECL
 tprintf(const char *fmt, ...)
-#else
-char *
-tprintf(va_alist)
-    va_dcl
-#endif
 {
   /* this is a generic function used to generate a format string */
-
-  static char buff[BUFFER_LEN * 3];	/* safety margin */
-  va_list args;
-#ifndef I_STDARG
-  char *fmt;
-
-  va_start(args);
-  fmt = va_arg(args, char *);
+#ifdef HAS_VSNPRINTF
+  static char buff[BUFFER_LEN];
 #else
+  static char buff[BUFFER_LEN * 3];	/* safety margin */
+#endif
+  va_list args;
+
   va_start(args, fmt);
+
+#ifdef HAS_VSNPRINTF
+  vsnprintf(buff, sizeof buff, fmt, args);
+#else
+  vsprintf(buff, fmt, args);
 #endif
 
-  (void) vsprintf(buff, fmt, args);
   buff[BUFFER_LEN - 1] = '\0';
   va_end(args);
   return (buff);
 }
 
+/* Safer replacement for notify(player, tprintf(fmt, ...)) */
+void WIN32_CDECL
+notify_format(dbref player, const char *fmt, ...)
+{
+#ifdef HAS_VSNPRINTF
+  char buff[BUFFER_LEN];
+#else
+  char buff[BUFFER_LEN * 3];
+#endif
+  va_list args;
+
+  va_start(args, fmt);
+
+#ifdef HAS_VSNPRINTF
+  vsnprintf(buff, sizeof buff, fmt, args);
+#else
+  vsprintf(buff, fmt, args);
+#endif
+
+  buff[BUFFER_LEN - 1] = '\0';
+  va_end(args);
+  notify(player, buff);
+}
+
+
 int
 could_doit(player, thing)
     dbref player;
@@ -131,47 +111,44 @@ could_doit(player, thing)
   return (eval_lock(player, thing, Basic_Lock));
 }
 
-void
+/* 1.7.4p2: Return value changed from void to int */
+int
 charge_action(player, thing, awhat)
     dbref player;
     dbref thing;
     const char *awhat;
 {
-  ATTR *d;
   ATTR *b;
-  char tbuf[BUFFER_LEN];
   char tbuf2[BUFFER_LEN];
   int num;
 
-  if (awhat && *awhat && (d = atr_get(thing, awhat))) {
-    strcpy(tbuf, uncompress(d->value));
+  if (!awhat || !*awhat)
+    return 0;
 
-    /* check if object has # of charges */
-    b = atr_get_noparent(thing, "CHARGES");
+  /* check if object has # of charges */
+  b = atr_get_noparent(thing, "CHARGES");
 
-    if (!b) {
-      /* no charges set, just execute the action */
-      parse_que(thing, tbuf, player);
-      return;
-    } else {
-      strcpy(tbuf2, uncompress(b->value));
-      num = atoi(tbuf2);
-      if (num) {
-	/* charges left, decrement and execute */
+  if (!b) {
+    /* no charges set, just execute the action */
+    return queue_attribute(thing, awhat, player);
+  } else {
+    strcpy(tbuf2, uncompress(AL_STR(b)));
+    num = atoi(tbuf2);
+    if (num) {
+      /* charges left, decrement and execute */
+      int res;
+      if ((res = queue_attribute(thing, awhat, player)))
 	(void) atr_add(thing, "CHARGES", tprintf("%d", num - 1),
 		       Owner(b->creator), NOTHING);
-	parse_que(thing, tbuf, player);
-	return;
-      } else if (!(d = atr_get(thing, "RUNOUT")))
-	/* no charges left and no runout; do nothing */
-	return;
-      /* no charges left, execute runout */
-      strcpy(tbuf, uncompress(d->value));
-      parse_que(thing, tbuf, player);
+      return res;
+    } else {
+      /* no charges left, try to execute runout */
+      return queue_attribute(thing, "RUNOUT", player);
     }
   }
 }
 
+
 void
 did_it(player, thing, what, def, owhat, odef, awhat, loc)
     dbref player;
@@ -182,17 +159,35 @@ did_it(player, thing, what, def, owhat, odef, awhat, loc)
     const char *odef;
     const char *awhat;
     dbref loc;
+{
+  /* Bunch o' nulls */
+  static char *myenv[10] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+  real_did_it(player, thing, what, def, owhat, odef, awhat, loc, myenv);
+}
+
+void
+real_did_it(player, thing, what, def, owhat, odef, awhat, loc, myenv)
+    dbref player;
+    dbref thing;
+    const char *what;
+    const char *def;
+    const char *owhat;
+    const char *odef;
+    const char *awhat;
+    dbref loc;
+    char *myenv[10];
 {
   /* executes the @attr, @oattr, @aattr for a command - gives a message
    * to the enactor and others in the room with the enactor, and executes
-   * an action.
+   * an action. We load wenv with the values in myenv.
    */
 
   ATTR *d;
-  char buff[BUFFER_LEN], *bp, *sp;
-  char const *asave, *ap;
+  char buff[BUFFER_LEN], *bp, *sp, *asave;
+  char const *ap;
   int j;
-  char *preserve[10];
+  char *preserves[10];
+  char *preserveq[NUMQ];
   int need_pres = 0;
 
   loc = (loc == NOTHING) ? Location(player) : loc;
@@ -206,8 +201,10 @@ did_it(player, thing, what, def, owhat, odef, awhat, loc)
       if (d) {
 	if (!need_pres) {
 	  need_pres = 1;
-	  save_global_regs("did_it_save", preserve);
+	  save_global_regs("did_it_save", preserveq);
+	  save_global_env("did_it_save", preserves);
 	}
+	restore_global_env("did_it", myenv);
 	asave = safe_uncompress(d->value);
 	ap = asave;
 	bp = buff;
@@ -226,8 +223,10 @@ did_it(player, thing, what, def, owhat, odef, awhat, loc)
 	if (d) {
 	  if (!need_pres) {
 	    need_pres = 1;
-	    save_global_regs("did_it_save", preserve);
+	    save_global_regs("did_it_save", preserveq);
+	    save_global_env("did_it_save", preserves);
 	  }
+	  restore_global_env("did_it", myenv);
 	  asave = safe_uncompress(d->value);
 	  ap = asave;
 	  bp = buff;
@@ -249,15 +248,28 @@ did_it(player, thing, what, def, owhat, odef, awhat, loc)
       }
     }
   }
-  if (need_pres)
-    restore_global_regs("did_it_save", preserve);
-  for (j = 0; j < 10; j++) {
-    wnxt[j] = NULL;
-    rnxt[j] = NULL;
+  if (need_pres) {
+    restore_global_regs("did_it_save", preserveq);
+    restore_global_env("did_it_save", preserves);
   }
+  for (j = 0; j < 10; j++)
+    wnxt[j] = myenv[j];
+  for (j = 0; j < NUMQ; j++)
+    rnxt[j] = NULL;
   charge_action(player, thing, awhat);
 }
 
+/* BEWARE:
+ *
+ * first_visible() does not behave as intended. It _should_ return the first
+ * object in `thing' that is !DARK. However, because of the controls() check
+ * the function will return a DARK object if the player owns it.
+ *
+ * The behavior is left as is because so many functions in fundb.c rely on
+ * the incorrect behavior to return expected values. The lv*() functions
+ * also make rewriting this fairly pointless.
+ */
+
 dbref
 first_visible(player, thing)
     dbref player;
@@ -267,9 +279,11 @@ first_visible(player, thing)
   int ldark;
   dbref loc;
 
-  if (!(GoodObject(thing)))
+  if (!GoodObject(thing) || IsRoom(thing))
     return NOTHING;
   loc = IsExit(thing) ? Source(thing) : Location(thing);
+  if (!GoodObject(loc))
+    return NOTHING;
   ldark = IsPlayer(loc) ? Opaque(loc) : Dark(loc);
 
   while (GoodObject(thing)) {
@@ -279,7 +293,7 @@ first_visible(player, thing)
 	  return thing;
 	lck = 1;
       }
-      if (controls(player, thing))
+      if (controls(player, thing))	/* this is what causes DARK objects to show */
 	return thing;
     } else {
       return thing;
@@ -303,7 +317,8 @@ can_see(player, thing, can_see_loc)
    * 'look' 3) unconnected (sleeping) players aren't listed in a 'look'
    */
   if (player == thing || IsExit(thing) ||
-      (IsPlayer(thing) && !Connected(thing))) return 0;
+      (IsPlayer(thing) && !Connected(thing)))
+    return 0;
 
   /* if the room is lit, you can see any non-dark objects */
   else if (can_see_loc)
@@ -315,9 +330,7 @@ can_see(player, thing, can_see_loc)
 }
 
 int
-controls(who, what)
-    dbref who;
-    dbref what;
+controls(dbref who, dbref what)
 {
   /* Wizard controls everything 
    * owners control their stuff
@@ -328,6 +341,7 @@ controls(who, what)
    * Those who pass the enterlock of a ZoneMaster control his objects,
    * but not the ZoneMaster himself.
    */
+  struct boolexp *c;
 
   if (!GoodObject(what))
     return 0;
@@ -344,13 +358,22 @@ controls(who, what)
   if (Owns(who, what) && (!Inheritable(what) || Inheritable(who)))
     return 1;
 
-  if ((Zone(what) != NOTHING) &&
-      (!IsPlayer(what)) &&
-      !Inheritable(what) && (eval_lock(who, Zone(what), Zone_Lock))) return 1;
+  if (Inheritable(what) || IsPlayer(what))
+    return 0;
+
+  if (!ZONE_CONTROL_ZMP && (Zone(what) != NOTHING) &&
+      eval_lock(who, Zone(what), Zone_Lock))
+    return 1;
 
   if (ZMaster(Owner(what)) && !IsPlayer(what) &&
-      (eval_lock(who, Owner(what), Zone_Lock))) return 1;
+      eval_lock(who, Owner(what), Zone_Lock))
+    return 1;
 
+  c = getlock_noparent(what, Control_Lock);
+  if (c != TRUE_BOOLEXP) {
+    if (eval_boolexp(who, c, what))
+      return 1;
+  }
   return 0;
 }
 
@@ -364,24 +387,20 @@ can_pay_fees(who, pennies)
    * aren't subject to either.
    */
 
-#ifdef QUOTA
-  int pay_quota();
-#endif				/* QUOTA */
-
   /* check database size -- EVERYONE is subject to this! */
   if (DBTOP_MAX && (db_top >= DBTOP_MAX + 1) && (first_free == NOTHING)) {
-    notify(who, "Sorry, there is no more room in the database.");
+    notify(who, T("Sorry, there is no more room in the database."));
     return 0;
   }
   /* Can they afford it? */
   if (!NoPay(who) && (Pennies(Owner(who)) < pennies)) {
-    notify(who, tprintf("Sorry, you don't have enough %s.", MONIES));
+    notify_format(who, T("Sorry, you don't have enough %s."), MONIES);
     return 0;
   }
   /* check building quota */
 #ifdef QUOTA
   if (!NoQuota(who) && !pay_quota(who, QUOTA_COST)) {
-    notify(who, "Sorry, your building quota has run out.");
+    notify(who, T("Sorry, your building quota has run out."));
     return 0;
   }
 #endif				/* QUOTA */
@@ -404,8 +423,8 @@ giveto(who, pennies)
     return;
 
   who = Owner(who);
-  if ((Pennies(who) + pennies) > MAX_PENNIES)
-    s_Pennies(who, MAX_PENNIES);
+  if ((Pennies(who) + pennies) > Max_Pennies(who))
+    s_Pennies(who, Max_Pennies(who));
   else
     s_Pennies(who, Pennies(who) + pennies);
 }
@@ -464,7 +483,7 @@ get_current_quota(who)
   else
     limit = owned;
 
-  atr_add(Owner(who), "RQUOTA", tprintf("%d", limit), GOD, NOTHING);
+  (void) atr_add(Owner(who), "RQUOTA", tprintf("%d", limit), GOD, NOTHING);
 
   return (limit);
 }
@@ -478,14 +497,14 @@ change_quota(who, payment)
   /* add or subtract from quota */
 
   /* wizards and royalty don't need a quota */
-  if (NoQuota(Owner(who)))
+  if (NoQuota(who))
     return;
 
-  atr_add(Owner(who), "RQUOTA",
-	  tprintf("%d", get_current_quota(who) + payment), GOD, NOTHING);
+  (void) atr_add(Owner(who), "RQUOTA",
+		 tprintf("%d", get_current_quota(who) + payment), GOD, NOTHING);
 }
 
-int
+static int
 pay_quota(who, cost)
     dbref who;
     int cost;
@@ -497,7 +516,7 @@ pay_quota(who, cost)
   int curr;
 
   /* wizards and royalty don't need a quota */
-  if (NoQuota(Owner(who)))
+  if (NoQuota(who))
     return 1;
 
   /* figure out how much we have, and if it's big enough */
@@ -513,21 +532,26 @@ pay_quota(who, cost)
 #endif				/* QUOTA */
 
 int
-forbidden_name(name)
-    const char *name;
+forbidden_name(const char *name)
 {
   /* checks to see if name is in the forbidden names file */
 
   char buf[BUFFER_LEN], *newlin, *ptr;
   FILE *fp;
+  char *upname;
 
+  upname = strupper(name);
+#ifdef macintosh
+  fp = fopen(NAMES_FILE, "rb");
+#else
   fp = fopen(NAMES_FILE, "r");
+#endif
   if (!fp)
     return 0;
-  while (!feof(fp)) {
-    fgets(buf, BUFFER_LEN, fp);
+  while (fgets(buf, BUFFER_LEN - 1, fp)) {
+    upcasestr(buf);
     /* step on the newline */
-    if ((newlin = (char *) index(buf, '\n')) != NULL)
+    if ((newlin = strchr(buf, '\n')) != NULL)
       *newlin = '\0';
     ptr = buf;
     if (name && ptr && quick_wild(ptr, name)) {
@@ -540,32 +564,36 @@ forbidden_name(name)
 }
 
 int
-ok_name(name)
-    const char *name;
+ok_name(const char *n)
 {
   /* is name valid for an object? */
 
-  const char *p;
+  const unsigned char *p, *name = (const unsigned char *) n;
+
+  if (!name || !*name)
+    return 0;
 
   /* No leading spaces */
-  if (isspace(*name))
+  if (isspace((unsigned char) *name))
     return 0;
 
   /* only printable characters */
   for (p = name; p && *p; p++) {
-    if (!isprint(*p))
+    if (!isprint((unsigned char) *p))
       return 0;
-    if ((*p == '[') || (*p == ']') || (*p == '%') || (*p == '\\'))
+    if (ONLY_ASCII_NAMES && *p > 127)
+      return 0;
+    if (strchr("[]%\\=&|", *p))
       return 0;
   }
 
   /* No trailing spaces */
   p--;
-  if (isspace(*p))
+  if (isspace((unsigned char) *p))
     return 0;
 
   /* Not too long */
-  if (strlen(name) >= OBJECT_NAME_LIMIT)
+  if (u_strlen(name) >= OBJECT_NAME_LIMIT)
     return 0;
 
   /* No magic cookies */
@@ -573,32 +601,29 @@ ok_name(name)
 	  && *name
 	  && *name != LOOKUP_TOKEN
 	  && *name != NUMBER_TOKEN
-	  && *name != NOT_TOKEN
-	  && !index(name, ARG_DELIMITER)
-	  && !index(name, AND_TOKEN) && !index(name, OR_TOKEN)
-	  && strcasecmp(name, "me")
-	  && strcasecmp(name, "home")
-	  && strcasecmp(name, "here"));
+	  && *name != NOT_TOKEN && strcasecmp((char *) name, "me")
+	  && strcasecmp((char *) name, "home")
+	  && strcasecmp((char *) name, "here"));
 }
 
 int
-ok_player_name(name)
-    const char *name;
+ok_player_name(const char *name)
 {
   /* is name okay for a player? */
 
-  const char *scan, *good;
+  const unsigned char *scan, *good;
 
   if (!ok_name(name) || forbidden_name(name) ||
-      strlen(name) >= PLAYER_NAME_LIMIT) return 0;
+      strlen(name) >= (size_t) PLAYER_NAME_LIMIT)
+    return 0;
 
-  good = PLAYER_NAME_SPACES ? " `$_-.,'" : "`$_-.,'";
+  good = (unsigned char *) (PLAYER_NAME_SPACES ? " `$_-.,'" : "`$_-.,'");
 
   /* Make sure that the name contains legal characters only */
-  for (scan = name; scan && *scan; scan++) {
-    if (isalpha(*scan) || isdigit(*scan))
+  for (scan = (unsigned char *) name; scan && *scan; scan++) {
+    if (isalnum((unsigned char) *scan))
       continue;
-    if (!index(good, *scan))
+    if (!strchr(good, *scan))
       return 0;
   }
 
@@ -624,6 +649,28 @@ ok_password(password)
   return 1;
 }
 
+int
+ok_command_name(const char *name)
+{
+  /* is name ok for a command or function?
+   * It must begin with an uppercase alpha, and contain only
+   * uppercase alpha, numbers, or underscore thereafter.
+   */
+  const unsigned char *p;
+  if (!isupper((unsigned char) *name))
+    return 0;
+  for (p = (unsigned char *) name; p && *p; p++) {
+    if (!(isupper(*p) || isdigit(*p) || (*p == '_')))
+      return 0;
+  }
+
+  /* Not too long */
+  if (strlen(name) >= COMMAND_NAME_LIMIT)
+    return 0;
+
+  return 1;
+}
+
 void
 sstrcat(string, app)
     char *string;
@@ -643,17 +690,13 @@ sstrcat(string, app)
 
 /* for lack of better place the @switch code is here */
 void
-do_switch(player, expression, argv, cause, first)
+do_switch(player, expression, argv, cause, first, notifyme)
     dbref player;
     char *expression;
     char **argv;
     dbref cause;
-    int first;
-
-
-
-
-				/* 0, match all, 1, match first */
+    int first;			/* 0, match all, 1, match first */
+    int notifyme;
 {
   int any = 0, a;
   char buff[BUFFER_LEN], *bp;
@@ -664,14 +707,15 @@ do_switch(player, expression, argv, cause, first)
     return;
 
   /* set up environment for any spawned commands */
-  for (a = 0; a < 10; a++) {
+  for (a = 0; a < 10; a++)
     wnxt[a] = wenv[a];
+  for (a = 0; a < NUMQ; a++)
     rnxt[a] = renv[a];
-  }
 
   /* now try a wild card match of buff with stuff in coms */
   for (a = 1;
-       !(first && any) && (a < (MAX_ARG - 1)) && argv[a] && argv[a + 1]; a += 2) {
+       !(first && any) && (a < (MAX_ARG - 1)) && argv[a] && argv[a + 1];
+       a += 2) {
     /* eval expression */
     ap = argv[a];
     bp = buff;
@@ -684,10 +728,7 @@ do_switch(player, expression, argv, cause, first)
       any = 1;
       tbuf1 = replace_string("#$", expression, argv[a + 1]);
       parse_que(player, tbuf1, cause);
-      free(tbuf1);
-#ifdef MEM_CHECK
-      del_check("replace_string.buff");
-#endif
+      mush_free(tbuf1, "replace_string.buff");
     }
   }
 
@@ -695,11 +736,12 @@ do_switch(player, expression, argv, cause, first)
   if ((a < MAX_ARG) && !any && argv[a]) {
     tbuf1 = replace_string("#$", expression, argv[a]);
     parse_que(player, tbuf1, cause);
-    free(tbuf1);
-#ifdef MEM_CHECK
-    del_check("replace_string.buff");
-#endif
+    mush_free(tbuf1, "replace_string.buff");
   }
+
+  /* Pop on @notify me, if requested */
+  if (notifyme)
+    parse_que(player, "@notify me", cause);
 }
 
 dbref
@@ -716,7 +758,7 @@ parse_match_possessive(player, str)
   box = name;
 
   /* check to see if we have an 's sequence */
-  if ((obj = (char *) index(name, '\'')) == NULL)
+  if ((obj = strchr(name, '\'')) == NULL)
     return NOTHING;
   *obj++ = '\0';		/* terminate */
   if ((*obj == '\0') || ((*obj != 's') && (*obj != 'S')))
@@ -725,7 +767,7 @@ parse_match_possessive(player, str)
   /* skip over the 's' and whitespace */
   do {
     obj++;
-  } while (isspace(*obj));
+  } while (isspace((unsigned char) *obj));
 
   /* we already have a terminating null, so we're okay to just do matches */
   loc = match_result(player, box, NOTYPE, MAT_NEIGHBOR | MAT_POSSESSION);
@@ -748,10 +790,9 @@ page_return(player, target, type, message, def)
   /* code for auto-return page - HAVEN, IDLE, and AWAY messages */
 
   ATTR *d;
-  char buff[BUFFER_LEN], *bp;
-  char const *asave, *ap;
+  char buff[BUFFER_LEN], *bp, *asave;
+  char const *ap;
   struct tm *ptr;
-  time_t t;
 
   if (message && *message) {
     d = atr_get(target, message);
@@ -764,18 +805,13 @@ page_return(player, target, type, message, def)
       *bp = '\0';
       free((Malloc_t) asave);
       if (*buff) {
-	t = time(NULL);
-	ptr = (struct tm *) localtime(&t);
-	notify(player, tprintf("%s message from %s: %s", type,
-			       Name(target), buff));
+	ptr = (struct tm *) localtime(&mudtime);
+	notify_format(player, T("%s message from %s: %s"), type,
+		      Name(target), buff);
 	if (!Haven(target))
-	  notify(target,
-		 tprintf("[%d:%02d] %s message sent to %s.",
-			 MILITARY_TIME ? ptr->tm_hour :
-			 (ptr->tm_hour ? (ptr->tm_hour >
-					  12 ? (ptr->tm_hour -
-						12) : ptr->tm_hour) : 12),
-			 ptr->tm_min, type, Name(player)));
+	  notify_format(target,
+			T("[%d:%02d] %s message sent to %s."), ptr->tm_hour,
+			ptr->tm_min, type, Name(player));
       }
     } else if (def && *def)
       notify(player, def);
@@ -847,19 +883,19 @@ do_verb(player, cause, arg1, argv)
   victim = match_result(player, arg1, NOTYPE, MAT_EVERYTHING);
 
   if (victim == NOTHING) {
-    notify(player, "What was the victim of the verb?");
+    notify(player, T("What was the victim of the verb?"));
     return;
   }
   /* find the object that executes the action */
 
   if (!argv || !argv[1] || !*argv[1]) {
-    notify(player, "What do you want to do with the verb?");
+    notify(player, T("What do you want to do with the verb?"));
     return;
   }
-  actor = match_result(player, argv[1], NOTYPE, MAT_NEAR_THINGS);
+  actor = match_result(player, argv[1], NOTYPE, MAT_EVERYTHING);
 
   if (actor == NOTHING) {
-    notify(player, "What do you want to do the verb?");
+    notify(player, T("What do you want to do the verb?"));
     return;
   }
   /* Control check is fascist. 
@@ -875,7 +911,7 @@ do_verb(player, cause, arg1, argv)
   if (!(Wizard(player) ||
 	(controls(player, victim) && controls(player, actor)) ||
 	((controls(cause, actor) && Can_Examine(player, victim))))) {
-    notify(player, "Permission denied.");
+    notify(player, T("Permission denied."));
     return;
   }
   /* We're okay.  Send out messages. */
@@ -885,9 +921,9 @@ do_verb(player, cause, arg1, argv)
     wenv[i] = argv[i + 7];
   }
 
-  did_it(actor, victim,
-	 upcasestr(argv[2]), argv[3], upcasestr(argv[4]), argv[5],
-	 NULL, Location(actor));
+  real_did_it(actor, victim,
+	      upcasestr(argv[2]), argv[3], upcasestr(argv[4]), argv[5],
+	      NULL, Location(actor), wenv);
 
   for (i = 0; i < 10; i++)
     wenv[i] = wsave[i];
@@ -910,9 +946,9 @@ struct guh_args {
 
 static int
 grep_util_helper(player, thing, pattern, atr, args)
-    dbref player;
-    dbref thing;
-    char const *pattern;
+    dbref player __attribute__ ((__unused__));
+    dbref thing __attribute__ ((__unused__));
+    char const *pattern __attribute__ ((__unused__));
     ATTR *atr;
     void *args;
 {
@@ -953,7 +989,7 @@ grep_util(player, thing, pattern, lookfor, len, insensitive)
    */
   struct guh_args guh;
 
-  guh.buff = (char *) malloc(BUFFER_LEN + 1);
+  guh.buff = (char *) mush_malloc(BUFFER_LEN + 1, "grep_util.buff");
   guh.bp = guh.buff;
   guh.lookfor = lookfor;
   guh.len = len;
@@ -967,63 +1003,57 @@ struct gh_args {
   char *lookfor;
   int len;
   int insensitive;
-  char repl[BUFFER_LEN];
-  int rlen;
 };
 
 static int
 grep_helper(player, thing, pattern, atr, args)
     dbref player;
-    dbref thing;
-    char const *pattern;
+    dbref thing __attribute__ ((__unused__));
+    char const *pattern __attribute__ ((__unused__));
     ATTR *atr;
     void *args;
 {
   struct gh_args *gh = args;
   int found;
-  int d;
   char *s;
   char tbuf1[BUFFER_LEN];
+  char buf[BUFFER_LEN];
+  char *tbp;
 
   found = 0;
+  tbp = tbuf1;
+
   s = (char *) uncompress(AL_STR(atr));	/* warning: static */
-  for (d = 0; (d < BUFFER_LEN) && *s;) {
+  while (s && *s) {
     if ((gh->insensitive && !strncasecmp(gh->lookfor, s, gh->len)) ||
 	(!gh->insensitive && !strncmp(gh->lookfor, s, gh->len))) {
       found = 1;
-      if ((d + gh->rlen) < BUFFER_LEN) {
-	strcpy(tbuf1 + d, ANSI_HILITE);
-	strncpy(tbuf1 + d + strlen(ANSI_HILITE), s, gh->len);
-	strcpy(tbuf1 + d + strlen(ANSI_HILITE) + gh->len, ANSI_NORMAL);
-	d += gh->rlen;
-	s += gh->len;
-      } else
-	tbuf1[d++] = *s++;
-    } else
-      tbuf1[d++] = *s++;
+      strncpy(buf, s, gh->len);
+      buf[gh->len] = '\0';
+      s += gh->len;
+      safe_format(tbuf1, &tbp, "%s%s%s", ANSI_HILITE, buf, ANSI_NORMAL);
+    } else {
+      safe_chr(*s, tbuf1, &tbp);
+      s++;
+    }
   }
-  tbuf1[d++] = 0;
+  *tbp = '\0';
+
   /* if we got it, display it */
   if (found)
-    notify(player, tprintf("%s%s [#%d%s%s %s",
-			   ANSI_HILITE, AL_NAME(atr), Owner(AL_CREATOR(atr)),
-			   (AL_FLAGS(atr) & AF_LOCKED) ? "+]:" : "]:",
-			   ANSI_NORMAL, tbuf1));
+    notify_format(player, "%s%s [#%d%s]%s %s",
+		  ANSI_HILITE, AL_NAME(atr),
+		  Owner(AL_CREATOR(atr)),
+		  privs_to_letters(attr_privs, AL_FLAGS(atr)),
+		  ANSI_NORMAL, tbuf1);
   return found;
 }
 
+/* lookfor is an unparsed arg. flag is 0 for list and 1 for hilite.
+ * insensitive is 0 for case-sensitive, 1 for case-insensitive
+ */
 void
-do_grep(player, obj, lookfor, flag, insensitive)
-    dbref player;
-    char *obj;
-    char *lookfor;
-    int flag;
-    int insensitive;
-
-
-				/* this is an UNPARSED arg */
-				/* 0 for just list, 1 for hilites */
-				/* 0 for case-sensitive, 1 if not */
+do_grep(dbref player, char *obj, char *lookfor, int flag, int insensitive)
 {
   struct gh_args gh;
   dbref thing;
@@ -1032,11 +1062,11 @@ do_grep(player, obj, lookfor, flag, insensitive)
   char *tp;
 
   if ((len = strlen(lookfor)) < 1) {
-    notify(player, "What pattern do you want to grep for?");
+    notify(player, T("What pattern do you want to grep for?"));
     return;
   }
   /* find the attribute pattern */
-  pattern = (char *) index(obj, '/');
+  pattern = strchr(obj, '/');
   if (!pattern)
     pattern = (char *) "*";	/* set it to global match */
   else
@@ -1044,24 +1074,23 @@ do_grep(player, obj, lookfor, flag, insensitive)
 
   /* now we've got the object. match for it. */
   if ((thing = noisy_match_result(player, obj, NOTYPE, MAT_EVERYTHING)) ==
-      NOTHING) return;
+      NOTHING)
+    return;
   if (!Can_Examine(player, thing)) {
-    notify(player, "Permission denied.");
+    notify(player, T("Permission denied."));
     return;
   }
-  /* we can think of adding in the hilites like doing a find and replace */
-  sprintf(gh.repl, "%s%s%s", ANSI_HILITE, lookfor, ANSI_NORMAL);
-  gh.rlen = strlen(gh.repl);
 
   if (flag) {
     gh.lookfor = lookfor;
     gh.len = len;
     gh.insensitive = insensitive;
-    (void) atr_iter_get(player, thing, pattern, grep_helper, &gh);
+    if (!atr_iter_get(player, thing, pattern, grep_helper, &gh))
+      notify(player, T("No matching attributes."));
   } else {
     tp = grep_util(player, thing, pattern, lookfor, len, insensitive);
-    notify(player, tprintf("Matches of '%s' on %s(#%d): %s", lookfor,
-			   Name(thing), thing, tp));
-    free(tp);
+    notify_format(player, T("Matches of '%s' on %s(#%d): %s"), lookfor,
+		  Name(thing), thing, tp);
+    mush_free((Malloc_t) tp, "grep_util.buff");
   }
 }
