@@ -8,7 +8,6 @@
 #include "config.h"
 
 #include <windows.h>		/* for service and thread routines */
-#undef OPAQUE			/* clashes with MUSH definition */
 
 #ifdef I_STDLIB
 #include <stdlib.h>
@@ -18,7 +17,7 @@
 
 #include "conf.h"
 #include "mushdb.h"
-#include "intrface.h"
+
 #include "match.h"
 #include "externs.h"
 #include "mymalloc.h"
@@ -27,8 +26,8 @@
 #define THIS_SERVICE "PennMUSH"
 #define THIS_SERVICE_DISPLAY "PennMUSH for Win32"
 
-int main _((int argc, char **argv));
-void mainthread _((int argc, char **argv));
+int WIN32_CDECL main(int argc, char **argv);
+void mainthread(int argc, char **argv);
 
 
 SERVICE_STATUS ssStatus;	/*  current status of the service */
@@ -59,13 +58,13 @@ static int CmdStatusService(void);
 static void CmdDisplayFormat(void);
 static char *convert_error(DWORD error);
 static DWORD get_service_status(SERVICE_STATUS * svcstatus, int leave_open);
-int service_error(DWORD error_code, char *themessage, ...);
+int WIN32_CDECL service_error(DWORD error_code, char *themessage, ...);
 
 
 /* Need to include library: advapi32.lib for services routines */
 
 
-int
+int WIN32_CDECL
 main(argc, argv)
     int argc;
     char **argv;
@@ -101,7 +100,7 @@ main(argc, argv)
 
       status = get_service_status(&svcstatus, TRUE);
       if (status == 0 && svcstatus.dwCurrentState == SERVICE_RUNNING) {
-	fprintf(stderr, "The MUSH is already running as a service.\n");
+	fprintf(stderr, T("The MUSH is already running as a service.\n"));
 	return 1;
       }
       worker_thread(NULL);
@@ -115,7 +114,7 @@ main(argc, argv)
 
     status = get_service_status(&svcstatus, TRUE);
     if (status == 0 && svcstatus.dwCurrentState == SERVICE_RUNNING) {
-      fprintf(stderr, "The MUSH is already running as a service.\n");
+      fprintf(stderr, T("The MUSH is already running as a service.\n"));
       return 1;
     }
     /*  Under Windows 95 they won't be able to use the service manager */
@@ -131,12 +130,14 @@ main(argc, argv)
 
      */
 
-    fprintf(stderr, "Attempting to start PennMUSH as a service ...\n");
+    fprintf(stderr, T("Attempting to start PennMUSH as a service ...\n"));
     if (!StartServiceCtrlDispatcher(dispatchTable)) {
       fprintf(stderr,
-	      "Unable to start service, assuming running console-mode application.\n");
+	      T
+	      ("Unable to start service, assuming running console-mode application.\n"));
       fprintf(stderr,
-	      "You can save time on the next invocation by specifying: pennmush /run\n");
+	      T
+	      ("You can save time on the next invocation by specifying: pennmush /run\n"));
       worker_thread(NULL);
     }
   }				/*  end of argc == 1 */
@@ -254,7 +255,7 @@ service_ctrl(DWORD dwCtrlCode)
 
     shutdown_flag = 1;
 
-    flag_broadcast(0, 0, "GAME: Game shutdown by system operator");
+    flag_broadcast(0, 0, T("GAME: Game shutdown by system operator"));
 
     return;
 
@@ -362,7 +363,7 @@ shut_down_handler(DWORD dwCtrlType)
       TerminateThread(threadHandle, 1);
     threadHandle = NULL;
 
-    panic("System shutdown by system operator");
+    panic(T("System shutdown by system operator"));
 
     _exit(99);
 
@@ -403,7 +404,7 @@ worker_thread(VOID * notused)
   char *p;
 
   if (!GetModuleFileName(NULL, fullfilename, sizeof(fullfilename))) {
-    service_error(GetLastError(), "Cannot locate full filename");
+    service_error(GetLastError(), T("Cannot locate full filename"));
     Win32_Exit(1);
   }
 /*  remove last part of file name to get working directory */
@@ -433,12 +434,10 @@ worker_thread(VOID * notused)
 
 }				/*  end of worker_thread */
 
-void
+void WIN32_CDECL
 Win32_Exit(int exit_code)
 {
 
-  kill_timer();			/* stop timer thread */
-
 /*  if running as a thread, end the thread, otherwise just exit */
 
   fflush(stderr);
@@ -482,7 +481,7 @@ close_service_handles(void)
 
 /*  We put out *so* many error messages, let's centralise the whole thing */
 
-int
+int WIN32_CDECL
 service_error(DWORD error_code, char *themessage, ...)
 {
   va_list arglist;
@@ -518,7 +517,7 @@ open_service_manager(void)
 
   if (!SCmanager)
     return service_error(GetLastError(),
-			 "Unable to talk to the Service Control Manager");
+			 T("Unable to talk to the Service Control Manager"));
 
   return FALSE;
 
@@ -534,7 +533,7 @@ get_service(void)
   service = OpenService(SCmanager, THIS_SERVICE, SERVICE_ALL_ACCESS);
 
   if (!service)
-    return service_error(GetLastError(), "Cannot access service definition");
+    return service_error(GetLastError(), T("Cannot access service definition"));
 
   return FALSE;
 }				/*  end of get_service */
@@ -599,7 +598,7 @@ CmdInstallService(int argc, char *argv[])
  */
 
   if (!GetModuleFileName(NULL, fullfilename, sizeof(fullfilename)))
-    return service_error(GetLastError(), "Cannot locate full filename");
+    return service_error(GetLastError(), T("Cannot locate full filename"));
 
 /*
    Open a handle to the Service Control Manager.
@@ -612,21 +611,18 @@ CmdInstallService(int argc, char *argv[])
    Now create the service definition.
  */
 
-  service =
-    CreateService(SCmanager, THIS_SERVICE, THIS_SERVICE_DISPLAY,
-		  SERVICE_ALL_ACCESS, SERVICE_WIN32_OWN_PROCESS,
-		  SERVICE_AUTO_START, SERVICE_ERROR_NORMAL, fullfilename, NULL,	/*  no load ordering group */
-		  NULL,		/*  no tag identifier */
-		  NULL,		/*  no dependencies */
-		  NULL,		/*  LocalSystem account */
-		  NULL);	/*  no password */
+  service = CreateService(SCmanager, THIS_SERVICE, THIS_SERVICE_DISPLAY, SERVICE_ALL_ACCESS, SERVICE_WIN32_OWN_PROCESS, SERVICE_AUTO_START, SERVICE_ERROR_NORMAL, fullfilename, NULL,	/*  no load ordering group */
+			  NULL,	/*  no tag identifier */
+			  NULL,	/*  no dependencies */
+			  NULL,	/*  LocalSystem account */
+			  NULL);	/*  no password */
 
   if (!service)
-    return service_error(GetLastError(), "Unable to create service");
+    return service_error(GetLastError(), T("Unable to create service"));
 
   close_service_handles();
 
-  fprintf(stderr, "Service successfully installed\n");
+  fprintf(stderr, T("Service successfully installed\n"));
 
   return FALSE;
 }				/*  end of CmdInstallService */
@@ -646,7 +642,7 @@ CmdRemoveService(void)
  */
 
   if (status = get_service_status(&svcstatus, TRUE))
-    return service_error(status, "Unable to access service details");
+    return service_error(status, T("Unable to access service details"));
 
 /*
    Check to see that the service is not running.
@@ -654,18 +650,19 @@ CmdRemoveService(void)
 
   if (svcstatus.dwCurrentState != SERVICE_STOPPED)
     return service_error(0,
-			 "You must stop the service before you can remove it.");
+			 T
+			 ("You must stop the service before you can remove it."));
 
 /*
    Everything is fine, so delete the service definition.
  */
 
   if (!DeleteService(service))
-    return service_error(GetLastError(), "Cannot remove service");
+    return service_error(GetLastError(), T("Cannot remove service"));
 
   close_service_handles();
 
-  fprintf(stderr, "Service successfully removed\n");
+  fprintf(stderr, T("Service successfully removed\n"));
 
   return FALSE;
 }				/*  end of CmdRemoveService */
@@ -686,21 +683,21 @@ CmdStartService(void)
  */
 
   if (status = get_service_status(&svcstatus, TRUE))
-    return service_error(status, "Unable to access service details");
+    return service_error(status, T("Unable to access service details"));
 
   if (svcstatus.dwCurrentState != SERVICE_STOPPED)
-    return service_error(0, "The service is not currently stopped.");
+    return service_error(0, T("The service is not currently stopped."));
 
 /*
    Everything is fine, so start the service
  */
 
   if (!StartService(service, 0, NULL))
-    return service_error(GetLastError(), "Cannot start service");
+    return service_error(GetLastError(), T("Cannot start service"));
 
   close_service_handles();
 
-  fprintf(stderr, "Start request sent to service\n");
+  fprintf(stderr, T("Start request sent to service\n"));
 
   return FALSE;
 }				/*  end of CmdStartService */
@@ -720,21 +717,21 @@ CmdStopService(void)
  */
 
   if (status = get_service_status(&svcstatus, TRUE))
-    return service_error(status, "Unable to access service details");
+    return service_error(status, T("Unable to access service details"));
 
   if (svcstatus.dwCurrentState != SERVICE_RUNNING)
-    return service_error(0, "The service is not currently running.");
+    return service_error(0, T("The service is not currently running."));
 
 /*
    Everything is fine, so stop the service
  */
 
   if (!ControlService(service, SERVICE_CONTROL_STOP, &svcstatus))
-    return service_error(GetLastError(), "Cannot stop service");
+    return service_error(GetLastError(), T("Cannot stop service"));
 
   close_service_handles();
 
-  fprintf(stderr, "Stop request sent to service\n");
+  fprintf(stderr, T("Stop request sent to service\n"));
 
   return FALSE;
 }				/*  end of CmdStopService */
@@ -756,32 +753,32 @@ CmdStatusService(void)
  */
 
   if (status = get_service_status(&svcstatus, FALSE))
-    return service_error(status, "Unable to access service details");
+    return service_error(status, T("Unable to access service details"));
 
   switch (svcstatus.dwCurrentState) {
   case SERVICE_STOPPED:
-    p = "The service is not running.";
+    p = T("The service is not running.");
     break;
   case SERVICE_START_PENDING:
-    p = "The service is starting.";
+    p = T("The service is starting.");
     break;
   case SERVICE_STOP_PENDING:
-    p = "The service is stopping.";
+    p = T("The service is stopping.");
     break;
   case SERVICE_RUNNING:
-    p = "The service is running.";
+    p = T("The service is running.");
     break;
   case SERVICE_CONTINUE_PENDING:
-    p = "The service continue is pending.";
+    p = T("The service continue is pending.");
     break;
   case SERVICE_PAUSE_PENDING:
-    p = "The service pause is pending.";
+    p = T("The service pause is pending.");
     break;
   case SERVICE_PAUSED:
-    p = "The service is paused.";
+    p = T("The service is paused.");
     break;
   default:
-    p = "Unrecognised status.";
+    p = T("Unrecognised status.");
     break;
   }				/*  end of switch */
 
@@ -799,18 +796,19 @@ CmdStatusService(void)
 static void
 CmdDisplayFormat(void)
 {
-  fprintf(stderr, "Usage is :-\n");
-  fprintf(stderr, " %s           - runs as a service, or stand-alone\n",
+  fprintf(stderr, T("Usage is :-\n"));
+  fprintf(stderr, T(" %s           - runs as a service, or stand-alone\n"),
 	  THIS_SERVICE);
-  fprintf(stderr, " %s /run      - runs stand-alone\n", THIS_SERVICE);
-  fprintf(stderr, " %s /start    - starts this service\n", THIS_SERVICE);
-  fprintf(stderr, " %s /stop     - stops this service\n", THIS_SERVICE);
-  fprintf(stderr, " %s /install  - installs this service\n", THIS_SERVICE);
-  fprintf(stderr, " %s /remove   - removes (un-installs) this service\n",
+  fprintf(stderr, T(" %s /run      - runs stand-alone\n"), THIS_SERVICE);
+  fprintf(stderr, T(" %s /start    - starts this service\n"), THIS_SERVICE);
+  fprintf(stderr, T(" %s /stop     - stops this service\n"), THIS_SERVICE);
+  fprintf(stderr, T(" %s /install  - installs this service\n"), THIS_SERVICE);
+  fprintf(stderr, T(" %s /remove   - removes (un-installs) this service\n"),
 	  THIS_SERVICE);
-  fprintf(stderr, " %s /status   - displays the status of this service\n",
+  fprintf(stderr, T(" %s /status   - displays the status of this service\n"),
+	  THIS_SERVICE);
+  fprintf(stderr, T(" %s /help     - displays this information\n"),
 	  THIS_SERVICE);
-  fprintf(stderr, " %s /help     - displays this information\n", THIS_SERVICE);
 }				/*  end of CmdDisplayFormat */
 
 static char *
