@@ -6,11 +6,7 @@
 /* commands which set parameters */
 #include <stdio.h>
 #include <ctype.h>
-#ifdef I_STRING
 #include <string.h>
-#else
-#include <strings.h>
-#endif
 #ifdef I_SYS_TIME
 #include <sys/time.h>
 #else
@@ -19,52 +15,44 @@
 #ifdef I_SYS_TYPES
 #include <sys/types.h>
 #endif
-#ifdef I_STDLIB
 #include <stdlib.h>
-#endif
 
 #include "conf.h"
-#ifdef WIN32
-#include "shs.h"
-#endif
 #include "mushdb.h"
 #include "match.h"
-#include "intrface.h"
+#include "attrib.h"
 #include "externs.h"
 #include "ansi.h"
 #include "command.h"
 #include "mymalloc.h"
+#include "flags.h"
+#include "dbdefs.h"
+#include "lock.h"
+#include "log.h"
 #include "confmagic.h"
 
-extern void set_flag
-_((dbref player, dbref thing, char *flag, int negate, int hear, int listener));	/* from flags.c */
-
-extern void delete_player _((dbref player, char *alias));
-extern void add_player _((dbref player, char *alias));
-extern void charge_action _((dbref player, dbref thing, const char *awhat));
-void do_name _((dbref player, const char *name, char *newname));
-void do_chown _((dbref player, const char *name, const char *newobj));
-static int chown_ok _((dbref player, dbref thing, dbref newowner));
-void chown_object _((dbref player, dbref thing, dbref newowner));
-void do_chzone _((dbref player, const char *name, const char *newobj));
+void do_name(dbref player, const char *name, char *newname);
+void do_chown(dbref player, const char *name, const char *newobj, int preserve);
+static int chown_ok(dbref player, dbref thing, dbref newowner);
+int do_chzone(dbref player, const char *name, const char *newobj, int noisy);
 void do_attrib_flags
-_((dbref player, const char *obj, const char *atrname, const char *flag));
-int do_set _((dbref player, const char *name, char *flag));
-void do_cpattr _((dbref player, char *oldpair, char **newpair, int move));
-void do_gedit _((dbref player, char *it, char **argv));
-void do_edit _((dbref player, dbref thing, char *q, char **argv));
-void do_trigger _((dbref player, char *object, char **argv));
-void do_use _((dbref player, const char *what));
-void do_parent _((dbref player, char *name, char *parent_name));
-void do_wipe _((dbref player, char *name));
-static int af_helper _((dbref player, dbref thing, char const *pattern,
-			ATTR *atr, void *args));
-static int gedit_helper _((dbref player, dbref thing, char const *pattern,
-			   ATTR *atr, void *args));
-static int wipe_helper _((dbref player, dbref thing, char const *pattern,
-			  ATTR *atr, void *args));
-static void copy_attrib_flags
-_((dbref player, dbref target, ATTR *atr, int flags));
+  (dbref player, const char *obj, const char *atrname, const char *flag);
+int do_set(dbref player, const char *name, char *flag);
+void do_cpattr
+  (dbref player, char *oldpair, char **newpair, int move, int noflagcopy);
+void do_gedit(dbref player, char *it, char **argv);
+void do_trigger(dbref player, char *object, char **argv);
+void do_use(dbref player, const char *what);
+void do_parent(dbref player, char *name, char *parent_name);
+void do_wipe(dbref player, char *name);
+static int af_helper(dbref player, dbref thing, char const *pattern,
+		     ATTR *atr, void *args);
+static int gedit_helper(dbref player, dbref thing, char const *pattern,
+			ATTR *atr, void *args);
+static int wipe_helper(dbref player, dbref thing, char const *pattern,
+		       ATTR *atr, void *args);
+static void copy_attrib_flags(dbref player, dbref target, ATTR *atr, int flags);
+
 
 void
 do_name(player, name, newname)
@@ -74,20 +62,15 @@ do_name(player, name, newname)
 {
   dbref thing;
   char *password;
-  char tbuf1[BUFFER_LEN];
 
   if ((thing = match_controlled(player, name)) != NOTHING) {
     /* check for bad name */
-    if ((*newname == '\0') || index(newname, '[')) {
-      notify(player, "Give it what new name?");
+    if ((*newname == '\0') || strchr(newname, '[')) {
+      notify(player, T("Give it what new name?"));
       return;
     }
     /* check for renaming a player */
     if (IsPlayer(thing)) {
-      if (Guest(player)) {
-	notify(player, "Guests may not rename themselves.");
-	return;
-      }
       if (PLAYER_NAME_SPACES) {
 	if (*newname == '\"') {
 	  for (; *newname && ((*newname == '\"') || isspace(*newname));
@@ -98,133 +81,147 @@ do_name(player, name, newname)
 	      password++;
 	    if (*password == '\"') {
 	      *password++ = '\0';
-	      while (*password && isspace(*password))
+	      while (*password && isspace((unsigned char) *password))
 		password++;
 	      break;
 	    }
 	  }
 	} else {
 	  password = newname;
-	  while (*password && !isspace(*password))
-	    password++;
-	  *password++ = '\0';
-	  while (*password && isspace(*password))
+	  while (*password && !isspace((unsigned char) *password))
 	    password++;
+	  if (*password) {
+	    *password++ = '\0';
+	    while (*password && isspace((unsigned char) *password))
+	      password++;
+	  }
 	}
       } else {
 
 	/* split off password */
 	for (password = newname + strlen(newname) - 1;
-	     *password && !isspace(*password); password--) ;
-	for (; *password && isspace(*password); password--) ;
+	     *password && !isspace((unsigned char) *password); password--) ;
+	for (; *password && isspace((unsigned char) *password); password--) ;
 	/* eat whitespace */
 	if (*password) {
 	  *++password = '\0';	/* terminate name */
 	  password++;
-	  while (*password && isspace(*password))
+	  while (*password && isspace((unsigned char) *password))
 	    password++;
 	}
       }
       /* check for null password */
       if (!God(player) && !*password) {
-	notify(player, "You must specify a password to change a player name.");
-	notify(player, "E.g.: name player = newname password");
+	notify(player,
+	       T("You must specify a password to change a player name."));
+	notify(player, T("E.g.: @name player = newname password"));
 	return;
       } else if (!God(player) && !password_check(thing, password)) {
-	notify(player, "Incorrect password.");
+	notify(player, T("Incorrect password."));
 	return;
       } else if (strcasecmp(newname, Name(thing))
 		 && !ok_player_name(newname)) {
 	/* strcasecmp allows changing foo to Foo, etc. */
-	notify(player, "You can't give a player that name.");
+	notify(player, T("You can't give a player that name."));
 	return;
       }
       /* everything ok, notify */
-      do_log(LT_CONN, 0, 0, "Name change by %s(#%d) to %s",
+      do_log(LT_CONN, 0, 0, T("Name change by %s(#%d) to %s"),
 	     Name(thing), thing, newname);
-      fflush(connlog_fp);
       if (Suspect(thing))
-	flag_broadcast(WIZARD, 0, "Broadcast: Suspect %s changed name to %s.",
+	flag_broadcast(WIZARD, 0,
+		       T("Broadcast: Suspect %s changed name to %s."),
 		       Name(thing), newname);
-      if (USE_RWHO) {
-	sprintf(tbuf1, "%d@%s", thing, MUDNAME);
-	rwhocli_userlogout(tbuf1);
-      }
       delete_player(thing, NULL);
-      SET(Name(thing), newname);
+      set_name(thing, newname);
       add_player(thing, NULL);
-      if (USE_RWHO)
-	rwhocli_userlogin(tbuf1, newname, time((time_t *) 0));
-      notify(player, "Name set.");
+      notify(player, T("Name set."));
       return;
     } else {
       if (!ok_name(newname)) {
-	notify(player, "That is not a reasonable name.");
+	notify(player, T("That is not a reasonable name."));
 	return;
       }
     }
 
     /* everything ok, change the name */
-    SET(Name(thing), newname);
-    notify(player, "Name set.");
+    set_name(thing, newname);
+    notify(player, T("Name set."));
   }
 }
 
 void
-do_chown(player, name, newobj)
+do_chown(player, name, newobj, preserve)
     dbref player;
     const char *name;
     const char *newobj;
+    int preserve;
 {
   dbref thing;
   dbref newowner = NOTHING;
   char *sp;
-  long match_flags = MAT_POSSESSION | MAT_HERE | MAT_EXIT;
+  long match_flags = MAT_POSSESSION | MAT_HERE | MAT_EXIT | MAT_ABSOLUTE;
 
 
   /* check for '@chown <object>/<atr>=<player>'  */
-  sp = (char *) index(name, '/');
+  sp = strchr(name, '/');
   if (sp) {
     do_atrchown(player, name, newobj);
     return;
   }
-  if (Wizard(player)) {
-    match_flags |= MAT_PLAYER | MAT_ABSOLUTE;
-  } else if (Hasprivs(player)) {
-    match_flags |= MAT_ABSOLUTE;
-  }
-  if ((thing = noisy_match_result(player, name, TYPE_THING, match_flags)) ==
-      NOTHING) return;
+  if (Wizard(player))
+    match_flags |= MAT_PLAYER;
+
+  if ((thing = noisy_match_result(player, name, TYPE_THING, match_flags))
+      == NOTHING)
+    return;
 
   if (!*newobj || !strcasecmp(newobj, "me")) {
     newowner = player;
   } else {
     if ((newowner = lookup_player(newobj)) == NOTHING) {
-      notify(player, "I couldn't find that player.");
+      notify(player, T("I couldn't find that player."));
       return;
     }
   }
 
   if (IsPlayer(thing) && !God(player)) {
-    notify(player, "Players always own themselves.");
+    notify(player, T("Players always own themselves."));
     return;
   }
   /* Permissions checking */
   if (!chown_ok(player, thing, newowner)) {
-    notify(player, "Permission denied.");
+    notify(player, T("Permission denied."));
+    return;
+  }
+  if (IsThing(thing) && !Hasprivs(player) &&
+      !(GoodObject(Location(thing)) && (Location(thing) == player))) {
+    notify(player, T("You must carry the object to @chown it."));
+    return;
+  }
+  if (preserve && !Wizard(player)) {
+    notify(player, T("You cannot @CHOWN/PRESERVE. Use normal @CHOWN."));
     return;
   }
   /* chowns to the zone master don't count towards fees */
   if (!ZMaster(newowner)) {
-    if (!can_pay_fees(player, OBJECT_DEPOSIT(Pennies(thing))))	/* not enough money or quota */
+    /* Debit the owner-to-be */
+    if (!can_pay_fees(newowner, Pennies(thing))) {
+      /* not enough money or quota */
+      if (newowner != player)
+	notify(player,
+	       T
+	       ("That player doesn't have enough money or quota to receive that object."));
       return;
-    giveto(Owner(thing), OBJECT_DEPOSIT(Pennies(thing)));
+    }
+    /* Credit the current owner */
+    giveto(Owner(thing), Pennies(thing));
 #ifdef QUOTA
     change_quota(Owner(thing), QUOTA_COST);
 #endif
   }
-  chown_object(player, thing, newowner);
-  notify(player, "Owner changed.");
+  chown_object(player, thing, newowner, preserve);
+  notify(player, T("Owner changed."));
 }
 
 static int
@@ -233,6 +230,11 @@ chown_ok(player, thing, newowner)
     dbref thing;
     dbref newowner;
 {
+
+  /* Cant' touch garbage */
+  if (IsGarbage(thing))
+    return 0;
+
   /* Wizards can do it all */
   if (Wizard(player))
     return 1;
@@ -278,10 +280,11 @@ chown_ok(player, thing, newowner)
 
 /* Actually change the ownership of something, and fix bits */
 void
-chown_object(player, thing, newowner)
+chown_object(player, thing, newowner, preserve)
     dbref player;
     dbref thing;
     dbref newowner;
+    int preserve;
 {
   undestroy(player, thing);
   if (God(player)) {
@@ -291,92 +294,123 @@ chown_object(player, thing, newowner)
   }
   Zone(thing) = Zone(newowner);
   Flags(thing) &= ~CHOWN_OK;
-  Flags(thing) &= ~WIZARD;
-#ifdef ROYALTY_FLAG
-  Flags(thing) &= ~ROYALTY;
-#endif
-  Flags(thing) &= ~INHERIT;
-  Flags(thing) |= HALT;
-  Powers(thing) = 0;		/* wipe out all powers */
-  do_halt(thing, "", thing);
+  if (!preserve || !Wizard(player)) {
+    Flags(thing) &= ~WIZARD;
+    Flags(thing) &= ~ROYALTY;
+    Flags(thing) &= ~INHERIT;
+    Flags(thing) |= HALT;
+    Powers(thing) = 0;		/* wipe out all powers */
+    do_halt(thing, "", thing);
+  } else {
+    if ((newowner != player) && Wizard(thing) && !God(player)) {
+      notify(player,
+	     T
+	     ("Warning: WIZ flag reset because @CHOWN/PRESERVE is to a third party."));
+      Flags(thing) &= ~WIZARD;
+    }
+    if (Powers(thing) || Wizard(thing) || Royalty(thing) || Inherit(thing))
+      notify(player,
+	     T
+	     ("Warning: @CHOWN/PRESERVE on a target with WIZ, ROY, INHERIT, or @power privileges."));
+  }
 }
 
 
-void
-do_chzone(player, name, newobj)
-    dbref player;
-    const char *name;
-    const char *newobj;
+int
+do_chzone(dbref player, char const *name, char const *newobj, int noisy)
 {
   dbref thing;
   dbref zone;
 
   if ((thing = noisy_match_result(player, name, NOTYPE, MAT_NEARBY)) == NOTHING)
-    return;
+    return 0;
 
-  if (!strcasecmp(newobj, "none"))
+  if (!newobj || !*newobj || !strcasecmp(newobj, "none"))
     zone = NOTHING;
   else {
-    if ((zone = noisy_match_result(player, newobj, NOTYPE, MAT_EVERYTHING)) ==
-	NOTHING) return;
+    if ((zone = noisy_match_result(player, newobj, NOTYPE, MAT_EVERYTHING))
+	== NOTHING)
+      return 0;
+  }
 
-    if (!IsThing(zone)
-	&& (!DO_GLOBALS || !IsRoom(zone))) {
-      notify(player, "Invalid zone object type.");
-      return;
-    }
+  if (Zone(thing) == zone) {
+    if (noisy)
+      notify(player, T("That object is already in that zone."));
+    return 0;
   }
 
   /* we do use ownership instead of control as a criterion because
    * we only want the owner to be able to rezone the object. Also,
    * this allows players to @chzone themselves to an object they own.
    */
-  if (!Wizard(player) && !Owns(player, thing)) {
-    notify(player, "You don't have the power to shift reality.");
-    return;
+  if (!(God(player) || (!God(thing) && Wizard(player)) || Owns(player, thing))) {
+    if (noisy)
+      notify(player, T("You don't have the power to shift reality."));
+    return 0;
   }
-  /* a player may change an object's zone to NOTHING or to an object he owns */
-  if ((zone != NOTHING) && !Wizard(player) && !Owns(player, zone)) {
-    notify(player, "You cannot move that object to that zone.");
-    return;
-  }
-  /* only rooms may be zoned to other rooms */
-  if ((zone != NOTHING) && IsRoom(zone) && !IsRoom(thing)) {
-    notify(player, "Only rooms may have zone master rooms.");
-    return;
+  /* a player may change an object's zone to:
+   * 1.  NOTHING 
+   * 2.  an object he owns
+   * 3.  an object with a chzone-lock that the player passes.
+   * Note that an object with no chzone-lock isn't valid
+   */
+  if (!(Wizard(player) || (zone == NOTHING) || Owns(player, zone) ||
+	((getlock(zone, Chzone_Lock) != TRUE_BOOLEXP) &&
+	 eval_lock(player, zone, Chzone_Lock)))) {
+    if (noisy)
+      notify(player, T("You cannot move that object to that zone."));
+    return 0;
   }
   /* Don't chzone object to itself for mortals! */
   if ((zone == thing) && !Hasprivs(player)) {
-    notify(player, "You shouldn't zone objects to themselves!");
-    return;
+    if (noisy)
+      notify(player, T("You shouldn't zone objects to themselves!"));
+    return 0;
   }
+  /* Don't allow circular zones */
+  if (GoodObject(zone)) {
+    dbref tmp;
+    for (tmp = Zone(zone); GoodObject(tmp); tmp = Zone(tmp)) {
+      if (tmp == thing) {
+	notify(player, T("You can't make circular zones!"));
+	return 0;
+      }
+      if (tmp == Zone(tmp))	/* Ran into an object zoned to itself */
+	break;
+    }
+  }
+
+
   /* Don't allow chzone to objects without elocks! 
+   * If no lock is set, set a default lock (warn if zmo are used for control)
    * This checks for many trivial elocks (canuse/1, where &canuse=1)
    */
   if (zone != NOTHING) {
     struct boolexp *key = getlock(zone, Zone_Lock);
     if (key == TRUE_BOOLEXP) {
-      notify(player, "ZMOs must be zone-locked before you @chzone!");
-      return;
-    }
-    /* Does the player's location pass it? If so, we have either
-     * an inexact or trivial elock */
-    if (eval_lock(Location(player), zone, Zone_Lock)) {
+      add_lock(GOD, zone, Zone_Lock, parse_boolexp(zone, "$me", Zone_Lock), -1);
+      if (noisy && !ZONE_CONTROL_ZMP)
+	notify(player,
+	       T("Unlocked ZMO - automatically zone-locking ZMO to its owner"));
+    } else if (eval_lock(Location(player), zone, Zone_Lock)) {
       /* Does #0 and #2 pass it? If so, probably trivial elock */
       if (eval_lock(PLAYER_START, zone, Zone_Lock) &&
 	  eval_lock(MASTER_ROOM, zone, Zone_Lock)) {
-	notify(player,
-	       "ZMO requires a more secure zone-lock before you @chzone!");
-	return;
+	if (noisy)
+	  notify(player,
+		 T("ZMO requires a more secure zone-lock before you @chzone!"));
+	return 0;
       }
       /* Probably inexact zone lock */
       notify(player,
-	     "Warning: ZMO may have loose zone lock. Lock ZMOs to =player, not player");
+	     T
+	     ("Warning: ZMO may have loose zone lock. Lock ZMOs to =player, not player"));
     }
   }
   /* Warn Wiz/Royals when they zone their stuff */
   if ((zone != NOTHING) && Hasprivs(Owner(thing))) {
-    notify(player, "Warning: @chzoning admin-owned object!");
+    if (noisy)
+      notify(player, T("Warning: @chzoning admin-owned object!"));
   }
   /* everything is okay, do the change */
   Zone(thing) = zone;
@@ -389,21 +423,22 @@ do_chzone(player, name, newobj)
      * risk. Be careful when @chzone'ing wizard or royal players.
      */
     Flags(thing) &= ~WIZARD;
-#ifdef ROYALTY_FLAG
     Flags(thing) &= ~ROYALTY;
-#endif
     Flags(thing) &= ~INHERIT;
     Powers(thing) = 0;		/* wipe out all powers */
   } else {
     if (zone != NOTHING) {
-      if Hasprivs
-	(thing)
-	  notify(player, "Warning: @chzoning a privileged player.");
+      if (Hasprivs(thing))
+	if (noisy)
+	  notify(player, T("Warning: @chzoning a privileged player."));
       if (Flags(thing) & INHERIT)
-	notify(player, "Warning: @chzoning an INHERIT player.");
+	if (noisy)
+	  notify(player, T("Warning: @chzoning an TRUST player."));
     }
   }
-  notify(player, "Zone changed.");
+  if (noisy)
+    notify(player, T("Zone changed."));
+  return 1;
 }
 
 struct af_args {
@@ -415,7 +450,7 @@ static int
 af_helper(player, thing, pattern, atr, args)
     dbref player;
     dbref thing;
-    char const *pattern;
+    char const *pattern __attribute__ ((__unused__));
     ATTR *atr;
     void *args;
 {
@@ -423,18 +458,27 @@ af_helper(player, thing, pattern, atr, args)
 
   /* We must be able to write to that attribute normally,
    * to prevent players from doing funky things to, say, LAST.
+   * There is one special case - the resetting of the SAFE flag.
    */
-  if (!Can_Write_Attr(player, thing, AL_ATTR(atr))) {
-    notify(player, tprintf("You cannot set %s/%s", Name(thing), AL_NAME(atr)));
+  if (!(Can_Write_Attr(player, thing, AL_ATTR(atr)) ||
+	((*af->flag == NOT_TOKEN) && (af->f == AF_SAFE) &&
+	 Can_Write_Attr_Ignore_Safe(player, thing, AL_ATTR(atr))))) {
+    notify_format(player, T("You cannot change that flag on %s/%s"),
+		  Name(thing), AL_NAME(atr));
     return 0;
   }
-  if (*af->flag == NOT_TOKEN)
-    AL_FLAGS(atr) &= ~af->f;
-  else
-    AL_FLAGS(atr) |= af->f;
 
-  if (!Quiet(player) && !Quiet(thing))
-    notify(player, tprintf("%s/%s - Set.", Name(thing), AL_NAME(atr)));
+  if (*af->flag == NOT_TOKEN) {
+    AL_FLAGS(atr) &= ~af->f;
+    if (!Quiet(player) && !(Quiet(thing) && (Owner(thing) == player)))
+      notify_format(player, T("%s/%s - %s reset."), Name(thing), AL_NAME(atr),
+		    (af->flag + 1));
+  } else {
+    AL_FLAGS(atr) |= af->f;
+    if (!Quiet(player) && !(Quiet(thing) && (Owner(thing) == player)))
+      notify_format(player, T("%s/%s - %s set."), Name(thing), AL_NAME(atr),
+		    af->flag);
+  }
 
   return 1;
 }
@@ -449,9 +493,9 @@ copy_attrib_flags(player, target, atr, flags)
   if (!atr)
     return;
   if (!Can_Write_Attr(player, target, AL_ATTR(atr))) {
-    notify(player,
-	   tprintf("You cannot set attrib flags on %s/%s", Name(target),
-		   AL_NAME(atr)));
+    notify_format(player,
+		  T("You cannot set attrib flags on %s/%s"), Name(target),
+		  AL_NAME(atr));
     return;
   }
   AL_FLAGS(atr) = flags;
@@ -472,18 +516,20 @@ do_attrib_flags(player, obj, atrname, flag)
     return;
 
   if (!flag || !*flag) {
-    notify(player, "What flag do you want to set?");
+    notify(player, T("What flag do you want to set?"));
     return;
   }
   /* move past NOT token if there is one */
-  for (p = flag; *p && ((*p == NOT_TOKEN) || isspace(*p)); p++) ;
+  for (p = flag; *p && ((*p == NOT_TOKEN) || isspace((unsigned char) *p));
+       p++) ;
 
   if ((af.f = string_to_atrflag(player, p)) < 0) {
-    notify(player, "Unrecognized attribute flag.");
+    notify(player, T("Unrecognized attribute flag."));
     return;
   }
   af.flag = flag;
-  (void) atr_iter_get(player, thing, atrname, af_helper, &af);
+  if (!atr_iter_get(player, thing, atrname, af_helper, &af))
+    notify(player, T("No attribute found to change."));
 }
 
 
@@ -498,7 +544,7 @@ do_set(player, name, flag)
   char *p;
 
   /* check for attribute flag set first */
-  if ((p = (char *) index(name, '/')) != NULL) {
+  if ((p = strchr(name, '/')) != NULL) {
     *p++ = '\0';
     do_attrib_flags(player, name, p, flag);
     return 1;
@@ -507,16 +553,16 @@ do_set(player, name, flag)
   if ((thing = match_controlled(player, name)) == NOTHING)
     return 0;
   if (God(thing) && !God(player)) {
-    notify(player, "Only God can set himself!");
+    notify(player, T("Only God can set himself!"));
     return 0;
   }
   her = Hearer(thing);
   listener = Listener(thing);
   /* check for attribute set first */
-  if ((p = (char *) index(flag, ':')) != NULL) {
+  if ((p = strchr(flag, ':')) != NULL) {
     *p++ = '\0';
     if (!command_check_byname(player, "ATTRIB_SET")) {
-      notify(player, "You may not set attributes.");
+      notify(player, T("You may not set attributes."));
       return 0;
     }
     return do_set_atr(thing, flag, p, player, 1);
@@ -524,11 +570,12 @@ do_set(player, name, flag)
   /* we haven't set an attribute, so we must be setting a flag */
 
   /* move p past NOT_TOKEN if present */
-  for (p = (char *) flag; *p && (*p == NOT_TOKEN || isspace(*p)); p++) ;
+  for (p = (char *) flag;
+       *p && (*p == NOT_TOKEN || isspace((unsigned char) *p)); p++) ;
 
   /* identify flag */
   if (*p == '\0') {
-    notify(player, "You must specify a flag to set.");
+    notify(player, T("You must specify a flag to set."));
     return 0;
   }
   set_flag(player, thing, p, (*flag == NOT_TOKEN) ? 1 : 0, her, listener);
@@ -536,11 +583,12 @@ do_set(player, name, flag)
 }
 
 void
-do_cpattr(player, oldpair, newpair, move)
+do_cpattr(player, oldpair, newpair, move, noflagcopy)
     dbref player;
     char *oldpair;
     char **newpair;
     int move;
+    int noflagcopy;
 {
   /* the command is of the format:
    * @cpattr oldobj/oldattr = newobj1/newattr1, newobj2/newattr2, etc.
@@ -557,14 +605,14 @@ do_cpattr(player, oldpair, newpair, move)
 
   /* must copy from something */
   if (!oldpair || !*oldpair) {
-    notify(player, "What do you want to copy from?");
+    notify(player, T("What do you want to copy from?"));
     return;
   }
   /* find the old object */
   strcpy(tbuf1, oldpair);
-  p = (char *) index(tbuf1, '/');
+  p = strchr(tbuf1, '/');
   if (!p || !*p) {
-    notify(player, "What object do you want to copy the attribute from?");
+    notify(player, T("What object do you want to copy the attribute from?"));
     return;
   }
   *p++ = '\0';
@@ -577,12 +625,12 @@ do_cpattr(player, oldpair, newpair, move)
   /* find the old attribute */
   a = atr_get_noparent(oldobj, strupper(p));
   if (!a) {
-    notify(player, "No such attribute to copy from.");
+    notify(player, T("No such attribute to copy from."));
     return;
   }
   /* check permissions to get it */
   if (!Can_Read_Attr(player, oldobj, a)) {
-    notify(player, "Permission to read attribute denied.");
+    notify(player, T("Permission to read attribute denied."));
     return;
   }
   /* we can read it. Copy the value. */
@@ -591,47 +639,179 @@ do_cpattr(player, oldpair, newpair, move)
   /* now we loop through our new object pairs and copy, calling @set. */
   for (i = 1; i < MAX_ARG && (newpair[i] != NULL); i++) {
     if (!*newpair[i]) {
-      notify(player, "What do you want to copy to?");
+      notify(player, T("What do you want to copy to?"));
     } else {
       strcpy(tbuf1, newpair[i]);
-      q = (char *) index(tbuf1, '/');
+      q = strchr(tbuf1, '/');
       if (!q || !*q) {
 	q = (char *) AL_NAME(a);
       } else {
 	*q++ = '\0';
       }
       newobj = noisy_match_result(player, tbuf1, NOTYPE, MAT_EVERYTHING);
-      if (GoodObject(newobj) && do_set_atr(newobj, q, text, player, 1)) {
+      if (GoodObject(newobj) &&
+	  ((newobj != oldobj) || strcasecmp(AL_NAME(a), q)) &&
+	  do_set_atr(newobj, q, text, player, 1))
 	copies++;
-	/* copy the attribute flags too */
+      /* copy the attribute flags too */
+      if (!noflagcopy)
 	copy_attrib_flags(player, newobj,
 			  atr_get_noparent(newobj, strupper(q)), a->flags);
-      }
+
     }
   }
 
   free((Malloc_t) text);	/* safe_uncompress malloc()s memory */
   if (copies) {
-    notify(player, tprintf("Attribute %s (%d copies)",
-			   (move ? "moved" : "copied"), copies));
+    notify_format(player, T("Attribute %s (%d copies)"),
+		  (move ? "moved" : "copied"), copies);
     if (move)
-      do_set_atr(oldobj, p, NULL, player, 1);
+      do_set_atr(oldobj, AL_NAME(a), NULL, player, 1);
   } else {
-    notify(player, tprintf("Unable to %s attribute.",
-			   (move ? "move" : "copy")));
+    notify_format(player, T("Unable to %s attribute."),
+		  (move ? "move" : "copy"));
   }
   return;
 }
 
 static int
-gedit_helper(player, thing, pattern, atr, args)
+gedit_helper(player, thing, pattern, a, args)
     dbref player;
     dbref thing;
-    char const *pattern;
-    ATTR *atr;
+    char const *pattern __attribute__ ((__unused__));
+    ATTR *a;
     void *args;
 {
-  do_edit(player, thing, (char *) AL_NAME(atr), (char **) args);
+  int ansi_long_flag = 0;
+  const char *r;
+  char *s, *val;
+  char tbuf1[BUFFER_LEN], tbuf_ansi[BUFFER_LEN];
+  char *tbufp, *tbufap;
+  char **argv = (char **) args;
+  size_t rlen, vlen;
+
+  val = argv[1];
+  vlen = strlen(val);
+  r = argv[2] ? argv[2] : "";
+  rlen = strlen(r);
+
+  tbufp = tbuf1;
+  tbufap = tbuf_ansi;
+
+  if (!a) {			/* Shouldn't ever happen, but better safe than sorry */
+    notify(player, T("No such attribute, try set instead."));
+    return 0;
+  }
+  if (!Can_Write_Attr(player, thing, a)) {
+    notify(player, T("You need to control an attribute to edit it."));
+    return 0;
+  }
+  s = (char *) uncompress(a->value);	/* warning: pointer to static buffer */
+
+  if (vlen == 1 && *val == '$') {
+    /* append */
+    safe_str(s, tbuf1, &tbufp);
+    safe_str(r, tbuf1, &tbufp);
+
+    if (safe_format(tbuf_ansi, &tbufap, "%s%s%s%s", s, ANSI_HILITE, r,
+		    ANSI_NORMAL))
+      ansi_long_flag = 1;
+  } else if (vlen == 1 && *val == '^') {
+    /* prepend */
+    safe_str(r, tbuf1, &tbufp);
+    safe_str(s, tbuf1, &tbufp);
+
+    if (safe_format(tbuf_ansi, &tbufap, "%s%s%s%s", ANSI_HILITE, r, ANSI_NORMAL,
+		    s))
+      ansi_long_flag = 1;
+  } else if (!*val) {
+    /* insert replacement string between every character */
+    ansi_string *haystack;
+    size_t last = 0;
+    int too_long = 0;
+
+    haystack = parse_ansi_string(s);
+
+    /* Add one at the start */
+    if (!safe_strl(r, rlen, tbuf1, &tbufp)) {
+      for (last = 0; last < haystack->len; last++) {
+	/* Add the next character */
+	if (safe_ansi_string(haystack, last, 1, tbuf1, &tbufp)) {
+	  too_long = 1;
+	  break;
+	}
+	if (!ansi_long_flag) {
+	  if (safe_ansi_string(haystack, last, 1, tbuf_ansi, &tbufap))
+	    ansi_long_flag = 1;
+	}
+	/* Copy in r */
+	if (safe_strl(r, rlen, tbuf1, &tbufp)) {
+	  too_long = 1;
+	  break;
+	}
+	if (!ansi_long_flag) {
+	  if (safe_format(tbuf_ansi, &tbufap, "%s%s%s", ANSI_HILITE, r,
+			  ANSI_NORMAL))
+	    ansi_long_flag = 1;
+	}
+      }
+    }
+    free_ansi_string(haystack);
+  } else {
+    /* find and replace */
+    ansi_string *haystack;
+    size_t last = 0;
+    char *p;
+    int too_long = 0;
+
+    haystack = parse_ansi_string(s);
+
+    while (last < haystack->len
+	   && (p = strstr(haystack->text + last, val)) != NULL) {
+      if (safe_ansi_string(haystack, last, p - (haystack->text + last),
+			   tbuf1, &tbufp)) {
+	too_long = 1;
+	break;
+      }
+      if (!ansi_long_flag) {
+	if (safe_ansi_string(haystack, last, p - (haystack->text + last),
+			     tbuf_ansi, &tbufap))
+	  ansi_long_flag = 1;
+      }
+
+      /* Copy in r */
+      if (safe_strl(r, rlen, tbuf1, &tbufp)) {
+	too_long = 1;
+	break;
+      }
+      if (!ansi_long_flag) {
+	if (safe_format(tbuf_ansi, &tbufap, "%s%s%s", ANSI_HILITE, r,
+			ANSI_NORMAL))
+	  ansi_long_flag = 1;
+      }
+      last = p - haystack->text + vlen;
+    }
+    if (last < haystack->len && !too_long) {
+      safe_ansi_string(haystack, last, haystack->len, tbuf1, &tbufp);
+      if (!ansi_long_flag) {
+	if (safe_ansi_string(haystack, last, haystack->len, tbuf_ansi, &tbufap))
+	  ansi_long_flag = 1;
+      }
+    }
+    free_ansi_string(haystack);
+  }
+
+  *tbufp = '\0';
+  *tbufap = '\0';
+
+  if (do_set_atr(thing, AL_NAME(a), tbuf1, player, 0) &&
+      !Quiet(player) && !Quiet(thing)) {
+    if (!ansi_long_flag && ShowAnsi(player))
+      notify_format(player, "%s - Set: %s", AL_NAME(a), tbuf_ansi);
+    else
+      notify_format(player, "%s - Set: %s", AL_NAME(a), tbuf1);
+  }
+
   return 1;
 }
 
@@ -646,121 +826,31 @@ do_gedit(player, it, argv)
   char *q;
 
   if (!(it && *it)) {
-    notify(player, "I need to know what you want to edit.");
+    notify(player, T("I need to know what you want to edit."));
     return;
   }
   strcpy(tbuf1, it);
-  q = (char *) index(tbuf1, '/');
+  q = strchr(tbuf1, '/');
   if (!(q && *q)) {
-    notify(player, "I need to know what you want to edit.");
+    notify(player, T("I need to know what you want to edit."));
     return;
   }
   *q++ = '\0';
   thing = noisy_match_result(player, tbuf1, NOTYPE, MAT_EVERYTHING);
 
   if ((thing == NOTHING) || !controls(player, thing)) {
-    notify(player, "Permission denied.");
+    notify(player, T("Permission denied."));
     return;
   }
+
   if (!argv[1] || !*argv[1]) {
-    notify(player, "Nothing to do.");
+    notify(player, T("Nothing to do."));
     return;
   }
-  (void) atr_iter_get(player, thing, q, gedit_helper, argv);
+  if (!atr_iter_get(player, thing, q, gedit_helper, argv))
+    notify(player, T("No matching attributes."));
 }
 
-void
-do_edit(player, thing, q, argv)
-    dbref player;
-    dbref thing;
-    char *q;
-    char **argv;		/* attribute name */
-{
-  int d, len, ansi_d, ansi_long_flag = 0;
-  ATTR *a;
-  char *r, *s, *val;
-  char tbuf1[BUFFER_LEN], tbuf_ansi[BUFFER_LEN];
-
-  if (God(thing) && !God(player)) {
-    notify(player, "Only God can edit himself!");
-    return;
-  }
-  val = argv[1];
-  r = (argv[2]) ? argv[2] : (char *) "";
-
-  a = atr_get_noparent(thing, q);
-  if (!a) {
-    notify(player, "No such attribute, try set instead.");
-    return;
-  }
-  if ((a->flags & AF_LOCKED) && (Owner(player) != Owner(a->creator))) {
-    notify(player, "You need to control an attribute to edit it.");
-    return;
-  }
-  s = (char *) uncompress(a->value);	/* warning: pointer to static buffer */
-
-  if (!strcmp(val, "$")) {
-    /* append */
-    strcpy(tbuf1, s);
-    if (strlen(s) + strlen(r) < BUFFER_LEN)
-      strcat(tbuf1, r);
-    if (strlen(tbuf1) + strlen(ANSI_HILITE) + strlen(ANSI_NORMAL) < BUFFER_LEN) {
-      sprintf(tbuf_ansi, "%s%s%s%s", s, ANSI_HILITE, r, ANSI_NORMAL);
-    } else {
-      strcpy(tbuf_ansi, tbuf1);
-    }
-
-  } else if (!strcmp(val, "^")) {
-    /* prepend */
-    strcpy(tbuf1, r);
-    strncat(tbuf1, s, BUFFER_LEN - strlen(r) - 1);
-    if (strlen(tbuf1) + strlen(ANSI_HILITE) + strlen(ANSI_NORMAL) < BUFFER_LEN) {
-      sprintf(tbuf_ansi, "%s%s%s%s", ANSI_HILITE, r, ANSI_NORMAL, s);
-    } else {
-      strcpy(tbuf_ansi, tbuf1);
-    }
-  } else {
-    /* find and replace */
-    len = strlen(val);
-    for (d = 0, ansi_d = 0; (d < BUFFER_LEN) && *s;)
-      if (strncmp(val, s, len) == 0) {
-	if ((d + strlen(r)) < BUFFER_LEN) {
-	  strcpy(tbuf1 + d, r);
-	  d += strlen(r);
-	  if ((ansi_d + strlen(r) + strlen(ANSI_HILITE) + strlen(ANSI_NORMAL)) <
-	      BUFFER_LEN) {
-	    sprintf(tbuf_ansi + ansi_d, "%s%s%s", ANSI_HILITE, r, ANSI_NORMAL);
-	    ansi_d += strlen(r) + strlen(ANSI_HILITE) + strlen(ANSI_NORMAL);
-	  } else {
-	    /* Hmm. What here? We can't assume that r will still fit. */
-	    /* For now, let's just give up. After all, how often is this
-	     * going to arise, anyway? */
-	    if (!ansi_long_flag) {
-	      ansi_long_flag = 1;
-	      strcpy(tbuf_ansi,
-		     "Sorry, that attribute became too long to display. Examine it to see its new value.");
-	      /* Not informative, but at least not a lie. */
-	    }
-	  }
-	  s += len;
-	} else {
-	  break;
-	}
-      } else {
-	tbuf1[d++] = tbuf_ansi[ansi_d++] = *s++;
-      }
-    tbuf1[d++] = 0;
-    tbuf_ansi[ansi_d++] = '\0';
-  }
-
-  if (do_set_atr(thing, a->name, tbuf1, player, 0) &&
-      !Quiet(player) && !Quiet(thing)) {
-    if (!ansi_long_flag && ShowAnsi(player))
-      notify(player, tprintf("%s - Set: %s", a->name, tbuf_ansi));
-    else
-      notify(player, tprintf("%s - Set: %s", a->name, tbuf1));
-  }
-}
 
 void
 do_trigger(player, object, argv)
@@ -776,7 +866,7 @@ do_trigger(player, object, argv)
   strcpy(tbuf1, object);
   for (s = tbuf1; *s && (*s != '/'); s++) ;
   if (!*s) {
-    notify(player, "I need to know what attribute to trigger.");
+    notify(player, T("I need to know what attribute to trigger."));
     return;
   }
   *s++ = '\0';
@@ -787,20 +877,23 @@ do_trigger(player, object, argv)
     return;
 
   if (!controls(player, thing) && !(Owns(player, thing) && LinkOk(thing))) {
-    notify(player, "Permission denied.");
+    notify(player, T("Permission denied."));
     return;
   }
   if (God(thing) && !God(player)) {
-    notify(player, "You can't trigger God!");
+    notify(player, T("You can't trigger God!"));
     return;
   }
   /* trigger modifies the stack */
   for (a = 0; a < 10; a++)
     wnxt[a] = argv[a + 1];
 
-  charge_action(player, thing, upcasestr(s));
-  if (!Quiet(player) && !Quiet(thing))
-    notify(player, tprintf("%s - Triggered.", Name(thing)));
+  if (charge_action(player, thing, upcasestr(s))) {
+    if (!Quiet(player) && !Quiet(thing))
+      notify_format(player, "%s - Triggered.", Name(thing));
+  } else {
+    notify(player, T("No such attribute."));
+  }
 }
 
 
@@ -815,10 +908,11 @@ do_use(player, what)
 
   /* if we pass the use key, do it */
 
-  if ((thing = noisy_match_result(player, what, TYPE_THING, MAT_NEAR_THINGS)) !=
-      NOTHING) {
+  if ((thing =
+       noisy_match_result(player, what, TYPE_THING,
+			  MAT_NEAR_THINGS)) != NOTHING) {
     if (!eval_lock(player, thing, Use_Lock)) {
-      did_it(player, thing, "UFAIL", "Pemission denied.", "OUFAIL", NULL,
+      did_it(player, thing, "UFAIL", T("Pemission denied."), "OUFAIL", NULL,
 	     "AUFAIL", NOTHING);
       return;
     } else
@@ -840,25 +934,15 @@ do_parent(player, name, parent_name)
   if ((thing = noisy_match_result(player, name, NOTYPE, MAT_NEARBY)) == NOTHING)
     return;
 
-#ifdef NEVER
-  /* in the old days, players could not be parented */
-  if (IsRoom(thing)) {
-    notify(player, "Don't you like your biological parents?");
-    return;
-  }
-#endif
   if (!parent_name || !*parent_name || !strcasecmp(parent_name, "none"))
     parent = NOTHING;
-  else {
-    if (
-	(parent =
-	 noisy_match_result(player, parent_name, NOTYPE,
-			    MAT_EVERYTHING)) == NOTHING) return;
-  }
+  else if ((parent = noisy_match_result(player, parent_name, NOTYPE,
+					MAT_EVERYTHING)) == NOTHING)
+    return;
 
   /* do control check */
   if (!controls(player, thing) && !(Owns(player, thing) && LinkOk(thing))) {
-    notify(player, "Permission denied.");
+    notify(player, T("Permission denied."));
     return;
   }
   /* a player may change an object's parent to NOTHING or to an 
@@ -869,35 +953,32 @@ do_parent(player, name, parent_name)
    * control check (wich does those things
    * anyway, right?)]
    */
-  if ((parent != NOTHING) && !controls(player, parent) && !(LinkOk(parent)
-							    && eval_lock(player,
-									 parent,
-									 Parent_Lock)))
-  {
-    notify(player, "Permission denied.");
+  if ((parent != NOTHING) && !controls(player, parent) &&
+      !(LinkOk(parent) && eval_lock(player, parent, Parent_Lock))) {
+    notify(player, T("Permission denied."));
     return;
   }
   /* check to make sure no recursion can happen */
   if (parent == thing) {
-    notify(player, "A thing cannot be its own ancestor!");
+    notify(player, T("A thing cannot be its own ancestor!"));
     return;
   }
   if (parent != NOTHING) {
     for (i = 0, check = Parent(parent);
 	 (i < MAX_PARENTS) && (check != NOTHING); i++, check = Parent(check)) {
       if (check == thing) {
-	notify(player, "You are not allowed to be your own ancestor!");
+	notify(player, T("You are not allowed to be your own ancestor!"));
 	return;
       }
     }
     if (i >= MAX_PARENTS) {
-      notify(player, "Too many ancestors.");
+      notify(player, T("Too many ancestors."));
       return;
     }
   }
   /* everything is okay, do the change */
   Parent(thing) = parent;
-  notify(player, "Parent changed.");
+  notify(player, T("Parent changed."));
 }
 
 static int
@@ -906,7 +987,7 @@ wipe_helper(player, thing, pattern, atr, args)
     dbref thing;
     char const *pattern;
     ATTR *atr;
-    void *args;
+    void *args __attribute__ ((__unused__));
 {
   /* for added security, only God can modify wiz-only-modifiable
    * attributes using this command and wildcards.  Wiping a specific
@@ -927,7 +1008,7 @@ do_wipe(player, name)
   dbref thing;
   char *pattern;
 
-  if ((pattern = (char *) index(name, '/')) != NULL)
+  if ((pattern = strchr(name, '/')) != NULL)
     *pattern++ = '\0';
 
   if ((thing = noisy_match_result(player, name, NOTYPE, MAT_NEARBY)) == NOTHING)
@@ -937,15 +1018,27 @@ do_wipe(player, name)
    * doesn't own the object. Thus, the check is on Owns not controls.
    */
   if (!Wizard(player) && !Owns(player, thing)) {
-    notify(player, "Permission denied.");
+    notify(player, T("Permission denied."));
     return;
   }
   /* protect SAFE objects unless doing a non-wildcard pattern */
   if (Safe(thing) && !(pattern && *pattern && !wildcard(pattern))) {
-    notify(player, "That object is protected.");
+    notify(player, T("That object is protected."));
     return;
   }
-  (void) atr_iter_get(player, thing, pattern, wipe_helper, NULL);
+  if (!atr_iter_get(player, thing, pattern, wipe_helper, NULL))
+    notify(player, T("No matching attributes."));
+  else
+    notify(player, T("Attributes wiped."));
+}
 
-  notify(player, "Attributes wiped.");
+COMMAND (cmd_with) {
+  dbref what;
+
+  what = noisy_match_result(player, arg_left, NOTYPE, MAT_NEARBY);
+  if (!GoodObject(what))
+    return;
+
+  if (!atr_comm_match(what, player, '$', ':', arg_right, 0, NULL, NULL))
+    notify(player, T("No matching command."));
 }
