@@ -22,7 +22,7 @@
 #include "confmagic.h"
 
 #ifndef HAS_STRDUP
-char *strdup _((const char *s));
+char *strdup(const char *s);
 
 char *
 strdup(s)
