@@ -12,16 +12,17 @@
 #endif
 #include <time.h>
 #include "conf.h"
-#include "intrface.h"
+
 #include "externs.h"
 #include "version.h"
+#include "patches.h"
 #ifndef WIN32
 #include "buildinf.h"
 #endif
 #include "confmagic.h"
 
 extern time_t start_time;	/* from bsd.c */
-void do_version _((dbref player));
+void do_version(dbref player);
 
 void
 do_version(player)
@@ -29,19 +30,22 @@ do_version(player)
 {
   char buff[BUFFER_LEN];
 
-  notify(player, tprintf("You are connected to %s", MUDNAME));
+  notify_format(player, T("You are connected to %s"), MUDNAME);
 
   strcpy(buff, ctime(&start_time));
   buff[strlen(buff) - 1] = '\0';	/* eat the newline */
-  notify(player, tprintf("Last restarted: %s", buff));
+  notify_format(player, T("Last restarted: %s"), buff);
 
-  notify(player, tprintf("%s", VERSION));
+  notify_format(player, "%s", VERSION);
+#ifdef PATCHES
+  notify_format(player, "Patches: %s", PATCHES);
+#endif
 #ifdef WIN32
-  notify(player, tprintf("Build date: %s", __DATE__));
+  notify_format(player, T("Build date: %s"), __DATE__);
 #else
-  notify(player, tprintf("Build date: %s", BUILDDATE));
-  notify(player, tprintf("Compiler: %s", COMPILER));
-  notify(player, tprintf("Compilation flags: %s", CCFLAGS));
-  notify(player, tprintf("Malloc package: %d", MALLOC_PACKAGE));
+  notify_format(player, T("Build date: %s"), BUILDDATE);
+  notify_format(player, T("Compiler: %s"), COMPILER);
+  notify_format(player, T("Compilation flags: %s"), CCFLAGS);
+  notify_format(player, T("Malloc package: %d"), MALLOC_PACKAGE);
 #endif
 }
